@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2005-2006 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2005-2008 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -35,8 +35,8 @@
 */
 
 
-#ifndef _uim_agent_debug_h_included_
-#define _uim_agent_debug_h_included_
+#ifndef UIM_AGENT_DEBUG_H
+#define UIM_AGENT_DEBUG_H
 
 #include <stdio.h>
 #include <stdarg.h>
