@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2005-2006 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2005-2008 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -34,8 +34,8 @@
   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
-#ifndef _uim_agent_im_h_included_
-#define _uim_agent_im_h_included_
+#ifndef UIM_AGENT_IM_H
+#define UIM_AGENT_IM_H
 
 #include <string.h>
 #include <locale.h>
