@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -31,8 +31,8 @@
 
 */
 
-#ifndef CALLBACKS_H
-#define CALLBACKS_H
+#ifndef UIM_FEP_CALLBACKS_H
+#define UIM_FEP_CALLBACKS_H
 
 struct preedit_tag {
   /* ʸ */
