@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -141,10 +141,10 @@ void update_backtick(void)
   char sendbuf[CANDSIZE];
   if (s_candbuf[0] == '\0') {
     /* ⡼ɽ */
-    sprintf(sendbuf, "%s%s", s_win_no, s_modebuf);
+    snprintf(sendbuf, sizeof(sendbuf), "%s%s", s_win_no, s_modebuf);
   } else {
     /* ɽ */
-    sprintf(sendbuf, "%s", s_candbuf);
+    strlcpy(sendbuf,  s_candbuf, sizeof(sendbuf));
   }
   sendline(sendbuf);
 }
