@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2006 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -31,8 +31,8 @@
 
 */
 
-#ifndef DRAW_H
-#define DRAW_H
+#ifndef UIM_FEP_DRAW_H
+#define UIM_FEP_DRAW_H
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
