@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2006 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -31,8 +31,8 @@
 
 */
 
-#ifndef HELPER_H
-#define HELPER_H
+#ifndef UIM_FEP_HELPER_H
+#define UIM_FEP_HELPER_H
 
 #define str_has_prefix(str, prefix) (strncmp((str), (prefix), strlen((prefix))) == 0)
 
