@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -31,8 +31,10 @@
 
 */
 
-#ifndef UDSOCK_H
-#define UDSOCK_H
+#ifndef UIM_FEP_UDSOCK_H
+#define UIM_FEP_UDSOCK_H
+
+#include "uim.h"
 
 #ifndef UNIX_PATH_MAX
 #define UNIX_PATH_MAX 100
@@ -44,5 +46,6 @@ void sendline(const char *buf);
 void init_recvsocket(const char *sock_path);
 int recvline(char *buf, int n);
 void close_socket(void);
+uim_bool get_ud_path(char *, int);
 
 #endif
