@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -183,12 +183,13 @@ putbackch(int c, int *lastch)
 #endif
 
 static int
-nexttoken(FILE *fp, char *tokenbuf, int *lastch)
+nexttoken(FILE *fp, char **tokenbuf, int *lastch, size_t *buflen)
 {
     int c;
     int token;
     char *p;
     int i, j;
+    size_t len = 0;
 
     while ((c = nextch(fp, lastch)) == ' ' || c == '\t') {
     }
@@ -215,8 +216,13 @@ nexttoken(FILE *fp, char *tokenbuf, int *lastch)
 	token = TILDE;
 	break;
     case '"':
-	p = tokenbuf;
+	p = *tokenbuf;
 	while ((c = nextch(fp, lastch)) != '"') {
+	    if (len >= *buflen - 1) {
+		    *buflen += BUFSIZ;
+		    *tokenbuf = (char *)realloc(*tokenbuf, *buflen);
+		    p = *tokenbuf + len;
+	    }
 	    if (c == '\n' || c == EOF) {
 		putbackch(c, lastch);
 		token = ERROR;
@@ -227,15 +233,19 @@ nexttoken(FILE *fp, char *tokenbuf, int *lastch)
 		case '\\':
 		case '"':
 		    *p++ = c;
+		    len++;
 		    break;
 		case 'n':
 		    *p++ = '\n';
+		    len++;
 		    break;
 		case 'r':
 		    *p++ = '\r';
+		    len++;
 		    break;
 		case 't':
 		    *p++ = '\t';
+		    len++;
 		    break;
 		case '0':
 		case '1':
@@ -254,6 +264,7 @@ nexttoken(FILE *fp, char *tokenbuf, int *lastch)
 		    }
 		    putbackch(c, lastch);
 		    *p++ = (char)i;
+		    len++;
 		    break;
 		case 'X':
 		case 'x':
@@ -278,6 +289,7 @@ nexttoken(FILE *fp, char *tokenbuf, int *lastch)
 			goto string_error;
 		    }
 		    *p++ = (char)i;
+		    len++;
 		    break;
 		case EOF:
 		    putbackch(c, lastch);
@@ -285,10 +297,12 @@ nexttoken(FILE *fp, char *tokenbuf, int *lastch)
 		    goto string_error;
 		default:
 		    *p++ = c;
+		    len++;
 		    break;
 		}
 	    } else {
 		*p++ = c;
+		len++;
 	    }
 	}
 	*p = '\0';
@@ -305,11 +319,22 @@ nexttoken(FILE *fp, char *tokenbuf, int *lastch)
 	break;
     default:
 	if (isalnum(c) || c == '_' || c == '-') {
-	    p = tokenbuf;
+	    if (len >= *buflen - 1) {
+		*buflen += BUFSIZ;
+		*tokenbuf = (char *)realloc(*tokenbuf,  *buflen);
+	    }
+	    p = *tokenbuf;
 	    *p++ = c;
+	    len++;
 	    c = nextch(fp, lastch);
 	    while (isalnum(c) || c == '_' || c == '-') {
+	        if (len >= *buflen - 1) {
+			*buflen += BUFSIZ;
+			*tokenbuf = (char *)realloc(*tokenbuf,  *buflen);
+			p = *tokenbuf + len;
+		}
 		*p++ = c;
+		len++;
 		c = nextch(fp, lastch);
 	    }
 	    *p = '\0';
@@ -440,7 +465,7 @@ get_mb_string(char *buf, KeySym ks)
 	return 0;
     len = strlen(mb);
     strlcpy(buf, mb, MB_LEN_MAX + 1);
-    free(mb);
+    g_free(mb);
 
     return len;
 }
@@ -450,7 +475,7 @@ get_mb_string(char *buf, KeySym ks)
 #define SEQUENCE_MAX    10
 
 static int
-parse_compose_line(FILE *fp, char* tokenbuf)
+parse_compose_line(FILE *fp, char **tokenbuf, size_t *buflen)
 {
     int token;
     unsigned modifier_mask;
@@ -479,7 +504,7 @@ parse_compose_line(FILE *fp, char* tokenbuf)
     g_get_charset(&encoding);
 
     do {
-	token = nexttoken(fp, tokenbuf, &lastch);
+	token = nexttoken(fp, tokenbuf, &lastch, buflen);
     } while (token == ENDOFLINE);
 
     if (token == ENDOFFILE) {
@@ -488,41 +513,42 @@ parse_compose_line(FILE *fp, char* tokenbuf)
 
     n = 0;
     do {
-	if ((token == KEY) && (strcmp("include", tokenbuf) == 0)) {
+	if ((token == KEY) && (strcmp("include", *tokenbuf) == 0)) {
 	    char *filename;
 	    FILE *infp;
-	    token = nexttoken(fp, tokenbuf, &lastch);
+	    token = nexttoken(fp, tokenbuf, &lastch, buflen);
 	    if (token != KEY && token != STRING)
 		goto error;
 
-	    if ((filename = TransFileName(tokenbuf)) == NULL)
+	    if ((filename = TransFileName(*tokenbuf)) == NULL)
 		goto error;
 	    infp = fopen(filename, "r");
 	    free(filename);
 	    if (infp == NULL)
 		goto error;
 	    ParseComposeStringFile(infp);
-	    return (0);
-	} else if ((token == KEY) && (strcmp("None", tokenbuf) == 0)) {
+	    fclose(infp);
+	    return 0;
+	} else if ((token == KEY) && (strcmp("None", *tokenbuf) == 0)) {
 	    modifier = 0;
 	    modifier_mask = AllMask;
-	    token = nexttoken(fp, tokenbuf, &lastch);
+	    token = nexttoken(fp, tokenbuf, &lastch, buflen);
 	} else {
 	    modifier_mask = modifier = 0;
 	    exclam = False;
 	    if (token == EXCLAM) {
 		exclam = True;
-		token = nexttoken(fp, tokenbuf, &lastch);
+		token = nexttoken(fp, tokenbuf, &lastch, buflen);
 	    }
 	    while (token == TILDE || token == KEY) {
 		tilde = False;
 		if (token == TILDE) {
 		    tilde = True;
-		    token = nexttoken(fp, tokenbuf, &lastch);
+		    token = nexttoken(fp, tokenbuf, &lastch, buflen);
 		    if (token != KEY)
 			goto error;
 		}
-		tmp = modmask(tokenbuf);
+		tmp = modmask(*tokenbuf);
 		if (!tmp) {
 		    goto error;
 		}
@@ -532,7 +558,7 @@ parse_compose_line(FILE *fp, char* tokenbuf)
 		} else {
 		    modifier |= tmp;
 		}
-		token = nexttoken(fp, tokenbuf, &lastch);
+		token = nexttoken(fp, tokenbuf, &lastch, buflen);
 	    }
 	    if (exclam) {
 		modifier_mask = AllMask;
@@ -544,17 +570,17 @@ parse_compose_line(FILE *fp, char* tokenbuf)
 	    goto error;
 	}
 
-	token = nexttoken(fp, tokenbuf, &lastch);
+	token = nexttoken(fp, tokenbuf, &lastch, buflen);
 	if (token != KEY) {
 	    goto error;
 	}
 
-	token = nexttoken(fp, tokenbuf, &lastch);
+	token = nexttoken(fp, tokenbuf, &lastch, buflen);
 	if (token != GREATER) {
 	    goto error;
 	}
 
-	keysym = XStringToKeysym(tokenbuf);
+	keysym = XStringToKeysym(*tokenbuf);
 	if (keysym == NoSymbol) {
 	    goto error;
 	}
@@ -565,33 +591,33 @@ parse_compose_line(FILE *fp, char* tokenbuf)
 	n++;
 	if (n >= SEQUENCE_MAX)
 	    goto error;
-	token = nexttoken(fp, tokenbuf, &lastch);
+	token = nexttoken(fp, tokenbuf, &lastch, buflen);
     } while (token != COLON);
 
-    token = nexttoken(fp, tokenbuf, &lastch);
+    token = nexttoken(fp, tokenbuf, &lastch, buflen);
     if (token == STRING) {
-	if ((rhs_string_mb = (char *)malloc(strlen(tokenbuf) + 1)) == NULL)
+	if ((rhs_string_mb = (char *)malloc(strlen(*tokenbuf) + 1)) == NULL)
 	    goto error;
-	strcpy(rhs_string_mb, tokenbuf);
-	token = nexttoken(fp, tokenbuf, &lastch);
+	strcpy(rhs_string_mb, *tokenbuf);
+	token = nexttoken(fp, tokenbuf, &lastch, buflen);
 	if (token == KEY) {
-	    rhs_keysym = XStringToKeysym(tokenbuf);
+	    rhs_keysym = XStringToKeysym(*tokenbuf);
 	    if (rhs_keysym == NoSymbol) {
 		free(rhs_string_mb);
 		goto error;
 	    }
-	    token = nexttoken(fp, tokenbuf, &lastch);
+	    token = nexttoken(fp, tokenbuf, &lastch, buflen);
 	}
 	if (token != ENDOFLINE && token != ENDOFFILE) {
 	    free(rhs_string_mb);
 	    goto error;
 	}
     } else if (token == KEY) {
-	rhs_keysym = XStringToKeysym(tokenbuf);
+	rhs_keysym = XStringToKeysym(*tokenbuf);
 	if (rhs_keysym == NoSymbol) {
 	    goto error;
 	}
-	token = nexttoken(fp, tokenbuf, &lastch);
+	token = nexttoken(fp, tokenbuf, &lastch, buflen);
 	if (token != ENDOFLINE && token != ENDOFFILE) {
 	    goto error;
 	}
@@ -611,7 +637,12 @@ parse_compose_line(FILE *fp, char* tokenbuf)
 	goto error;
     }
 
-    rhs_string_utf8 = g_locale_to_utf8(rhs_string_mb, -1, NULL, NULL, NULL);
+    {
+	char *result;
+	result = g_locale_to_utf8(rhs_string_mb, -1, NULL, NULL, NULL);
+	rhs_string_utf8 = strdup(result);
+	g_free(result);
+    }
 
     for (i = 0; i < n; i++) {
 	for (p = *top; p; p = p->next) {
@@ -651,7 +682,7 @@ parse_compose_line(FILE *fp, char* tokenbuf)
     return n;
 error:
     while (token != ENDOFLINE && token != ENDOFFILE) {
-	token = nexttoken(fp, tokenbuf, &lastch);
+	token = nexttoken(fp, tokenbuf, &lastch, buflen);
     }
     return 0;
 }
@@ -659,22 +690,19 @@ error:
 static void
 ParseComposeStringFile(FILE *fp)
 {
-    char tb[8192];
-    char* tbp;
+    char *tbp, *p[1];
     struct stat st;
+    size_t buflen = BUFSIZ;
 
-    if (fstat(fileno(fp), &st) != -1) {
-	unsigned long size = (unsigned long)st.st_size;
-	if (size <= sizeof tb)
-	    tbp = tb;
-	else
-	    tbp = (char *)malloc(size);
+    if (fstat(fileno(fp), &st) != -1 && S_ISREG(st.st_mode) &&
+	st.st_size > 0) {
 
+	tbp = (char *)malloc(buflen);
+	p[0] = tbp;
 	if (tbp != NULL) {
-	    while (parse_compose_line(fp, tbp) >= 0) {
+	    while (parse_compose_line(fp, p, &buflen) >= 0) {
 	    }
-	    if (tbp != tb)
-		free (tbp);
+	    free(p[0]);
 	}
     }
 }
@@ -840,7 +868,7 @@ get_compose_filename()
 	char *p = buf;
 	int n;
 	char *args[2], *from, *to;
-	while (isspace(*p)) {
+	while (isspace((unsigned char)*p)) {
 	    ++p;
 	}
 	if (iscomment(*p)) {
@@ -883,7 +911,7 @@ parse_line(char *line, char **argv, int argsize)
     char *p = line;
 
     while (argc < argsize) {
-	while (isspace(*p)) {
+	while (isspace((unsigned char)*p)) {
 	    ++p;
 	}
 	if (*p == '\0') {
