@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -30,8 +30,8 @@
   SUCH DAMAGE.
 */
 
-#ifndef _compose_h_included_
-#define _compose_h_included_
+#ifndef UIM_GTK_COMPOSE_H
+#define UIM_GTK_COMPOSE_H
 
 #include <X11/X.h>
 
