@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -35,8 +35,8 @@
  * key utility for uim-gtk
  */
 
-#ifndef _key_util_gtk_h_included_
-#define _key_util_gtk_h_included_
+#ifndef UIM_GTK_KEY_UTIL_GTK_H
+#define UIM_GTK_KEY_UTIL_GTK_H
 
 /*
  * Initialize modifier key mappings.
