@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2006 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -34,8 +34,8 @@
 /* FIXME! This is a ad-hoc solution to advance
    annotation related discussion. */
 
-#ifndef _uim_eb_h_included_
-#define _uim_eb_h_included_
+#ifndef UIM_GTK_UIM_EB_H
+#define UIM_GTK_UIM_EB_H
 
 #include <gtk/gtk.h>
 
@@ -50,4 +50,4 @@ void    uim_eb_destroy     (uim_eb *ueb);
 
 G_END_DECLS
 
-#endif /* _uim_eb_h_included_ */
+#endif /* UIM_GTK_UIM_EB_H */
