@@ -1,7 +1,7 @@
 /*
  *  $Id:$
  *  Copyright (c) 2003,2004 Masahito Omote <omote@utyuuzin.net>
- *                2005 uim Project http://uim.freedesktop.org/
+ *                2005-2008 uim Project http://code.google.com/p/uim/
  *
  *  All rights reserved.
  *
@@ -31,8 +31,8 @@
  *  SUCH DAMAGE.
  */
 
-#ifndef __dict_anthy_h_included__
-#define __dict_anthy_h_included__
+#ifndef UIM_DICT_ANTHY_H
+#define UIM_DICT_ANTHY_H
 
 #include "dict-word.h"
 #include "dict-dict.h"
@@ -57,4 +57,4 @@ int    dict_anthy_delete_priv_dic         (char  *phon,
 #ifdef __cplusplus
 }
 #endif
-#endif /* __dict_anthy_h_included__ */
+#endif /* UIM_DICT_ANTHY_H */
