@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2006-2007 uim Project http://uim.freedesktop.org/
+ *  Copyright (c) 2006-2008 uim Project http://code.google.com/p/uim/
  *
  *  All rights reserved.
  *
@@ -29,8 +29,8 @@
  *  SUCH DAMAGE.
  */
 
-#ifndef __dict_canna_h_included__
-#define __dict_canna_h_included__
+#ifndef UIM_DICT_CANNA_H
+#define UIM_DICT_CANNA_H
 
 #include "dict-word.h"
 #include "dict-dict.h"
@@ -39,4 +39,4 @@
 #define CANNA_PRIV_DIC_BUFLEN	1024
 #define CANNA_STD_CONTEXT	0
 
-#endif /* __dict_canna_h_included__ */
+#endif /* UIM_DICT_CANNA_H */
