@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2004-2007 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2004-2008 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -31,8 +31,8 @@
 
 */
 
-#ifndef __dict_dict_h_includeded__
-#define __dict_dict_h_includeded__
+#ifndef UIM_DICT_DICT_H
+#define UIM_DICT_DICT_H
 
 #include "dict-word.h"
 
@@ -105,4 +105,4 @@ typedef enum {
   DICT_ENUM_DICTIONARY_TYPE_UNKOWN
 } DictEnumDictionaryType;
 
-#endif /* __dict_dict_h_includeded__ */
+#endif /* UIM_DICT_DICT_H */
