@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2004-2007 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2004-2008 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -31,8 +31,8 @@
 
 */
 
-#ifndef __dict_util_h_includeded__
-#define __dict_util_h_includeded__
+#ifndef UIM_DICT_UTIL_H
+#define UIM_DICT_UTIL_H
 
 #include <glib.h>
 
@@ -43,4 +43,4 @@ gchar *utf8_convert    (const gchar *in);
 gchar *utf8_to_eucjp   (const gchar *utf8);
 gchar *eucjp_to_utf8   (const gchar *eucjp);
 
-#endif /* __dict_util_h_includeded__ */
+#endif /* UIM_DICT_UTIL_H */
