@@ -1,7 +1,7 @@
 /*
  *  $Id:$
  *  Copyright (c) 2003,2004 Masahito Omote <omote@utyuuzin.net>
- *                2005-2006 uim Project http://uim.freedesktop.org/
+ *                2005-2008 uim Project http://code.google.com/p/uim/
  *
  *  All rights reserved.
  *
@@ -31,8 +31,8 @@
  *  SUCH DAMAGE.
  */
 
-#ifndef __WORD_LIST_VIEW_H__
-#define __WORD_LIST_VIEW_H__
+#ifndef UIM_DICT_WORD_LIST_VIEW_H
+#define UIM_DICT_WORD_LIST_VIEW_H
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -127,4 +127,4 @@ GtkSelectionMode word_list_view_get_selection_mode (WordListView *);
 #ifdef __cplusplus
 }
 #endif
-#endif /* __WORD_LIST_VIEW_H__ */
+#endif /* UIM_DICT_WORD_LIST_VIEW_H */
