@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2004-2006 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2004-2008 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -37,7 +37,6 @@
 #include <gdk/gdkkeysyms.h>
 
 #include "gettext.h"
-#include "uim-stdint.h"
 
 #include "dict-word.h"
 #include "dict-anthy.h"
@@ -583,7 +582,7 @@ help_about_action_cb(GtkAction *action, WordListWindow *window)
   };
   const gchar *copyright = N_(
     "Copyright (C) 2003-2004 Masahito Omote\n"
-    "Copyright (C) 2004-2006 uim Project\n"
+    "Copyright (C) 2004-2008 uim Project\n"
     "All rights reserved.");
 
   transparent = NULL;
@@ -598,7 +597,7 @@ help_about_action_cb(GtkAction *action, WordListWindow *window)
 			 "name", name,
 			 "version", VERSION,
 			 "copyright", copyright,
-			 "website", "http://uim.freedesktop.org",
+			 "website", "http://code.google.com/p/uim/",
 			 "authors", authors,
 			 "logo", transparent,
 			 NULL);
@@ -607,7 +606,7 @@ help_about_action_cb(GtkAction *action, WordListWindow *window)
   GtkWidget *about_dialog, *label1;
   const gchar *copyright = N_(
     "Copyright 2003-2004 Masahito Omote &lt;omote@utyuuzin.net&gt;\n"
-    "Copyright 2004-2006 uim Project http://uim.freedesktop.org/\n"
+    "Copyright 2004-2008 uim Project http://code.google.com/p/uim/\n"
     "All rights reserved.");
   gchar *about_name =
     g_strdup_printf("<span size=\"20000\">%s %s </span>\n\n<span size=\"14000\">%s </span>\n", _(name), VERSION, _(copyright));
