@@ -1,6 +1,6 @@
 /*
 
- Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
+ Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
 
  All rights reserved.
 
@@ -30,8 +30,8 @@
  SUCH DAMAGE.
 
 */
-#ifndef _UIM_CANDWIN_QT_H_
-#define _UIM_CANDWIN_QT_H_
+#ifndef UIM_QT_CANDWIN_QT_H
+#define UIM_QT_CANDWIN_QT_H
 
 #include <uim/uim.h>
 #include <uim/uim-helper.h>
@@ -136,4 +136,4 @@ public:
     }
 };
 
-#endif  /* _UIM_CANDWIN_QT_H_ */
+#endif  /* UIM_QT_CANDWIN_QT_H */
