@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
+Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
 
 All rights reserved.
 
@@ -30,8 +30,8 @@ OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 SUCH DAMAGE.
 
 */
-#ifndef _BUSHU_VIEW_WIDGET_H_
-#define _BUSHU_VIEW_WIDGET_H_
+#ifndef UIM_QT_CHARDICT_BUSHU_VIEW_WIDGET_H
+#define UIM_QT_CHARDICT_BUSHU_VIEW_WIDGET_H
 
 #include "chardict-qt.h"
 
@@ -68,4 +68,4 @@ protected:
     CharGridView *m_charGridView;
 };
 
-#endif /* Not def: _BUSHU_VIEW_WIDGET_H_ */
+#endif /* Not def: UIM_QT_CHARDICT_BUSHU_VIEW_WIDGET_H */
