@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
+Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
 
 All rights reserved.
 
@@ -30,8 +30,8 @@ OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 SUCH DAMAGE.
 
 */
-#ifndef _UIM_CHARDICT_QT_H_
-#define _UIM_CHARDICT_QT_H_
+#ifndef UIM_QT_CHARDICT_CHARDICT_QT_H
+#define UIM_QT_CHARDICT_CHARDICT_QT_H
 
 #include <qstring.h>
 #include <qwidget.h>
@@ -100,4 +100,4 @@ signals:
     void charSelected( const QString & );
 };
 
-#endif /* Not def: _UIM_CHARDICT_QT_H_ */
+#endif /* Not def: UIM_QT_CHARDICT_CHARDICT_QT_H */
