@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2003-2006 uim Project http://uim.freedesktop.org/
+Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
 
 All rights reserved.
 
@@ -30,8 +30,8 @@ OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 SUCH DAMAGE.
 
 */
-#ifndef _CANDIDATE_WINDOW_H_
-#define _CANDIDATE_WINDOW_H_
+#ifndef UIM_QT_IMMODULE_CANDIDATE_WINDOW_H
+#define UIM_QT_IMMODULE_CANDIDATE_WINDOW_H
 
 #include <uim/uim.h>
 
@@ -146,4 +146,4 @@ public:
 
     QSize sizeHint( void ) const;
 };
-#endif /* Not def: _CANDIDATE_WINDOW_H_ */
+#endif /* Not def: UIM_QT_IMMODULE_CANDIDATE_WINDOW_H */
