@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
+Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
 
 All rights reserved.
 
@@ -33,15 +33,23 @@ SUCH DAMAGE.
 #include <config.h>
 
 #include <qinputcontext.h>
+#if UIM_QT_USE_JAPANESE_KANA_KEYBOARD_HACK
+#include <qwindowdefs.h>
+#endif
 
 #include <locale.h>
 
 #include "uim/uim.h"
+#include "uim/counted-init.h"
 
 #include "immodule-plugin.h"
 #include "immodule-quiminputcontext_with_slave.h"
 #include "immodule-quiminfomanager.h"
 
+#if UIM_QT_USE_JAPANESE_KANA_KEYBOARD_HACK
+#include "uim/uim-x-util.h"
+#endif
+
 QUimInfoManager *UimInputContextPlugin::infoManager = NULL;
 
 
@@ -114,9 +122,12 @@ UimInputContextPlugin::getQUimInfoManager()
 void
 UimInputContextPlugin::uimInit()
 {
-    if ( !uim_init() ) {
+    if ( !uim_counted_init() ) {
         if (!infoManager)
             infoManager = new QUimInfoManager();
+#if UIM_QT_USE_JAPANESE_KANA_KEYBOARD_HACK
+	uim_x_kana_input_hack_init(qt_xdisplay());
+#endif
         uimReady = true;
     }
 }
@@ -126,7 +137,7 @@ UimInputContextPlugin::uimQuit()
 {
     if ( uimReady )
     {
-        uim_quit();
+        uim_counted_quit();
         delete infoManager;
         uimReady = false;
     }
