@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
+Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
 
 All rights reserved.
 
@@ -163,15 +163,11 @@ void QUimHelperManager::parseHelperStr( const QString &str )
     }
     else if ( str.startsWith( "custom_reload_notify" ) )
     {
-#if 1
         uim_prop_reload_configs();
-#else
-        // Use QUimInputContext::reloadUim() since uim_prop_reload_configs
-        // cannot update IM list of libuim yet (bug #2412).
-        // But disabled since uim_quit() and uim_init() in reloadUim() 
-        // causes some problems for uim-pref-qt.
-        QUimInputContext::reloadUim();
-#endif
+
+        QUimInfoManager *infoManager =
+            UimInputContextPlugin::getQUimInfoManager();
+	infoManager->initUimInfo();
     }
 }
 
