@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
+Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
 
 All rights reserved.
 
@@ -47,6 +47,7 @@ SUCH DAMAGE.
 #include "uim/uim.h"
 #include "uim/uim-helper.h"
 #include "uim/uim-im-switcher.h"
+#include "uim/uim-scm.h"
 
 #include "immodule-quiminputcontext.h"
 #include "immodule-candidatewindow.h"
@@ -58,6 +59,12 @@ SUCH DAMAGE.
 #include "immodule-quiminputcontext_compose.h"
 #endif
 
+#if UIM_QT_USE_JAPANESE_KANA_KEYBOARD_HACK
+#include <X11/Xlib.h>
+
+#include "uim/uim-x-util.h"
+#endif
+
 #define DEFAULT_SEPARATOR_STR "|"
 
 QUimInputContext *focusedInputContext = NULL;
@@ -85,6 +92,10 @@ QUimInputContext::QUimInputContext( const char *imname, const char *lang )
 
     contextList.append( this );
 
+    // must be initialized before createUimContext() call
+    if ( !m_HelperManager )
+        m_HelperManager = new QUimHelperManager();
+
     if ( imname )
         m_uc = createUimContext( imname );
 
@@ -95,9 +106,6 @@ QUimInputContext::QUimInputContext( const char *imname, const char *lang )
     cwin->setQUimInputContext( this );
     cwin->hide();
 
-    if ( !m_HelperManager )
-        m_HelperManager = new QUimHelperManager();
-
 #ifdef Q_WS_X11
     if ( !mTreeTop )
         create_compose_tree();
@@ -137,7 +145,7 @@ uim_context QUimInputContext::createUimContext( const char *imname )
 
     uim_context uc = uim_create_context( this, "UTF-8",
                                          NULL, ( char * ) imname,
-                                         uim_iconv,
+                                         NULL,
                                          QUimInputContext::commit_cb );
 
     m_HelperManager->checkHelperConnection();
@@ -171,6 +179,21 @@ uim_context QUimInputContext::createUimContext( const char *imname )
     return uc;
 }
 
+#ifdef Q_WS_X11
+bool QUimInputContext::x11FilterEvent( QWidget *keywidget, XEvent *event )
+{
+    // to suppress warning
+    keywidget = keywidget;
+    event = event;
+
+#if UIM_QT_USE_JAPANESE_KANA_KEYBOARD_HACK
+    return uim_x_kana_input_hack_filter_event( m_uc, event );
+#else
+    return FALSE;
+#endif
+}
+#endif // Q_WS_X11
+
 bool QUimInputContext::filterEvent( const QEvent *event )
 {
 #ifdef ENABLE_DEBUG
@@ -352,27 +375,6 @@ QUimInputContext * QUimInputContext::focusedIC()
     return focusedInputContext;
 }
 
-void QUimInputContext::reloadUim()
-{
-    QUimInputContext *ic;
-    QUimInfoManager *infoManager = UimInputContextPlugin::getQUimInfoManager();
-
-    for ( ic = contextList.first(); ic; ic = contextList.next() )
-    {
-        ic->reset();
-        uim_release_context( ic->m_uc );
-    }
-
-    uim_quit();
-    uim_init();
-    infoManager->initUimInfo();
-
-    for ( ic = contextList.first(); ic; ic = contextList.next() )
-    {
-        ic->m_uc = ic->createUimContext( ic->m_imname );
-    }
-}
-
 void QUimInputContext::setMicroFocus( int x, int y, int w, int h, QFont * /* f */)
 {
 #ifdef ENABLE_DEBUG
@@ -732,7 +734,7 @@ void QUimInputContext::switch_system_global_im( const char *name )
 
 void QUimInputContext::readIMConf()
 {
-    char * leftp = uim_symbol_value_str( "candidate-window-position" );
+    char * leftp = uim_scm_symbol_value_str( "candidate-window-position" );
     if ( leftp && !strcmp( leftp, "left" ) )
         cwin->setAlwaysLeftPosition( true );
     else
