@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2003-2006 uim Project http://uim.freedesktop.org/
+Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
 
 All rights reserved.
 
@@ -30,16 +30,17 @@ OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 SUCH DAMAGE.
 
 */
-#ifndef _QUIMINPUT_CONTEXT_H_
-#define _QUIMINPUT_CONTEXT_H_
+#ifndef UIM_QT_IMMODULE_QUIMINPUTCONTEXT_H
+#define UIM_QT_IMMODULE_QUIMINPUTCONTEXT_H
 
 #include <qinputcontext.h>
 #include <qptrlist.h>
 
-#include "uim/uim.h"
-#include "uim/uim-util.h"
-#include "uim/uim-helper.h"
-#include "uim/uim-im-switcher.h"
+#ifdef Q_WS_X11
+#define UIM_QT_USE_JAPANESE_KANA_KEYBOARD_HACK 1
+#endif
+
+class QString;
 
 class CandidateWindow;
 class QUimHelperManager;
@@ -47,6 +48,7 @@ class QUimHelperManager;
 typedef struct _DefTree DefTree;
 class Compose;
 #endif
+class QUimTextUtil;
 
 class PreeditSegment
 {
@@ -71,6 +73,9 @@ public:
     virtual QString identifierName();
     virtual QString language();
 
+#ifdef Q_WS_X11
+    virtual bool x11FilterEvent( QWidget *keywidget, XEvent *event );
+#endif
     virtual bool filterEvent( const QEvent *event );
     virtual void reset();
     virtual void setFocus();
@@ -88,14 +93,18 @@ public:
 
     void readIMConf();
 
-protected:
-    uim_context createUimContext( const char *imname );
-    virtual bool isPreeditPreservationEnabled();  // not a QInputContext func
+    QUimTextUtil *textUtil() { return mTextUtil; }
 
-    void createUimInfo();
-private:
     QString getPreeditString();
     int getPreeditCursorPosition();
+
+    void saveContext();
+    void restoreContext();
+
+protected:
+    uim_context createUimContext( const char *imname );
+
+private:
     int getPreeditSelectionLength();
 
     /* callbacks for uim */
@@ -133,13 +142,14 @@ private:
     static char *TransFileName( char *name );
     static void ParseComposeStringFile( FILE *fp );
     static void FreeComposeTree( DefTree *top );
-    static int parse_compose_line( FILE *fp, char *tokenbuf );
+    static int parse_compose_line( FILE *fp, char **tokenbuf, size_t *buflen );
     static int get_mb_string( char *buf, unsigned int ks );
     static const char *get_encoding( void );
     static char *get_lang_region( void );
 
     Compose *mCompose;
 #endif
+    QUimTextUtil *mTextUtil;
 
 protected:
     QString m_imname;
@@ -148,17 +158,9 @@ protected:
     bool candwinIsActive;
 
     QPtrList<PreeditSegment> psegs;
-    QString preeditString;
 
     CandidateWindow *cwin;
     static QUimHelperManager *m_HelperManager;
 };
 
-struct UIMInfo
-{
-    const char *lang;
-    const char *name;
-    const char *short_desc;
-};
-
-#endif /* Not def: _QUIMINPUT_CONTEXT_H_ */
+#endif /* Not def: UIM_QT_IMMODULE_QUIMINPUTCONTEXT_H */
