@@ -1,6 +1,6 @@
 /*
 
- Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
+ Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
 
  All rights reserved.
 
@@ -30,8 +30,8 @@
  SUCH DAMAGE.
 
 */
-#ifndef _UIM_PREF_QT_H_
-#define _UIM_PREF_QT_H_
+#ifndef UIM_QT_PREF_QT_H
+#define UIM_QT_PREF_QT_H
 
 #include <qdialog.h>
 #include <qdict.h>
@@ -138,4 +138,4 @@ protected:
     QPtrList<UimCustomItemIface> m_customIfaceList;
 };
 
-#endif /* Not def: _UIM_PREF_QT_H_ */
+#endif /* Not def: UIM_QT_PREF_QT_H */
