@@ -1,6 +1,6 @@
 /*
 
- Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
+ Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
 
  All rights reserved.
 
@@ -30,8 +30,8 @@
  SUCH DAMAGE.
 
 */
-#ifndef _UIM_STATE_INDICATOR_H_
-#define _UIM_STATE_INDICATOR_H_
+#ifndef UIM_QT_TOOLBAR_COMMON_UIM_STATE_INDICATOR_H
+#define UIM_QT_TOOLBAR_COMMON_UIM_STATE_INDICATOR_H
 
 #include <qhbox.h>
 #include <qintdict.h>
@@ -113,4 +113,4 @@ protected:
 };
 
 
-#endif /* Not def: _UIM_STATE_INDICATOR_H_ */
+#endif /* Not def: UIM_QT_TOOLBAR_COMMON_UIM_STATE_INDICATOR_H */
