@@ -1,6 +1,6 @@
 /*
 
- Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
+ Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
 
  All rights reserved.
 
@@ -30,8 +30,8 @@
  SUCH DAMAGE.
 
 */
-#ifndef _UIM_TOOLBAR_DRAGGING_HANDLER_H_
-#define _UIM_TOOLBAR_DRAGGING_HANDLER_H_
+#ifndef UIM_QT_TOOLBAR_DRAGGING_HANDLER_H
+#define UIM_QT_TOOLBAR_DRAGGING_HANDLER_H
 
 #include <qframe.h>
 #include <qevent.h>
@@ -83,4 +83,4 @@ private:
     int offsetY;
 };
 
-#endif  /* _UIM_TOOLBAR_DRAGGING_HANDLER_H_ */
+#endif  /* UIM_QT_TOOLBAR_DRAGGING_HANDLER_H */
