@@ -81,6 +81,8 @@ CandidateWindow::CandidateWindow( QWidget *parent, const char * name )
     cList->setVScrollBarMode( Q3ScrollView::AlwaysOff );
     cList->setHScrollBarMode( Q3ScrollView::AlwaysOff );
     cList->setAllColumnsShowFocus( true );
+    QSizePolicy sp( QSizePolicy::Preferred, QSizePolicy::Preferred, false );
+    cList->setSizePolicy( sp );
     QObject::connect( cList, SIGNAL( clicked( Q3ListViewItem * ) ),
                       this , SLOT( slotCandidateSelected( Q3ListViewItem * ) ) );
     QObject::connect( cList, SIGNAL( selectionChanged( Q3ListViewItem * ) ),
@@ -88,6 +90,7 @@ CandidateWindow::CandidateWindow( QWidget *parent, const char * name )
 
     //setup NumberLabel
     numLabel = new QLabel( this, "candidateLabel" );
+    numLabel->adjustSize();
 
     stores.clear();
 
@@ -98,7 +101,7 @@ CandidateWindow::CandidateWindow( QWidget *parent, const char * name )
 
     isAlwaysLeft = false;
 
-    subWin = new SubWindow( 0 );
+    subWin = NULL;
 }
 
 CandidateWindow::~CandidateWindow()
@@ -123,11 +126,15 @@ void CandidateWindow::activateCandwin( int dLimit )
     candidateIndex = -1;
     displayLimit = dLimit;
     pageIndex = 0;
+
+    if ( !subWin )
+        subWin = new SubWindow( this );
 }
 
 void CandidateWindow::deactivateCandwin()
 {
-    subWin->cancelHook();
+    if ( subWin )
+        subWin->cancelHook();
 
     hide();
     clearCandidates();
@@ -255,6 +262,7 @@ void CandidateWindow::setPage( int page )
         updateLabel();
 
     // size adjustment
+    cList->updateGeometry();
     adjustSize();
 }
 
@@ -378,14 +386,16 @@ void CandidateWindow::updateLabel()
 
 void CandidateWindow::slotHookSubwindow( Q3ListViewItem * item )
 {
-    // cancel previous hook
-    subWin->cancelHook();
+    if ( subWin ) {
+        // cancel previous hook
+        subWin->cancelHook();
 
-    // hook annotation
-    QString annotationString = item->text( 2 );
-    if ( !annotationString.isEmpty() )
-    {
-        subWin->hookPopup( "Annotation", annotationString );
+        // hook annotation
+        QString annotationString = item->text( 2 );
+        if ( !annotationString.isEmpty() )
+        {
+            subWin->hookPopup( "Annotation", annotationString );
+        }
     }
 }
 
@@ -393,13 +403,15 @@ void CandidateWindow::slotHookSubwindow( Q3ListViewItem * item )
 void CandidateWindow::moveEvent( QMoveEvent *e )
 {
     // move subwindow
-    subWin->layoutWindow( e->pos().x() + width(), e->pos().y() );
+    if ( subWin )
+        subWin->layoutWindow( e->pos().x() + width(), e->pos().y() );
 }
 
 void CandidateWindow::resizeEvent( QResizeEvent *e )
 {
     // move subwindow
-    subWin->layoutWindow( pos().x() + e->size().width(), pos().y() );
+    if ( subWin )
+        subWin->layoutWindow( pos().x() + e->size().width(), pos().y() );
 }
 
 
@@ -426,7 +438,7 @@ QSize CandidateListView::sizeHint( void ) const
     
     // 2004-08-02 Kazuki Ohta <mover@hct.zaq.ne.jp>
     // FIXME!:
-    //    There may be more proper way. Now width is adjusted by indeterminal 3 spaces.
+    //    There may be more proper way. Now width is adjusted by indeterminal 5 'J'.
     int maxCharIndex = 0, maxCharCount = 0;
     for ( int i = 0; i < childCount(); i++ )
     {
@@ -437,7 +449,8 @@ QSize CandidateListView::sizeHint( void ) const
         }
     }
     QFontMetrics fm( font() );
-    width = fm.width( itemAtIndex( maxCharIndex )->text( 0 ) + "   " + itemAtIndex( maxCharIndex )->text( 1 ) );
+    width = fm.width( itemAtIndex( maxCharIndex )->text( 0 ) + "JJJJJ" + itemAtIndex( maxCharIndex )->text( 1 ) ) + itemMargin() * 4 + frameWidth() * 2;
+
     if ( width < MIN_CAND_WIDTH )
         width = MIN_CAND_WIDTH;
     
