@@ -1,5 +1,5 @@
 ;;;
-;;; Copyright (c) 2003-2006 uim Project http://uim.freedesktop.org/
+;;; Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -370,13 +370,18 @@
 (define skk-context-new
   (lambda (id im)
     (if (not skk-dic-init)
-	(begin
+	(let ((hostname (if skk-skkserv-use-env?
+			    (or (getenv "SKKSERVER") "localhost")
+			    skk-skkserv-hostname)))
 	  (set! skk-dic-init #t)
 	  (if skk-use-recursive-learning?
 	      (require "skk-editor.scm"))
 	  (require "skk-dialog.scm")
 	  (skk-lib-dic-open
-	     skk-dic-file-name skk-use-skkserv? skk-skkserv-portnum)
+	   skk-dic-file-name skk-use-skkserv? hostname skk-skkserv-portnum
+	   skk-skkserv-address-family)
+          (if skk-use-look?
+              (skk-lib-look-open skk-look-dict))
 	  (skk-read-personal-dictionary)))
     (let ((sc (skk-context-new-internal id im))
 	  (rkc (rk-context-new ja-rk-rule #t #f)))
@@ -430,9 +435,9 @@
 	   (lambda (l)
 	     (let ((c (string->charcode (caar l))))
 	       (cond
-		((char-upper-case? c)
+		((ichar-upper-case? c)
 		 (charcode->string (+ c 32)))
-		((char-lower-case? c)
+		((ichar-lower-case? c)
 		 (charcode->string (- c 32)))
 		(else
 		 (caar l)))))))
@@ -472,7 +477,7 @@
 
 (define skk-do-get-string
   (lambda (sc str kana)
-    (if str
+    (if (not (null? str))
 	(if (string? (car str))
 	    (skk-get-string-mode-part sc str kana)
 	    (string-append
@@ -607,7 +612,7 @@
 (define skk-append-string
   (lambda (sc str)
     (and
-     str
+     (not (null? str))
      (if (not (string? (car str)))
 	 (begin
 	   (skk-append-string sc (car str))
@@ -627,7 +632,7 @@
 (define skk-append-okuri-string
   (lambda (sc str)
     (and
-     str
+     (not (null? str))
      (if (not (string? (car str)))
 	 (begin
 	   (skk-append-okuri-string sc (car str))
@@ -723,7 +728,8 @@
 	   (not (skk-rk-pending? sc))
 	   (not (string=? (skk-context-dcomp-word sc) "")))
 	  (begin
-	    (im-pushback-preedit sc preedit-cursor "")
+	    (if skk-show-cursor-on-preedit?
+	        (im-pushback-preedit sc preedit-cursor ""))
 	    (im-pushback-preedit
 	     sc skk-preedit-attr-dcomp
 	     (skk-dcomp-word-tail sc))
@@ -745,7 +751,10 @@
 		  (not (skk-context-candidate-window sc))))
 		(im-pushback-preedit
 		 sc
-		 (bit-or skk-preedit-attr-conv-body preedit-cursor)
+		 (bitwise-ior skk-preedit-attr-conv-body
+			      (if skk-show-cursor-on-preedit?
+				  preedit-cursor
+				  preedit-none))
 		 (if skk-show-annotation-in-preedit?
 		     (skk-lib-eval-candidate (skk-get-current-candidate sc))
 		     (skk-lib-eval-candidate
@@ -753,7 +762,10 @@
 		       (skk-get-current-candidate sc)))))
 		(im-pushback-preedit
 		 sc
-		 (bit-or skk-preedit-attr-conv-body preedit-cursor)
+		 (bitwise-ior skk-preedit-attr-conv-body
+			      (if skk-show-cursor-on-preedit?
+				  preedit-cursor
+				  preedit-none))
 		 ""))
 	    (im-pushback-preedit
 	     sc skk-preedit-attr-conv-okuri
@@ -820,7 +832,8 @@
 	  (begin
 	    (im-pushback-preedit sc skk-preedit-attr-direct-pending-rk
 				 (rk-pending rkc))
-	    (im-pushback-preedit sc preedit-cursor ""))
+	    (if skk-show-cursor-on-preedit?
+	        (im-pushback-preedit sc preedit-cursor "")))
 	  (begin
 	    (im-pushback-preedit sc skk-preedit-attr-pending-rk
 				 (rk-pending rkc))
@@ -958,7 +971,7 @@
      ((or
        (and
 	(shift-key-mask key-state)
-	(not (char-graphic? key)))
+	(not (ichar-graphic? key)))
        (control-key-mask key-state)
        (alt-key-mask key-state)
        (meta-key-mask key-state)
@@ -979,7 +992,7 @@
 (define skk-proc-state-direct
   (lambda (c key key-state)
     (let* ((sc (skk-find-descendant-context c))
-	   (key-str (charcode->string (to-lower-char key)))
+	   (key-str (charcode->string (ichar-downcase key)))
 	   (rkc (skk-context-rk-context sc))
 	   (res #f)
 	   (kana (skk-context-kana-mode sc)))
@@ -1083,7 +1096,7 @@
        (if (or
 	    (and
 	     (shift-key-mask key-state)
-	     (not (char-graphic? key)))
+	     (not (ichar-graphic? key)))
 	    (control-key-mask key-state)
 	    (alt-key-mask key-state)
 	    (meta-key-mask key-state)
@@ -1094,7 +1107,7 @@
 	     (skk-commit-raw-with-preedit-update sc key key-state)
 	     #f)
 	   #t)
-       (if (char-upper-case? key)
+       (if (ichar-upper-case? key)
 	   (if (and 
 		(skk-rk-pending? sc)
 		(not (rk-current-seq rkc)))
@@ -1104,19 +1117,19 @@
 		 (skk-append-string sc (rk-push-key!
 					rkc
 					(charcode->string
-					 (to-lower-char key))))
+					 (ichar-downcase key))))
 		 #f)
 	       (let* ((residual-kana (rk-push-key-last! rkc)))
 		 ;; handle preceding "n"
 		 (if residual-kana
 		     (skk-commit sc (skk-get-string sc residual-kana kana)))
 		 (skk-context-set-state! sc 'skk-state-kanji)
-		 (set! key (to-lower-char key))
+		 (set! key (ichar-downcase key))
 		 #t))
 	   #t)
        ;; Hack to handle "n1" sequence as "1".
        ;; This should be handled in rk.scm. -- ekato
-       (if (and (not (alphabet-char? key))
+       (if (and (not (ichar-alphabetic? key))
 		(not (string-find (rk-expect rkc) key-str)))
 	   (let* ((residual-kana (rk-push-key-last! rkc)))
 	     (if residual-kana
@@ -1155,7 +1168,7 @@
 
 (define skk-sokuon-shiin-char?
   (lambda (c)
-    (and (alphabet-char? c)
+    (and (ichar-alphabetic? c)
 	 (and
 	  (not (= c 97))	;; a
 	  (not (= c 105))	;; i
@@ -1170,7 +1183,9 @@
 	   (s (cons key s))
 	   (rule (rk-context-rule rkc))
 	   (seq (rk-lib-find-seq (reverse s) rule)))
-	 (if (null? (cdar seq))
+	 (if (and
+	      seq
+	      (null? (cdar seq)))
 	     (cadr seq)
 	     #f))))
 
@@ -1384,7 +1399,7 @@
 	      (else
 	       ;; append latin string
 	       (begin
-		 (if (usual-char? key)
+		 (if (ichar-graphic? key)
 		     (let* ((s (charcode->string key))
 			    (p (cons s (cons s (cons s s)))))
 		       (skk-append-string sc p))))))
@@ -1411,9 +1426,9 @@
 	     (skk-begin-conversion sc)
 	     #f)
 	   #t)
-       (if (and (char-upper-case? key)
+       (if (and (ichar-upper-case? key)
 		(not (null? (skk-context-head sc))))
-	   (let ((key-str (charcode->string (to-lower-char key))))
+	   (let ((key-str (charcode->string (ichar-downcase key))))
 	     (set! res (skk-rk-push-key-match-without-new-seq rkc key-str))
 	     (if (and
 		  (skk-rk-pending? sc)
@@ -1431,7 +1446,7 @@
 		   #f)
 		 (begin
 		   (skk-context-set-state! sc 'skk-state-okuri)
-		   (set! key (to-lower-char key))
+		   (set! key (ichar-downcase key))
 		   (skk-context-set-okuri-head! sc key-str)
 		   (if (skk-sokuon-shiin-char? key)
 		       (begin
@@ -1468,10 +1483,10 @@
 	   #t)
        ;; Hack to handle "n1" sequence as "1".
        ;; This should be handled in rk.scm. -- ekato
-       (if (and (not (alphabet-char? key))
+       (if (and (not (ichar-alphabetic? key))
 		(not (string-find
 		      (rk-expect rkc)
-		      (charcode->string (to-lower-char key)))))
+		      (charcode->string (ichar-downcase key)))))
 	   (let* ((residual-kana (rk-push-key-last! rkc)))
 	     (if residual-kana
 		 (skk-context-set-head! sc
@@ -1481,7 +1496,7 @@
 	     #t)
 	   #t)
        (begin
-	 (set! key (to-lower-char key))  
+	 (set! key (ichar-downcase key))  
 	 (set! stat (skk-context-state sc))
 	 (set! res
 	       (rk-push-key!
@@ -1489,6 +1504,7 @@
 		(charcode->string key)))
 	 (and
 	  (if (and
+	       res
 	       skk-auto-start-henkan?
 	       (string-find skk-auto-start-henkan-keyword-list (car res))
 	       (not (null? (skk-context-head sc))))
@@ -1518,7 +1534,7 @@
 
 (define skk-setup-child-context
   (lambda (sc type)
-    (let ((csc (skk-context-new (skk-context-id sc)
+    (let ((csc (skk-context-new (skk-context-uc sc)
 				(skk-context-im sc)))
 	  (input-rule (skk-context-input-rule sc)))
       (skk-context-set-child-context! sc csc)
@@ -1966,7 +1982,7 @@
 	 (set! res
 	       (rk-push-key!
 		rkc
-		(charcode->string (to-lower-char key))))
+		(charcode->string (ichar-downcase key))))
 	 (if (and res
 	 	  (or
 		   (list? (car res))
@@ -1994,7 +2010,7 @@
 (define skk-proc-state-wide-latin
   (lambda (c key key-state)
     (let* ((char (charcode->string key))
-	   (w (ja-wide char))
+	   (w (if (symbol? key) #f (ja-wide char)))
 	   (sc (skk-find-descendant-context c)))
       (if skk-use-with-vi?
 	  (if (skk-vi-escape-key? key key-state)
@@ -2049,7 +2065,7 @@
 
 (define skk-press-key-handler
   (lambda (sc key state)
-    (if (control-char? key)
+    (if (ichar-control? key)
 	(im-commit-raw sc)
 	(skk-push-key sc key state))))
 
@@ -2067,6 +2083,18 @@
   (lambda (sc)
     (skk-flush sc)))
 
+(define skk-get-candidate-with-okuri
+  (lambda (cand okuri)
+    (let ((pos (string-contains cand ";" 0)))
+      (if pos
+	  (string-append
+	   (substring cand 0 pos)
+	   (skk-make-string okuri skk-type-hiragana)
+	   (substring cand pos (string-length cand)))
+	  (string-append
+	   cand
+	   (skk-make-string okuri skk-type-hiragana))))))
+
 (define skk-get-candidate-handler
   (lambda (sc idx accel-enum-hint)
     (let* ((dcsc (skk-find-descendant-context sc))
@@ -2083,8 +2111,7 @@
        (if (and
 	    (not (null? okuri))
 	    skk-show-candidates-with-okuri?)
-	   (string-append cand
-			  (skk-make-string okuri skk-type-hiragana))
+	   (skk-get-candidate-with-okuri cand okuri)
 	   cand)
        (cond
 	((eq? skk-candidate-selection-style 'uim)
@@ -2094,7 +2121,7 @@
 	       (set! idx (remainder idx skk-nr-candidate-max))
 	       (if (< idx (length skk-uim-heading-label-char-list))
 		   (charcode->string
-		    (char-upcase
+		    (ichar-upcase
 		     (string->charcode
 		      (nth idx skk-uim-heading-label-char-list))))
 		   ""))))
@@ -2103,7 +2130,7 @@
 	     (set! idx (remainder idx skk-nr-candidate-max)))
 	 (if (< idx (length skk-ddskk-like-heading-label-char-list))
 	     (charcode->string
-	      (char-upcase
+	      (ichar-upcase
 	       (string->charcode
 		(nth idx skk-ddskk-like-heading-label-char-list))))
 	     "")))
@@ -2138,4 +2165,10 @@
  skk-reset-handler
  skk-get-candidate-handler
  skk-set-candidate-index-handler
- context-prop-activate-handler)
+ context-prop-activate-handler
+ #f
+ #f
+ #f
+ #f
+ #f
+ )
