@@ -1,6 +1,6 @@
 ;;; ustr.scm: logical order string of abstract elements
 ;;;
-;;; Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
+;;; Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -234,7 +234,7 @@
 	     (<= pos (ustr-length ustr)))
 	(let* ((whole (ustr-whole-seq ustr))
 	       (latter (list-tail whole pos))
-	       (former (list-head whole pos)))
+	       (former (take whole pos)))
 	  (ustr-set-former-seq! ustr former)
 	  (ustr-set-latter-seq! ustr latter)
 	  #t)
