@@ -1,6 +1,6 @@
 ;; zaurus.scm: platform-specific support for Sharp Zaurus PDA
 ;;;
-;;; Copyright (c) 2003,2004 uim Project http://uim.freedesktop.org/
+;;; Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -16,10 +16,10 @@
 ;;;    may be used to endorse or promote products derived from this software
 ;;;    without specific prior written permission.
 ;;;
-;;; THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
+;;; THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS'' AND
 ;;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ;;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-;;; ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
+;;; ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE
 ;;; FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 ;;; DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 ;;; OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
@@ -73,6 +73,7 @@
 
 ;; "SELECT" on SL-5500 (center of round cursor keys)
 (define-key zaurus-select-key? "F30")
+(define-key zaurus-shift-select-key? "<Shift>F30")
 
 ;; Fn + "Katakana/Hiragana" ("Kigou")
 (define-key zaurus-kigou-key? "F31")
@@ -113,7 +114,7 @@
 ;; Fn + 8 ("half-width Katakana")
 (define-key zaurus-half-katakana-mode-key? "Private8")
 
-;; Fn + 9 ("wide-width Alphanumeric")
+;; Fn + 9 ("fullwidth Alphanumeric")
 (define-key zaurus-wide-latin-mode-key? "Private9")
 
 ;; Fn + 0 ("Alphanumeric")
@@ -158,24 +159,30 @@
 
 
 ;; save original definitions
-(define default-modifier-key? modifier-key?)
-(define default-generic-commit-key? generic-commit-key?)
-(define default-generic-cancel-key? generic-cancel-key?)
+(define zaurus-orig-modifier-key? modifier-key?)
+(define zaurus-orig-generic-begin-conv-key? generic-begin-conv-key?)
+(define zaurus-orig-generic-next-candidate-key? generic-next-candidate-key?)
+(define zaurus-orig-generic-commit-key? generic-commit-key?)
+(define zaurus-orig-generic-cancel-key? generic-cancel-key?)
+
+(define-key zaurus-begin-conv-key?
+  '(zaurus-orig-generic-begin-conv-key? zaurus-select-key?))
+(define-key zaurus-next-candidate-key?
+  '(zaurus-orig-generic-next-candidate-key? zaurus-select-key?))
 
 ;; "OK" key is not included to avoid accidential termination of application
-(define-key zaurus-commit-key? 'default-generic-commit-key?)
-     
-;; "CANCEL" key is already bound as 'escape in generic-cancel-key?, so
-;; ignore "CANCEL" key to avoid accidential termination of application
+(define-key zaurus-commit-key? 'zaurus-orig-generic-commit-key?)
+
+;; "CANCEL" key is already bound as 'escape("CANCEL" key in Zaurus) in
+;; generic-cancel-key?, so ignore it to avoid accidential termination
+;; of application
 (define zaurus-cancel-key?
   (lambda (key key-state)
     (or
      (and
-      (not (= key 'escape))
-      (default-generic-cancel-key? key key-state))
-     (and
-      (shift-key-mask key-state)
-      (zaurus-select-key? key key-state)))))
+      (not (eq? key 'escape))
+      (zaurus-orig-generic-cancel-key? key key-state))
+     (zaurus-shift-select-key? key key-state))))
 
 ;; additionally bind "OK" key for commit
 (define-key zaurus-barbarous-commit-key? '(zaurus-commit-key? zaurus-ok-key?))
@@ -193,11 +200,13 @@
       ('Mode_switch key-state)))))
 
 ;; ignore Fn key as modifier key
-(define-key modifier-key? '(default-modifier-key? zaurus-fn-key?))
+(define-key modifier-key? '(zaurus-orig-modifier-key? zaurus-fn-key?))
 
 ;; replace the default keybindings to fit Zaurus hardware
-;; you can use the barbarous bindings with careful key operation
+(define generic-begin-conv-key? zaurus-begin-conv-key?)
+(define generic-next-candidate-key? zaurus-next-candidate-key?)
 (define generic-commit-key? zaurus-commit-key?)
 (define generic-cancel-key? zaurus-cancel-key?)
+;; you can use the barbarous bindings with careful key operation
 ;(define generic-commit-key? zaurus-barbarous-commit-key?)
 ;(define generic-cancel-key? zaurus-barbarous-cancel-key?)
