@@ -1,6 +1,6 @@
 #!/usr/bin/env gosh
 
-;;; Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
+;;; Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -29,7 +29,7 @@
 ;;; SUCH DAMAGE.
 ;;;;
 
-;; These tests are passed at revision 4331 (new repository)
+;; These tests are passed at revision 5329 (new repository)
 
 ;; TODO:
 ;;
@@ -974,6 +974,13 @@
 		 (uim '(custom-hook-procs 'test-custom2 test-hook))))
 
   ("test custom-remove-hook"
+   (uim '(define custom-remove-hook-orig custom-remove-hook))
+   ;; Canonicalize to boolean since the pipe communication between
+   ;; uim-sh and gosh cannot treat (test-custom1 . #<closure (() 2)>)
+   ;; properly.
+   (uim '(define custom-remove-hook
+	   (lambda args
+	     (not (not (apply custom-remove-hook-orig args))))))
    ;; null
    (assert-equal ()
 		 (uim 'test-hook))
@@ -2312,12 +2319,16 @@
 		 (uim '(custom-value 'test-nr-candidate-max)))
    ;; valid value
    (assert-true  (uim-bool '(custom-prop-update-custom-handler
-			     test-context 'test-nr-candidate-max 5)))
+			     test-context 'test-nr-candidate-max "5")))
+   (assert-true  (uim-bool '(custom-prop-update-custom-handler
+			     test-context 'test-nr-candidate-max "'5")))
    (assert-equal 5
 		 (uim '(custom-value 'test-nr-candidate-max)))
    ;; invalid value is ignored
    (assert-false (uim-bool '(custom-prop-update-custom-handler
-			     test-context 'test-nr-candidate-max 25)))
+			     test-context 'test-nr-candidate-max "25")))
+   (assert-false (uim-bool '(custom-prop-update-custom-handler
+			     test-context 'test-nr-candidate-max "'25")))
    (assert-equal 5
 		 (uim '(custom-value 'test-nr-candidate-max)))))
 
