@@ -1,6 +1,6 @@
 #!/usr/bin/env gosh
 
-;;; Copyright (c) 2004-2007 uim Project http://uim.freedesktop.org/
+;;; Copyright (c) 2004-2008 uim Project http://code.google.com/p/uim/
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -29,6 +29,8 @@
 ;;; ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;;
 
+;; These tests are passed at revision 5329 (new repository)
+
 (use test.unit)
 
 (require "test/uim-test-utils")
@@ -55,7 +57,7 @@
 
    ;; When <extected> and the actual evaluated values are different,
    ;; it is countted as failure and reported.
-   (assert-equal -1
+   (assert-equal 1
 		 (uim '(- 0 -1))))
 
   ("test /"
