@@ -1,6 +1,6 @@
 #!/usr/bin/env gosh
 
-;;; Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
+;;; Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -29,7 +29,7 @@
 ;;; SUCH DAMAGE.
 ;;;;
 
-;; These tests are passed at revision 4331 (new repository)
+;; These tests are passed at revision 5329 (new repository)
 
 (use test.unit)
 
@@ -61,10 +61,14 @@
 		 (uim '(locale-new "d_DE")))
    (assert-equal '("" "" "")
 		 (uim '(locale-new "deu_DE")))
+   (assert-equal '("" "" "")
+		 (uim '(locale-new "de_de_DE")))
    (assert-equal '("" "" "")
 		 (uim '(locale-new "de_D")))
    (assert-equal '("" "" "")
 		 (uim '(locale-new "de_DEU")))
+   (assert-equal '("" "" "")
+		 (uim '(locale-new "de_de_DEU")))
    (assert-equal '("" "" "")
 		 (uim '(locale-new "_DE")))
    (assert-equal '("" "" "")
