@@ -1,6 +1,6 @@
 #!/usr/bin/env gosh
 
-;;; Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
+;;; Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -29,7 +29,7 @@
 ;;; SUCH DAMAGE.
 ;;;;
 
-;; These tests are passed at revision 4331 (new repository)
+;; These tests are passed at revision 5329 (new repository)
 
 (use test.unit)
 
@@ -49,13 +49,33 @@
   (setup
    (lambda ()
      (uim '(begin
+	     (require-extension (srfi 1))
 	     (require-module "anthy")
 	     (require-module "canna")
 	     (require-module "skk")
 	     (require-module "tcode")
-	     (set! enabled-im-list (append (delete 'm17n-en-ispell
-						   enabled-im-list)
-					   '(test-im test-im2)))
+	     ;; Disable IMs that affect the default IM selection.
+	     (define test-im-disabled-im-list '(look
+						m17n-unicode
+						m17n-rfc1345
+						m17n-en-ispell
+						m17n-latn-pre
+						m17n-latn-post
+						m17n-bopo-kbd
+						m17n-zh-quick
+						m17n-zh-cangjie
+						m17n-zh-tonepy
+						m17n-zh-tonepy-b5
+						m17n-zh-tonepy-gb
+						m17n-zh-py
+						m17n-zh-py-b5
+						m17n-zh-py-gb
+						m17n-syrc-phonetic
+						))
+	     (set! enabled-im-list
+		   (append (lset-difference eq? enabled-im-list
+					        test-im-disabled-im-list)
+			   '(test-im test-im2)))
 	     (for-each require-module installed-im-module-list)
 	     (define prev-im #f)
 	     (define prev-nr-ims (length im-list))
@@ -124,15 +144,17 @@
 		 (uim '(map im-name im-list)))
    ;; direct IM can be registered to null list
    (uim '(set! im-list ()))
-   ;; second time im-register-im for 'direct returns #f
-   (assert-false (uim-bool '(apply register-im (cons 'direct
-						     (cdr test-im-init-args)))))
+   ;; second time register-im for 'direct returns #t if im-list does not
+   ;; contain 'direct
+   (assert-true (uim-bool '(apply register-im (cons 'direct
+                                                    (cdr test-im-init-args)))))
    (assert-equal '(direct)
 		 (uim '(map im-name im-list)))
    ;; ordinary IM can be registered to null list
    (uim '(set! im-list ()))
-   ;; second time im-register-im for 'test-im returns #f
-   (assert-false (uim-bool '(apply register-im test-im-init-args)))
+   ;; second time register-im for 'test-im returns #t if im-list does not
+   ;; contain 'test-im
+   (assert-true  (uim-bool '(apply register-im test-im-init-args)))
    (assert-equal '(test-im)
 		 (uim '(map im-name im-list))))
 
@@ -169,12 +191,12 @@
 		   alt-set-candidate-index-handler
 		   alt-prop-activate-handler
 		   ;; replace with #f for R5RS compliant interpreter
-		   ()
-		   ()
-		   ()
-		   ()
-		   ()
-		   ())
+		   #f
+		   #f
+		   #f
+		   #f
+		   #f
+		   #f)
 		 (uim '(retrieve-im 'test-im)))
    ;; subsequent registration that has different im-name will be
    ;; registered as another IM
@@ -199,12 +221,12 @@
 		   alt-set-candidate-index-handler
 		   alt-prop-activate-handler
 		   ;; replace with #f for R5RS compliant interpreter
-		   ()
-		   ()
-		   ()
-		   ()
-		   ()
-		   ())
+		   #f
+		   #f
+		   #f
+		   #f
+		   #f
+		   #f)
 		 (uim '(retrieve-im 'test-im2))))
 
   ("test register-im (module-name)"
@@ -304,13 +326,13 @@
 		 (uim '(im-name (find-im-for-locale "pt"))))
    (uim '(unsetenv "UIM_IM_ENGINE"))
    ;; Chinese
-   (assert-equal 'm17n-zh-py
+   (assert-equal 'm17n-zh-pinyin
 		 (uim '(im-name (find-im-for-locale "zh"))))
-   (assert-equal 'm17n-zh-py
+   (assert-equal 'm17n-zh-pinyin
 		 (uim '(im-name (find-im-for-locale "zh.GB18030"))))
-   (assert-equal 'm17n-zh-py
+   (assert-equal 'm17n-zh-pinyin
 		 (uim '(im-name (find-im-for-locale "zh.Big5"))))
-   (assert-equal 'm17n-zh-py
+   (assert-equal 'm17n-zh-pinyin
 		 (uim '(im-name (find-im-for-locale "zh.UTF-8"))))
    (assert-equal 'direct
 		 (uim '(im-name (find-im-for-locale "zh_US"))))
@@ -355,13 +377,13 @@
    (assert-equal 'byeoru
 		 (uim '(im-name (find-im-for-locale "ko.UTF-8"))))
    ;; Vietnamese
-   (assert-equal 'm17n-vi-viqr
+   (assert-equal 'm17n-vi-vni
 		 (uim '(im-name (find-im-for-locale "vi"))))
-   (assert-equal 'm17n-vi-viqr
+   (assert-equal 'm17n-vi-vni
 		 (uim '(im-name (find-im-for-locale "vi_VN"))))
-   (assert-equal 'm17n-vi-viqr
+   (assert-equal 'm17n-vi-vni
 		 (uim '(im-name (find-im-for-locale "vi_VN.UTF-8"))))
-   (assert-equal 'm17n-vi-viqr
+   (assert-equal 'm17n-vi-vni
 		 (uim '(im-name (find-im-for-locale "vi.UTF-8"))))
    ;; native locale
    (uim '(unsetenv "LC_ALL"))
@@ -441,8 +463,10 @@
 		 (uim '(im-name (find-default-im "ja_JP.EUC-JP")))))
 
   ("test find-im"
-   ;; explicit IM specification
    (uim '(unsetenv "UIM_IM_ENGINE"))
+   (uim '(unsetenv "LANG"))
+   (uim '(unsetenv "LC_ALL"))
+   ;; explicit IM specification
    (uim '(set! default-im-name #f))
    (assert-equal 'direct
 		 (uim '(im-name (find-im 'direct #f))))
@@ -698,16 +722,15 @@
 				      (im-new 4 (retrieve-im 'anthy))))
 	     #t))))
      
-
-  ("test context-id"
+  ("test context-uc"
    (assert-equal 1
-		 (uim '(context-id (nth 0 context-list))))
+		 (uim '(context-uc (nth 0 context-list))))
    (assert-equal 2
-		 (uim '(context-id (nth 1 context-list))))
+		 (uim '(context-uc (nth 1 context-list))))
    (assert-equal 3
-		 (uim '(context-id (nth 2 context-list))))
+		 (uim '(context-uc (nth 2 context-list))))
    (assert-equal 4
-		 (uim '(context-id (nth 3 context-list)))))
+		 (uim '(context-uc (nth 3 context-list)))))
 
   ("test context-im"
    (assert-equal 'tcode
@@ -719,75 +742,65 @@
    (assert-equal 'anthy
 		 (uim '(im-name (context-im (nth 3 context-list))))))
    
-  ("test find-context"
-   (assert-equal 'tcode
-		 (uim '(im-name (context-im (find-context 1)))))
-   (assert-equal 'direct
-		 (uim '(im-name (context-im (find-context 2)))))
-   (assert-equal 'skk
-		 (uim '(im-name (context-im (find-context 3)))))
-   (assert-equal 'anthy
-		 (uim '(im-name (context-im (find-context 4))))))
-
   ("test remove-context"
    (assert-equal 4
 		 (uim '(length context-list)))
 
-   (uim '(begin (remove-context 3) #t))
+   (uim '(begin (remove-context (assv 3 context-list)) #t))
    (assert-equal 3
 		 (uim '(length context-list)))
    (assert-equal 'tcode
-		 (uim '(im-name (context-im (find-context 1)))))
+		 (uim '(im-name (context-im (assv 1 context-list)))))
    (assert-equal 'direct
-		 (uim '(im-name (context-im (find-context 2)))))
-   (assert-false (uim-bool '(find-context 3)))
+		 (uim '(im-name (context-im (assv 2 context-list)))))
+   (assert-false (uim-bool '(assv 3 context-list)))
    (assert-equal 'anthy
-		 (uim '(im-name (context-im (find-context 4)))))
+		 (uim '(im-name (context-im (assv 4 context-list)))))
 
-   (uim '(begin (remove-context 1) #t))
+   (uim '(begin (remove-context (assv 1 context-list)) #t))
    (assert-equal 2
 		 (uim '(length context-list)))
-   (assert-false (uim-bool '(find-context 1)))
+   (assert-false (uim-bool '(assv 1 context-list)))
    (assert-equal 'direct
-		 (uim '(im-name (context-im (find-context 2)))))
-   (assert-false (uim-bool '(find-context 3)))
+		 (uim '(im-name (context-im (assv 2 context-list)))))
+   (assert-false (uim-bool '(assv 3 context-list)))
    (assert-equal 'anthy
-		 (uim '(im-name (context-im (find-context 4)))))
+		 (uim '(im-name (context-im (assv 4 context-list)))))
 
    ;; test excessive removal
-   (uim '(begin (remove-context 1) #t))
+   (uim '(begin (remove-context (assv 1 context-list)) #t))
    (assert-equal 2
 		 (uim '(length context-list)))
-   (assert-false (uim-bool '(find-context 1)))
+   (assert-false (uim-bool '(assv 1 context-list)))
    (assert-equal 'direct
-		 (uim '(im-name (context-im (find-context 2)))))
-   (assert-false (uim-bool '(find-context 3)))
+		 (uim '(im-name (context-im (assv 2 context-list)))))
+   (assert-false (uim-bool '(assv 3 context-list)))
    (assert-equal 'anthy
-		 (uim '(im-name (context-im (find-context 4)))))
+		 (uim '(im-name (context-im (assv 4 context-list)))))
 
-   (uim '(begin (remove-context 4) #t))
+   (uim '(begin (remove-context (assv 4 context-list)) #t))
    (assert-equal 1
 		 (uim '(length context-list)))
-   (assert-false (uim-bool '(find-context 1)))
+   (assert-false (uim-bool '(assv 1 context-list)))
    (assert-equal 'direct
-		 (uim '(im-name (context-im (find-context 2)))))
-   (assert-false (uim-bool '(find-context 3)))
-   (assert-false (uim-bool '(find-context 4)))
+		 (uim '(im-name (context-im (assv 2 context-list)))))
+   (assert-false (uim-bool '(assv 3 context-list)))
+   (assert-false (uim-bool '(assv 4 context-list)))
 
-   (uim '(begin (remove-context 2) #t))
+   (uim '(begin (remove-context (assv 2 context-list)) #t))
    (assert-true  (uim-bool '(null? context-list)))
-   (assert-false (uim-bool '(find-context 1)))
-   (assert-false (uim-bool '(find-context 2)))
-   (assert-false (uim-bool '(find-context 3)))
-   (assert-false (uim-bool '(find-context 4)))
+   (assert-false (uim-bool '(assv 1 context-list)))
+   (assert-false (uim-bool '(assv 2 context-list)))
+   (assert-false (uim-bool '(assv 3 context-list)))
+   (assert-false (uim-bool '(assv 4 context-list)))
 
    ;; test exessive removal
-   (uim '(begin (remove-context 1)))
+   (uim '(begin (remove-context (assv 1 context-list))))
    (assert-true  (uim-bool '(null? context-list)))
-   (assert-false (uim-bool '(find-context 1)))
-   (assert-false (uim-bool '(find-context 2)))
-   (assert-false (uim-bool '(find-context 3)))
-   (assert-false (uim-bool '(find-context 4))))
+   (assert-false (uim-bool '(assv 1 context-list)))
+   (assert-false (uim-bool '(assv 2 context-list)))
+   (assert-false (uim-bool '(assv 3 context-list)))
+   (assert-false (uim-bool '(assv 4 context-list))))
 
   ("test register-context (add as new id)"
    (assert-equal 4
@@ -798,9 +811,9 @@
    (assert-equal 5
 		 (uim '(length context-list)))
    (assert-equal 'tcode
-		 (uim '(im-name (context-im (find-context 1)))))
+		 (uim '(im-name (context-im (assv 1 context-list)))))
    (assert-equal 'tutcode
-		 (uim '(im-name (context-im (find-context 5)))))
+		 (uim '(im-name (context-im (assv 5 context-list)))))
 
    ;; sparse id must be accepted
    (uim '(begin
@@ -809,7 +822,7 @@
    (assert-equal 6
 		 (uim '(length context-list)))
    (assert-equal 'hangul2
-		 (uim '(im-name (context-im (find-context 10)))))
+		 (uim '(im-name (context-im (assv 10 context-list)))))
 
    ;; additional sparse id
    (uim '(begin
@@ -818,7 +831,7 @@
    (assert-equal 7
 		 (uim '(length context-list)))
    (assert-equal 'hangul3
-		 (uim '(im-name (context-im (find-context 8)))))
+		 (uim '(im-name (context-im (assv 8 context-list)))))
 
    ;; decrimented id
    (uim '(begin
@@ -827,7 +840,7 @@
    (assert-equal 8
 		 (uim '(length context-list)))
    (assert-equal 'romaja
-		 (uim '(im-name (context-im (find-context 0))))))
+		 (uim '(im-name (context-im (assv 0 context-list))))))
 
   ("test register-context (duplicate id)"
    (assert-equal 4
