@@ -1,6 +1,6 @@
 #!/usr/bin/env gosh
 
-;;; Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
+;;; Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -29,7 +29,7 @@
 ;;; SUCH DAMAGE.
 ;;;;
 
-;; These tests are passed at revision 4331 (new repository)
+;; These tests are passed at revision 5329 (new repository)
 
 (use test.unit)
 
@@ -38,6 +38,7 @@
 (define-uim-test-case "testcase uim-util"
   (setup
    (lambda ()
+     (uim '(require "util.scm"))
      (uim '(define lst '(1 2 3 4)))))
 
   ("test getenv"
@@ -66,7 +67,47 @@
    (assert-true  (uim-bool '(unsetenv "UIM_NONEXISTING_ENV")))
    (assert-false (uim-bool '(getenv "UIM_NONEXISTING_ENV"))))
 
-  ("test string-split"
+  ;; See "Specification changes of utility procedures" of doc/COMPATIBILITY
+;;  ("test string-split (uim 1.4)"
+;;   ;; ordinary split
+;;   (assert-equal '("h" "geh" "ge")
+;;		 (uim '(string-split "hogehoge" "o")))
+;;   ;; case sensitive
+;;   (assert-equal '("hogehoge")
+;;		 (uim '(string-split "hogehoge" "O")))
+;;   ;; split by sequence
+;;   (assert-equal '("h" "eh" "e")
+;;		 (uim '(string-split "hogehoge" "og")))
+;;   ;; split by first character
+;;   (assert-equal '("oge" "oge")
+;;		 (uim '(string-split "hogehoge" "h")))
+;;   ;; split by first sequence
+;;   (assert-equal '("ge" "ge")
+;;		 (uim '(string-split "hogehoge" "ho")))
+;;   ;; split by last character
+;;   (assert-equal '("hog" "hog")
+;;		 (uim '(string-split "hogehoge" "e")))
+;;   ;; split by last sequence
+;;   (assert-equal '("ho" "ho")
+;;		 (uim '(string-split "hogehoge" "ge")))
+;;   ;; split by whole string
+;;   (assert-equal ()
+;;		 (uim '(string-split "hogehoge" "hogehoge")))
+;;   ;; repeated splitter
+;;   (assert-equal ()
+;;		 (uim '(string-split "hhh" "h")))
+;;   ;; split by space
+;;   (assert-equal '("h" "o" "g" "e" "hoge")
+;;		 (uim '(string-split " h o g e hoge" " ")))
+;;   ;; split by symbolic character
+;;   (assert-equal '("h" "o" "g" "e" "hoge")
+;;		 (uim '(string-split "|h|o|g|e|hoge" "|")))
+;;   ;; split by non existent character
+;;   (assert-equal '("hogehoge")
+;;		 (uim '(string-split "hogehoge" "|"))))
+
+  ;; See "Specification changes of utility procedures" of doc/COMPATIBILITY
+  ("test string-split (uim 1.5)"
    ;; ordinary split
    (assert-equal '("h" "geh" "ge")
 		 (uim '(string-split "hogehoge" "o")))
@@ -77,25 +118,28 @@
    (assert-equal '("h" "eh" "e")
 		 (uim '(string-split "hogehoge" "og")))
    ;; split by first character
-   (assert-equal '("oge" "oge")
+   (assert-equal '("" "oge" "oge")
 		 (uim '(string-split "hogehoge" "h")))
    ;; split by first sequence
-   (assert-equal '("ge" "ge")
+   (assert-equal '("" "ge" "ge")
 		 (uim '(string-split "hogehoge" "ho")))
    ;; split by last character
-   (assert-equal '("hog" "hog")
+   (assert-equal '("hog" "hog" "")
 		 (uim '(string-split "hogehoge" "e")))
    ;; split by last sequence
-   (assert-equal '("ho" "ho")
+   (assert-equal '("ho" "ho" "")
 		 (uim '(string-split "hogehoge" "ge")))
    ;; split by whole string
-   (assert-equal ()
+   (assert-equal '("" "")
 		 (uim '(string-split "hogehoge" "hogehoge")))
+   ;; repeated splitter
+   (assert-equal '("" "" "" "")
+		 (uim '(string-split "hhh" "h")))
    ;; split by space
-   (assert-equal '("h" "o" "g" "e" "hoge")
+   (assert-equal '("" "h" "o" "g" "e" "hoge")
 		 (uim '(string-split " h o g e hoge" " ")))
    ;; split by symbolic character
-   (assert-equal '("h" "o" "g" "e" "hoge")
+   (assert-equal '("" "h" "o" "g" "e" "hoge")
 		 (uim '(string-split "|h|o|g|e|hoge" "|")))
    ;; split by non existent character
    (assert-equal '("hogehoge")
@@ -124,6 +168,17 @@
 ;		 (uim '(string-to-list "a日b本語c")))
    )
 
+  ("test string-contains"
+   (assert-equal 0 (uim '(string-contains ""         "" 0)))
+   (assert-false   (uim '(string-contains ""         "f" 0)))
+   (assert-equal 0 (uim '(string-contains "foo"      "" 0)))
+   (assert-equal 0 (uim '(string-contains "foo"      "f" 0)))
+   (assert-equal 1 (uim '(string-contains "foo"      "o" 0)))
+   (assert-equal 1 (uim '(string-contains "foo"      "oo" 0)))
+   (assert-false   (uim '(string-contains "foo"      "oof" 0)))
+   (assert-equal 1 (uim '(string-contains "foo"      "o" 1)))
+   (assert-equal 2 (uim '(string-contains "foo"      "o" 2))))
+
   ("test string-prefix?"
    (assert-true  (uim-bool '(string-prefix? ""         "foo_bar")))
    (assert-true  (uim-bool '(string-prefix? "f"        "foo_bar")))
@@ -134,16 +189,16 @@
    (assert-true  (uim-bool '(string-prefix? "foo_ba"   "foo_bar")))
    (assert-true  (uim-bool '(string-prefix? "foo_bar"  "foo_bar")))
    (assert-false (uim-bool '(string-prefix? "foo_bar_" "foo_bar")))
-   (assert-false (uim-bool '(string-prefix? #f         "foo_bar")))
-   (assert-false (uim-bool '(string-prefix? "foo_bar"  #f)))
+   (assert-error (lambda () (uim-bool '(string-prefix? #f         "foo_bar"))))
+   (assert-error (lambda () (uim-bool '(string-prefix? "foo_bar"  #f))))
    (assert-false (uim-bool '(string-prefix? "Foo"      "foo_bar")))
    (assert-false (uim-bool '(string-prefix? "oo_"      "foo_bar")))
    (assert-false (uim-bool '(string-prefix? "bar"      "foo_bar")))
 
    (assert-true  (uim-bool '(string-prefix? ""    "")))
    (assert-false (uim-bool '(string-prefix? "foo" "")))
-   (assert-false (uim-bool '(string-prefix? #f    "")))
-   (assert-false (uim-bool '(string-prefix? ""    #f))))
+   (assert-error (lambda () (uim-bool '(string-prefix? #f    ""))))
+   (assert-error (lambda () (uim-bool '(string-prefix? ""    #f)))))
 
   ("test string-prefix-ci?"
    (assert-true  (uim-bool '(string-prefix-ci? ""         "foo_bar")))
@@ -155,8 +210,8 @@
    (assert-true  (uim-bool '(string-prefix-ci? "foo_ba"   "foo_bar")))
    (assert-true  (uim-bool '(string-prefix-ci? "foo_bar"  "foo_bar")))
    (assert-false (uim-bool '(string-prefix-ci? "foo_bar_" "foo_bar")))
-   (assert-false (uim-bool '(string-prefix-ci? #f         "foo_bar")))
-   (assert-false (uim-bool '(string-prefix-ci? "foo_bar"  #f)))
+   (assert-error (lambda () (uim-bool '(string-prefix-ci? #f         "foo_bar"))))
+   (assert-error (lambda () (uim-bool '(string-prefix-ci? "foo_bar"  #f))))
    (assert-true  (uim-bool '(string-prefix-ci? "Foo"      "foo_bar")))
    (assert-true  (uim-bool '(string-prefix-ci? "fOo"      "foo_bar")))
    (assert-true  (uim-bool '(string-prefix-ci? "fOO"      "foo_bar")))
@@ -167,8 +222,8 @@
 
    (assert-true  (uim-bool '(string-prefix-ci? ""    "")))
    (assert-false (uim-bool '(string-prefix-ci? "foo" "")))
-   (assert-false (uim-bool '(string-prefix-ci? #f    "")))
-   (assert-false (uim-bool '(string-prefix-ci? ""    #f))))
+   (assert-error (lambda () (uim-bool '(string-prefix-ci? #f    ""))))
+   (assert-error (lambda () (uim-bool '(string-prefix-ci? ""    #f)))))
 
   ("test string=?"
    (assert-true  (uim-bool '(string=? "foo1" "foo1")))
@@ -194,10 +249,11 @@
 		 (uim '(nthcdr 3 lst)))
    (assert-equal ()
 		 (uim '(nthcdr 4 lst)))
-   (assert-equal ()
+   (assert-equal #f
 		 (uim '(nthcdr 5 lst))))
 
   ("test charcode->string"
+   (assert-equal ""   (uim '(charcode->string 'return)))
    (assert-equal ""   (uim '(charcode->string 0)))
    (assert-equal "\n" (uim '(charcode->string 10)))
    (assert-equal "\r" (uim '(charcode->string 13)))
@@ -275,70 +331,6 @@
    (assert-equal "134217727" (uim '(digit->string 134217727)))
    )
 
-  ("test iterate-lists"
-   ;; single list cases (fast path)
-   (assert-equal '(4 3 2 1 0)
-		 (uim '(iterate-lists (lambda (state elms)
-					(if (null? elms)
-					    (cons #t state)
-					    (cons #f (cons (car elms)
-							   state))))
-				      ()
-				      '((0 1 2 3 4)))))
-   (assert-equal '()
-		 (uim '(iterate-lists (lambda (state elms)
-					(if (null? elms)
-					    (cons #t state)
-					    (cons #f (cons (car elms)
-							   state))))
-				      ()
-				      '(()))))
-   (assert-equal 'empty
-		 (uim '(iterate-lists (lambda (state elms)
-					(if (null? elms)
-					    (cons #t state)
-					    (cons #f (cons (car elms)
-							   state))))
-				      'empty
-				      '(()))))
-   ;; multiple lists cases (normal path)
-   (assert-equal '(("o" . "O") ("l" . "L") ("l" . "L") ("e" . "E") ("h" . "H"))
-		 (uim '(iterate-lists (lambda (state elms)
-					(if (null? elms)
-					    (cons #t state)
-					    (cons #f (cons (apply cons elms)
-							   state))))
-				      ()
-				      '(("h" "e" "l" "l" "o")
-					("H" "E" "L" "L" "O" "!")))))
-   (assert-equal '(("o" "O" 4) ("l" "L" 3) ("l" "L" 2) ("e" "E" 1) ("h" "H" 0))
-		 (uim '(iterate-lists (lambda (state elms)
-					(if (null? elms)
-					    (cons #t state)
-					    (cons #f (cons elms state))))
-				      ()
-				      '(("h" "e" "l" "l" "o")
-					("H" "E" "L" "L" "O" "!")
-					(0 1 2 3 4)))))
-   (assert-equal ()
-		 (uim '(iterate-lists (lambda (state elms)
-					(if (null? elms)
-					    (cons #t state)
-					    (cons #f (cons elms state))))
-				      ()
-				      '(("h" "e" "l" "l" "o")
-					()
-					(0 1 2 3 4)))))
-   (assert-equal 'empty
-		 (uim '(iterate-lists (lambda (state elms)
-					(if (null? elms)
-					    (cons #t state)
-					    (cons #f (cons elms state))))
-				      'empty
-				      '(("h" "e" "l" "l" "o")
-					()
-					(0 1 2 3 4))))))
-
   ;; compare string sequence
   ("test str-seq-equal?"
    (assert-true  (uim-bool '(str-seq-equal? () ())))
