@@ -1,4 +1,4 @@
-;;; Copyright (c) 2004-2007 uim Project http://uim.freedesktop.org/
+;;; Copyright (c) 2004-2008 uim Project http://code.google.com/p/uim/
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -41,7 +41,12 @@
 (if (version<? *gaunit-version* "0.1.1")
     (error "GaUnit 0.1.1 is required"))
 
+(sys-putenv "LIBUIM_SYSTEM_SCM_FILES" (string-append (sys-realpath ".")
+						     "/sigscheme/lib"))
 (sys-putenv "LIBUIM_SCM_FILES" (string-append (sys-realpath ".") "/scm"))
+;; FIXME: '.libs' is hardcoded
+(sys-putenv "LIBUIM_PLUGIN_LIB_DIR"
+	    (string-append (sys-realpath ".") "/uim/.libs"))
 (sys-putenv "LIBUIM_VERBOSE" "2")  ;; must be 1 or 2 (2 enables backtrace)
 (sys-putenv "LIBUIM_VANILLA" "1")
 
@@ -100,7 +105,7 @@
   (uim-sh-read (process-output *uim-sh-process*)))
 
 (define (uim-bool sexp)
-  (not (null? (uim sexp))))
+  (not (not (uim sexp))))
 
 ;; only the tricky tests require this 'require' emulation.
 (define (uim-define-siod-compatible-require)
@@ -119,13 +124,13 @@
       #t)))
 
 (eval
- (if (version>=? *gaunit-version* "0.0.6")
    '(begin
       (define (*uim-sh-setup-proc*)
         (set! *uim-sh-process* (run-process "uim/uim-sh"
                                             "-b"
                                             :input :pipe
-                                            :output :pipe)))
+                                            :output :pipe))
+	(uim '(%%set-current-error-port! (current-output-port))))
       (define (*uim-sh-teardown-proc*)
         (close-input-port (process-input *uim-sh-process*))
         (set! *uim-sh-process* #f))
@@ -139,66 +144,6 @@
              (define-test-case arg ...)
              (gaunit-delete-default-setup-proc! *uim-sh-setup-proc*)
              (gaunit-delete-default-teardown-proc! *uim-sh-teardown-proc*))))))
-
-   '(begin
-      (define (**default-test-suite**)
-        (with-module test.unit *default-test-suite*))
-      (define <test-case>
-        (with-module test.unit <test-case>))
-      (define make-tests
-        (with-module test.unit make-tests))
-      (define add-test-case!
-        (with-module test.unit add-test-case!))
-
-      (define (make-uim-sh-setup-proc . args)
-        (let-optionals* args ((additional-setup-proc (lambda () #f)))
-          (lambda ()
-            (set! *uim-sh-process* (run-process "uim/uim-sh"
-                                                "-b"
-                                                :input :pipe
-                                                :output :pipe))
-            (additional-setup-proc))))
-
-      (define (make-uim-sh-teardown-proc . args)
-        (let-optionals* args ((additional-teardown-proc (lambda () #f)))
-          (lambda ()
-            (close-input-port (process-input *uim-sh-process*))
-            (set! *uim-sh-process* #f)
-            (additional-teardown-proc))))
-
-      (define-syntax define-uim-test-case
-        (syntax-rules ()
-          ((_ name) #f)
-          ((_ name rest ...)
-           (add-test-case! (**default-test-suite**)
-                           (make-uim-test-case name rest ...)))))
-
-      (define-syntax make-uim-test-case
-        (syntax-rules (setup teardown)
-          ((_ name (setup setup-proc) (teardown teardown-proc) test ...)
-           (make <test-case>
-             :name name
-             :setup (make-uim-sh-setup-proc setup-proc)
-             :teardown (make-uim-sh-teardown-proc teardown-proc)
-             :tests (make-tests test ...)))
-          ((_ name (setup proc) test ...)
-           (make <test-case>
-             :name name
-             :setup (make-uim-sh-setup-proc proc)
-             :teardown (make-uim-sh-teardown-proc)
-             :tests (make-tests test ...)))
-          ((_ name (teardown proc) test ...)
-           (make <test-case>
-             :name name
-             :setup (make-uim-sh-setup-proc)
-             :teardown (make-uim-sh-teardown-proc proc)
-             :tests (make-tests test ...)))
-          ((_ name test ...)
-           (make <test-case>
-             :name name
-             :setup (make-uim-sh-setup-proc)
-             :teardown (make-uim-sh-teardown-proc)
-             :tests (make-tests test ...)))))))
  (current-module))
 
 (provide "test/uim-test-utils")
