@@ -3,7 +3,7 @@
  */
 /*
 
-  Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -35,9 +35,11 @@
 */
 #include <stdio.h>
 #include <ctype.h>
-#include "uim.h"
 #include <stdlib.h>
 
+#include "uim.h"
+#include "uim-util.h"
+
 static struct agent_context {
   uim_context uc;
   struct agent_context *next;
@@ -100,7 +102,7 @@ main(int argc, char **argv)
   while (1) {
     char buf[32];
     fgets(buf, 32, stdin);
-    if (isalpha((int)buf[0])) {
+    if (isalpha((unsigned char)buf[0])) {
       uim_press_key(ac->uc, buf[0], 0);
     } else {
       uim_press_key(ac->uc, UKey_Return, 0);
