@@ -120,8 +120,10 @@ init_canna_lib(uim_lisp str_)
    * initialization is exist at beginning of create_context(). I don't
    * know why this sequence is needed at here.  -- YamaKen 2007-07-21 */
   if (rk_initialized == -1) {
-    if (RkInitialize(cannaserver) == ERR)
-      uim_fatal_error("RkInitialize() failed");
+    if (RkInitialize(cannaserver) == ERR) {
+      uim_notify_fatal("RkInitialize() failed");
+      return uim_scm_f();
+    }
     RkFinalize();
   }
 
@@ -138,8 +140,10 @@ create_context(void)
   int buflen, i;
 
   if (rk_initialized == -1) {
-    if (RkInitialize(cannaserver) == ERR)
-      uim_fatal_error("RkInitialize() failed");
+    if (RkInitialize(cannaserver) == ERR) {
+      uim_notify_fatal("RkInitialize() failed");
+      return uim_scm_f();
+    }
     rk_initialized = 1;
   }
 
@@ -147,7 +151,10 @@ create_context(void)
   cc->rk_context_id = RkCreateContext();
   if (cc->rk_context_id == ERR) {
     free(cc);
-    uim_fatal_error("RkCreateContext() failed");
+    RkFinalize();
+    rk_initialized = -1;
+    uim_notify_fatal("RkCreateContext() failed");
+    return uim_scm_f();
   }    
 
   cc->rk_mode = (RK_XFER << RK_XFERBITS) | RK_KFER;
@@ -209,7 +216,7 @@ release_context(uim_lisp cc_)
   free(cc);
 
   if (err)
-    uim_fatal_error("canna-lib-release-context failed");
+    uim_notify_fatal("canna-lib-release-context failed");
 
   return uim_scm_f();
 }
@@ -260,8 +267,10 @@ begin_conversion(uim_lisp cc_, uim_lisp str_)
   len = strlen(str);
 
   segment_num = RkBgnBun(cc->rk_context_id, (char *)str, len, mode);
-  if (segment_num == ERR)
-    uim_fatal_error("RkBgnBun() failed");
+  if (segment_num == ERR) {
+    uim_notify_fatal("RkBgnBun() failed");
+    return uim_scm_f();
+  }
 
   cc->segment_num = segment_num;
   _update_status(cc);
