@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -33,12 +33,15 @@
 
 #include <config.h>
 
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
 #include <m17n.h>
 #include "gettext.h"
+#include "uim.h"
 #include "uim-scm.h"
+#include "uim-scm-abbrev.h"
 #include "uim-util.h"
 #include "plugin.h"
 
@@ -63,6 +66,8 @@ static struct ic_ {
   int  nr_candidates;
 } *ic_array;
 
+static MInputMethod *im_instance(int nth);
+
 /* Utility function */
 static char *
 m17nlib_utf8_find_next_char(const char *p)
@@ -84,7 +89,8 @@ unused_ic_id(void)
       return i;
   }
 
-  ic_array = realloc(ic_array, sizeof(struct ic_) * (nr_input_contexts + 1));
+  ic_array = uim_realloc(ic_array,
+			 sizeof(struct ic_) * (nr_input_contexts + 1));
   ic_array[nr_input_contexts].mic = NULL;
   nr_input_contexts++;
 
@@ -92,19 +98,13 @@ unused_ic_id(void)
 }
 
 static void
-pushback_input_method(MInputMethod *im, char *lib_lang, char *name)
+pushback_input_method(MInputMethod *im, char *lang, char *name)
 {
-  const char *lang;
-  
-  if (!strcmp(lib_lang, "t"))
-    lang = "";
-  else
-    lang = lib_lang;
-
-  im_array = realloc(im_array, sizeof(struct im_) * (nr_input_methods + 1));
+  im_array = uim_realloc(im_array,
+			 sizeof(struct im_) * (nr_input_methods + 1));
   im_array[nr_input_methods].im = im;
-  im_array[nr_input_methods].name = strdup(name);
-  im_array[nr_input_methods].lang = strdup(lang);
+  im_array[nr_input_methods].name = uim_strdup(name);
+  im_array[nr_input_methods].lang = uim_strdup(lang);
 
   nr_input_methods++;
 }
@@ -199,15 +199,19 @@ init_m17nlib()
   }
 
   for (elm = imlist; mplist_key(elm) != Mnil; elm = mplist_next(elm)) {
-    MDatabase *mdb = mplist_value(elm);
-    MSymbol *tag = mdatabase_tag(mdb); /* tag[1]: lang, tag[2]: name */
+    MDatabase *mdb;
+    MSymbol *tag, lang, imname;
+    uim_bool is_complete_im;
 
-    if (tag[1] != Mnil) {
-      MInputMethod *im = minput_open_im(tag[1], tag[2], NULL);
+    mdb = mplist_value(elm);
+    tag = mdatabase_tag(mdb);
+    lang = tag[1];
+    imname = tag[2];
+    is_complete_im = (lang != Mnil && imname != Mnil);  /* [uim-ja 30] */
 
-      if (im)
-	pushback_input_method(im, msymbol_name(im->language),
-			      msymbol_name(im->name));
+    if (is_complete_im) {
+      /* pass NULL as IM to enable lazy instantiation */
+      pushback_input_method(NULL, msymbol_name(lang), msymbol_name(imname));
     }
   }
 #if 0
@@ -232,7 +236,7 @@ convert_mtext2str(MText *mtext)
   mconv_encode(converter, mtext);
   buffer_for_converter[converter->nbytes] = 0;
 
-  return strdup(buffer_for_converter);
+  return uim_strdup(buffer_for_converter);
 }
 
 static uim_lisp
@@ -241,7 +245,7 @@ compose_modep(uim_lisp id_)
   int id;
   MInputContext *ic;
 
-  id = uim_scm_c_int(id_);
+  id = C_INT(id_);
   ic = ic_array[id].mic;
 
   if (!ic)
@@ -260,7 +264,7 @@ preedit_changedp(uim_lisp id_)
   int id;
   MInputContext *ic;
 
-  id = uim_scm_c_int(id_);
+  id = C_INT(id_);
   ic = ic_array[id].mic;
 
   if (!ic)
@@ -280,14 +284,14 @@ get_left_of_cursor(uim_lisp id_)
   char *buf, *p;
   MInputContext *ic;
 
-  id = uim_scm_c_int(id_);
+  id = C_INT(id_);
   ic = ic_array[id].mic;
 
   if (!ic)
-    return uim_scm_make_str("");
+    return MAKE_STR("");
 
   if (ic->cursor_pos == 0)
-    return uim_scm_make_str("");
+    return MAKE_STR("");
 
   buf = convert_mtext2str(ic->preedit);
   p = buf;
@@ -297,7 +301,7 @@ get_left_of_cursor(uim_lisp id_)
   *p = '\0';
 
   buflen = strlen(buf);
-  buf_ = uim_scm_make_str(buf);
+  buf_ = MAKE_STR(buf);
   free(buf);
 
   return buf_;
@@ -311,11 +315,11 @@ get_right_of_cursor(uim_lisp id_)
   char *buf, *p;
   MInputContext *ic;
 
-  id = uim_scm_c_int(id_);
+  id = C_INT(id_);
   ic = ic_array[id].mic;
 
   if (!ic)
-    return uim_scm_make_str("");
+    return MAKE_STR("");
 
   buf = convert_mtext2str(ic->preedit);
   p = buf;
@@ -324,7 +328,7 @@ get_right_of_cursor(uim_lisp id_)
     p = m17nlib_utf8_find_next_char(p);
 
   buflen = strlen(p);
-  buf_ = uim_scm_make_str(p);
+  buf_ = MAKE_STR(p);
   free(buf);
 
   return buf_;
@@ -338,14 +342,14 @@ get_left_of_candidate(uim_lisp id_)
   char *buf, *p;
   MInputContext *ic;
 
-  id = uim_scm_c_int(id_);
+  id = C_INT(id_);
   ic = ic_array[id].mic;
 
   if (!ic)
-    return uim_scm_make_str("");
+    return MAKE_STR("");
 
   if (ic->candidate_from == 0)
-    return uim_scm_make_str("");
+    return MAKE_STR("");
 
   buf = convert_mtext2str(ic->preedit);
   p = buf;
@@ -355,7 +359,7 @@ get_left_of_candidate(uim_lisp id_)
   *p = '\0';
 
   buflen = strlen(buf);
-  buf_ = uim_scm_make_str(buf);
+  buf_ = MAKE_STR(buf);
   free(buf);
 
   return buf_;
@@ -369,17 +373,17 @@ get_selected_candidate(uim_lisp id_)
   char *buf, *p, *start;
   MInputContext *ic;
 
-  id = uim_scm_c_int(id_);
+  id = C_INT(id_);
   ic = ic_array[id].mic;
 
   if (!ic)
-    return uim_scm_make_str("");
+    return MAKE_STR("");
 
   buf = convert_mtext2str(ic->preedit);
   p = buf;
 
   if (!p)
-    return uim_scm_make_str("");
+    return MAKE_STR("");
 
   for (i = 0; i < ic->candidate_from ;i++)
     p = m17nlib_utf8_find_next_char(p);
@@ -390,7 +394,7 @@ get_selected_candidate(uim_lisp id_)
   *p = '\0';
 
   buflen = strlen(start);
-  buf_ = uim_scm_make_str(start);
+  buf_ = MAKE_STR(start);
   free(buf);
 
   return buf_;
@@ -404,11 +408,11 @@ get_right_of_candidate(uim_lisp id_)
   char *buf, *p;
   MInputContext *ic;
 
-  id = uim_scm_c_int(id_);
+  id = C_INT(id_);
   ic = ic_array[id].mic;
 
   if (!ic)
-    return uim_scm_make_str("");
+    return MAKE_STR("");
 
   buf = convert_mtext2str(ic->preedit);
   p = buf;
@@ -417,7 +421,7 @@ get_right_of_candidate(uim_lisp id_)
     p = m17nlib_utf8_find_next_char(p);
 
   buflen = strlen(p);
-  buf_ = uim_scm_make_str(p);
+  buf_ = MAKE_STR(p);
   free(buf);
 
   return buf_;
@@ -426,23 +430,27 @@ get_right_of_candidate(uim_lisp id_)
 static uim_lisp
 get_nr_input_methods()
 {
-  return uim_scm_make_int(nr_input_methods);
+  return MAKE_INT(nr_input_methods);
 }
 
 static uim_lisp
 get_input_method_name(uim_lisp nth_)
 {
-  int nth = uim_scm_c_int(nth_);
+  int len, nth;
+  char *name;
+  
+  nth = C_INT(nth_);
 
   if (nth < nr_input_methods) {
-    char *name = alloca(strlen(im_array[nth].name) + 20);
+    len = strlen(im_array[nth].lang) + strlen(im_array[nth].name) + 7;
+    name = alloca(len);
 
-    if (!strcmp(im_array[nth].lang, ""))
-      sprintf(name, "m17n-%s", im_array[nth].name);
+    if (!strcmp(im_array[nth].lang, "t"))
+      snprintf(name, len, "m17n-%s", im_array[nth].name);
     else
-      sprintf(name, "m17n-%s-%s", im_array[nth].lang, im_array[nth].name);
+      snprintf(name, len, "m17n-%s-%s", im_array[nth].lang, im_array[nth].name);
 
-    return uim_scm_make_str(name);
+    return MAKE_STR(name);
   }
 
   return uim_scm_f();
@@ -451,10 +459,17 @@ get_input_method_name(uim_lisp nth_)
 static uim_lisp
 get_input_method_lang(uim_lisp nth_)
 {
-  int nth = uim_scm_c_int(nth_);
+  int nth;
+  const char *lang;
 
-  if (nth < nr_input_methods)
-    return uim_scm_make_str(im_array[nth].lang);
+  nth = C_INT(nth_);
+
+  if (nth < nr_input_methods) {
+    lang = im_array[nth].lang;
+    /* "*" is wildcard language. See langgroup-covers? and
+     * find-im-for-locale. */
+    return MAKE_STR((strcmp(lang, "t") == 0) ? "*" : lang);
+  }
 
   return uim_scm_f();
 }
@@ -466,13 +481,16 @@ get_input_method_short_desc(uim_lisp nth_)
   char *str = NULL, *p;
   uim_lisp ret;
 
-  nth = uim_scm_c_int(nth_);
+  nth = C_INT(nth_);
 
   if (nth < nr_input_methods) {
     MInputMethod *im;
     MText *desc;
 
-    im = im_array[nth].im;
+    im = im_instance(nth);
+    if (!im)
+      return MAKE_STR(N_("m17n library IM open error"));
+
     desc = minput_get_description(im->language, im->name);
     if (desc) {
       int i, len;
@@ -507,10 +525,10 @@ get_input_method_short_desc(uim_lisp nth_)
     }
 
     if (str) {
-      ret = uim_scm_make_str(str);
+      ret = MAKE_STR(str);
       free(str);
     } else {
-      ret = uim_scm_make_str(N_("An input method provided by the m17n library"));
+      ret = MAKE_STR(N_("An input method provided by the m17n library"));
     }
   } else
     ret = uim_scm_f();
@@ -518,6 +536,21 @@ get_input_method_short_desc(uim_lisp nth_)
   return ret;
 }
 
+static MInputMethod *
+im_instance(int nth)
+{
+  struct im_ *ent;
+
+  if (!(0 <= nth && nth < nr_input_methods))
+    return NULL;
+
+  ent = &im_array[nth];
+  if (!ent->im)
+    ent->im = minput_open_im(msymbol(ent->lang), msymbol(ent->name), NULL);
+
+  return ent->im;
+}
+
 static MInputMethod *
 find_im_by_name(const char *name)
 {
@@ -532,13 +565,13 @@ find_im_by_name(const char *name)
   for (i = 0; i < nr_input_methods; i++) {
     char buf[100];
 
-    if (!strcmp(im_array[i].lang, ""))
+    if (!strcmp(im_array[i].lang, "t"))
       snprintf(buf, 100, "%s", im_array[i].name);
     else
       snprintf(buf, 100, "%s-%s", im_array[i].lang, im_array[i].name);
 
     if (!strcmp(im_name, buf))
-      return im_array[i].im;
+      return im_instance(i);
   }
 
   return NULL;
@@ -552,7 +585,7 @@ alloc_id(uim_lisp name_)
   MInputMethod *im;
 
   id = unused_ic_id();
-  name = uim_scm_refer_c_str(name_);
+  name = REFER_C_STR(name_);
 
   im = find_im_by_name(name);
 
@@ -562,13 +595,13 @@ alloc_id(uim_lisp name_)
   ic_array[id].old_candidates = NULL;
   ic_array[id].new_candidates = NULL;
 
-  return uim_scm_make_int(id);
+  return MAKE_INT(id);
 }
 
 static uim_lisp
 free_id(uim_lisp id_)
 {
-  int id = uim_scm_c_int(id_);
+  int id = C_INT(id_);
 
   if (id < nr_input_contexts) {
     struct ic_ *ic = &ic_array[id];
@@ -589,9 +622,9 @@ push_symbol_key(uim_lisp id_, uim_lisp key_)
   MSymbol key;
   MInputContext *ic;
 
-  id = uim_scm_c_int(id_);
+  id = C_INT(id_);
   ic = ic_array[id].mic;
-  key = msymbol(uim_scm_c_str(key_));
+  key = msymbol(C_STR(key_));
 
   if (key == Mnil)
     return uim_scm_f();
@@ -611,7 +644,7 @@ get_result(uim_lisp id_)
   MInputContext *ic;
   uim_lisp  consumed_, commit_string_;
 
-  id = uim_scm_c_int(id_);
+  id = C_INT(id_);
   ic = ic_array[id].mic;
 
   produced  = mtext();
@@ -624,10 +657,10 @@ get_result(uim_lisp id_)
 
   commit_string = convert_mtext2str(produced);
   m17n_object_unref(produced);
-  commit_string_ = uim_scm_make_str(commit_string);
+  commit_string_ = MAKE_STR(commit_string);
   free(commit_string);
 
-  return uim_scm_cons(consumed_, commit_string_);
+  return CONS(consumed_, commit_string_);
 }
 
 static uim_lisp
@@ -636,7 +669,7 @@ commit(uim_lisp id_)
   int id;
   MInputContext *ic;
 
-  id = uim_scm_c_int(id_);
+  id = C_INT(id_);
   ic = ic_array[id].mic;
 
   /* To avoid a bug of m17n-lib */
@@ -651,7 +684,7 @@ candidate_showp(uim_lisp id_)
   int id;
   MInputContext *ic;
 
-  id = uim_scm_c_int(id_);
+  id = C_INT(id_);
   ic = ic_array[id].mic;
 
   if (ic->candidate_show == 1)
@@ -708,7 +741,7 @@ fill_new_candidates(uim_lisp id_)
   char **new_cands;
   MInputContext *ic;
 
-  id = uim_scm_c_int(id_);
+  id = C_INT(id_);
   ic = ic_array[id].mic;
   cands_num = calc_cands_num(id);
 
@@ -720,7 +753,7 @@ fill_new_candidates(uim_lisp id_)
   old_cands_free(ic_array[id].old_candidates);
   ic_array[id].old_candidates = ic_array[id].new_candidates;
 
-  new_cands = malloc (cands_num * sizeof(char *) + 2);
+  new_cands = uim_malloc(cands_num * sizeof(char *) + 2);
 
   if (mplist_key(group) == Mtext) {
     for (i = 0; mplist_key(group) != Mnil; group = mplist_next(group)) {
@@ -769,7 +802,7 @@ same_candidatesp(char **old, char **new)
 static uim_lisp
 candidates_changedp(uim_lisp id_)
 {
-  int id = uim_scm_c_int(id_);
+  int id = C_INT(id_);
 
   if (!same_candidatesp(ic_array[id].old_candidates,
 			ic_array[id].new_candidates))
@@ -781,9 +814,9 @@ candidates_changedp(uim_lisp id_)
 static uim_lisp
 get_nr_candidates(uim_lisp id_)
 {
-  int id = uim_scm_c_int(id_);
+  int id = C_INT(id_);
 
-  return uim_scm_make_int(calc_cands_num(id));
+  return MAKE_INT(calc_cands_num(id));
 }
 
 static uim_lisp
@@ -791,14 +824,14 @@ get_nth_candidate(uim_lisp id_, uim_lisp nth_)
 {
   int id, nth, nr;
   
-  id = uim_scm_c_int(id_);
-  nth = uim_scm_c_int(nth_);
+  id = C_INT(id_);
+  nth = C_INT(nth_);
   nr = ic_array[id].nr_candidates;
 
   if (nr >= nth)
-    return uim_scm_make_str(ic_array[id].new_candidates[nth]);
+    return MAKE_STR(ic_array[id].new_candidates[nth]);
   else
-    return uim_scm_make_str("");
+    return MAKE_STR("");
 }
 
 static uim_lisp
@@ -807,45 +840,45 @@ get_candidate_index(uim_lisp id_)
   int id;
   MInputContext *ic;
 
-  id = uim_scm_c_int(id_);
+  id = C_INT(id_);
   ic = ic_array[id].mic;
 
-  return uim_scm_make_int(ic->candidate_index);
+  return MAKE_INT(ic->candidate_index);
 }
 
 void
 uim_plugin_instance_init(void)
 {
-  uim_scm_init_subr_0("m17nlib-lib-init", init_m17nlib);
-  uim_scm_init_subr_0("m17nlib-lib-nr-input-methods",
-		      get_nr_input_methods);
-  uim_scm_init_subr_1("m17nlib-lib-nth-input-method-lang",
-		      get_input_method_lang);
-  uim_scm_init_subr_1("m17nlib-lib-nth-input-method-name",
-		      get_input_method_name);
-  uim_scm_init_subr_1("m17nlib-lib-nth-input-method-short-desc",
-		      get_input_method_short_desc);
-  uim_scm_init_subr_1("m17nlib-lib-alloc-context", alloc_id);
-  uim_scm_init_subr_1("m17nlib-lib-free-context", free_id);
-  uim_scm_init_subr_2("m17nlib-lib-push-symbol-key", push_symbol_key);
-  uim_scm_init_subr_1("m17nlib-lib-compose-mode?", compose_modep);
-  uim_scm_init_subr_1("m17nlib-lib-preedit-changed?", preedit_changedp);
-  uim_scm_init_subr_1("m17nlib-lib-get-left-of-cursor", get_left_of_cursor);
-  uim_scm_init_subr_1("m17nlib-lib-get-right-of-cursor", get_right_of_cursor);
-  uim_scm_init_subr_1("m17nlib-lib-get-left-of-candidate",
-		      get_left_of_candidate);
-  uim_scm_init_subr_1("m17nlib-lib-get-selected-candidate",
-		      get_selected_candidate);
-  uim_scm_init_subr_1("m17nlib-lib-get-right-of-candidate",
-		      get_right_of_candidate);
-  uim_scm_init_subr_1("m17nlib-lib-get-result", get_result);
-  uim_scm_init_subr_1("m17nlib-lib-commit", commit);
-  uim_scm_init_subr_1("m17nlib-lib-candidate-show?", candidate_showp);
-  uim_scm_init_subr_1("m17nlib-lib-fill-new-candidates!", fill_new_candidates);
-  uim_scm_init_subr_1("m17nlib-lib-candidates-changed?", candidates_changedp);
-  uim_scm_init_subr_1("m17nlib-lib-get-nr-candidates", get_nr_candidates);
-  uim_scm_init_subr_2("m17nlib-lib-get-nth-candidate", get_nth_candidate);
-  uim_scm_init_subr_1("m17nlib-lib-get-candidate-index", get_candidate_index);
+  uim_scm_init_proc0("m17nlib-lib-init", init_m17nlib);
+  uim_scm_init_proc0("m17nlib-lib-nr-input-methods",
+		     get_nr_input_methods);
+  uim_scm_init_proc1("m17nlib-lib-nth-input-method-lang",
+		     get_input_method_lang);
+  uim_scm_init_proc1("m17nlib-lib-nth-input-method-name",
+		     get_input_method_name);
+  uim_scm_init_proc1("m17nlib-lib-nth-input-method-short-desc",
+		     get_input_method_short_desc);
+  uim_scm_init_proc1("m17nlib-lib-alloc-context", alloc_id);
+  uim_scm_init_proc1("m17nlib-lib-free-context", free_id);
+  uim_scm_init_proc2("m17nlib-lib-push-symbol-key", push_symbol_key);
+  uim_scm_init_proc1("m17nlib-lib-compose-mode?", compose_modep);
+  uim_scm_init_proc1("m17nlib-lib-preedit-changed?", preedit_changedp);
+  uim_scm_init_proc1("m17nlib-lib-get-left-of-cursor", get_left_of_cursor);
+  uim_scm_init_proc1("m17nlib-lib-get-right-of-cursor", get_right_of_cursor);
+  uim_scm_init_proc1("m17nlib-lib-get-left-of-candidate",
+		     get_left_of_candidate);
+  uim_scm_init_proc1("m17nlib-lib-get-selected-candidate",
+		     get_selected_candidate);
+  uim_scm_init_proc1("m17nlib-lib-get-right-of-candidate",
+		     get_right_of_candidate);
+  uim_scm_init_proc1("m17nlib-lib-get-result", get_result);
+  uim_scm_init_proc1("m17nlib-lib-commit", commit);
+  uim_scm_init_proc1("m17nlib-lib-candidate-show?", candidate_showp);
+  uim_scm_init_proc1("m17nlib-lib-fill-new-candidates!", fill_new_candidates);
+  uim_scm_init_proc1("m17nlib-lib-candidates-changed?", candidates_changedp);
+  uim_scm_init_proc1("m17nlib-lib-get-nr-candidates", get_nr_candidates);
+  uim_scm_init_proc2("m17nlib-lib-get-nth-candidate", get_nth_candidate);
+  uim_scm_init_proc1("m17nlib-lib-get-candidate-index", get_candidate_index);
 }
 
 void
