@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -35,6 +35,7 @@
 #endif
 #include <errno.h>
 #include <signal.h>
+#include <stdio.h>
 #ifdef HAVE_STRING_H
 #include <string.h>
 #endif
@@ -47,15 +48,21 @@
 
 #include "uim.h"
 #include "uim-scm.h"
+#include "uim-scm-abbrev.h"
+#include "uim-util.h"
 #include "plugin.h"
+#include "uim-notify.h"
+#include "gettext.h"
 
 #define MANA_COMMAND "mana"
 
 static FILE *mana_r;
 static FILE *mana_w;
-static int mana_pid;
+static pid_t mana_pid;
 
-static char *mana_ipc_send_command(int *pid, FILE **read_fp, FILE **write_fp, const char *str);
+static char *mana_ipc_send_command(pid_t *pid,
+				   FILE **read_fp, FILE **write_fp,
+				   const char *str);
 static uim_lisp mana_init(void);
 static uim_lisp mana_eval(uim_lisp buf_);
 static uim_lisp eucjp_string_length(uim_lisp str_);
@@ -65,11 +72,11 @@ static FILE *log;
 #endif
 
 static char *
-mana_ipc_send_command(int *pid,
-		     FILE **read_fp, FILE **write_fp,
-		     const char *str)
+mana_ipc_send_command(pid_t *pid,
+		      FILE **read_fp, FILE **write_fp,
+		      const char *str)
 {
-  char *tmp = strdup("");
+  char *tmp = uim_strdup("");
   char buf[8192];
 
   struct sigaction act, oact;
@@ -93,9 +100,9 @@ mana_ipc_send_command(int *pid,
         fgets(buf, sizeof(buf), *read_fp);
         if (buf != NULL) {
           if (strcmp(buf, "err") == 0)
-            fprintf(stderr, "mana not found\n");
+            uim_notify_fatal(N_("uim-mana: Command 'mana' not found."));
           else
-            fprintf(stderr, "%s", buf);
+            uim_notify_fatal("uim-mana: %s", buf);
 	}
       }
 
@@ -130,7 +137,7 @@ mana_ipc_send_command(int *pid,
 
   while (fgets (buf, sizeof(buf), *read_fp) != NULL) {
 
-    tmp = realloc(tmp, strlen(tmp) + strlen(buf) + 1);
+    tmp = uim_realloc(tmp, strlen(tmp) + strlen(buf) + 1);
     strcat(tmp, buf);
 
     if (strchr( buf, '\n' )) {
@@ -166,7 +173,7 @@ mana_init(void)
     fclose(mana_r);
     fclose(mana_w);
     mana_r = mana_w = NULL;
-    fprintf(stderr, "mana not found\n");
+    uim_notify_fatal(N_("uim-mana: Command 'mana' not found."));
     return uim_scm_f();
   }
   
@@ -183,10 +190,11 @@ mana_init(void)
 static uim_lisp
 mana_eval(uim_lisp buf_)
 {
-  const char *buf = uim_scm_refer_c_str(buf_);
+  const char *buf = REFER_C_STR(buf_);
   char *ret_buf;
   char *eval_buf;
   uim_lisp ret;
+  int len;
 
   if (mana_pid == 0)
     return uim_scm_f();
@@ -202,8 +210,9 @@ mana_eval(uim_lisp buf_)
   fflush(log);
 #endif
 
-  eval_buf = malloc(strlen("'") + strlen(ret_buf) + 1);
-  sprintf(eval_buf, "'%s", ret_buf);
+  len = strlen("'") + strlen(ret_buf) + 1;
+  eval_buf = uim_malloc(len);
+  snprintf(eval_buf, len, "'%s", ret_buf);
   ret = uim_scm_eval_c_string(eval_buf);
   free(ret_buf);
   free(eval_buf);
@@ -214,7 +223,7 @@ mana_eval(uim_lisp buf_)
 static uim_lisp
 eucjp_string_length(uim_lisp str_)
 {
-  const unsigned char *str = (const unsigned char *)uim_scm_refer_c_str(str_);
+  const unsigned char *str = (const unsigned char *)REFER_C_STR(str_);
   int len = strlen((const char *)str);
 
   int ascii = 0;
@@ -229,15 +238,15 @@ eucjp_string_length(uim_lisp str_)
       mbyte++;
   }
 
-  return uim_scm_make_int(ascii + (mbyte / 2));
+  return MAKE_INT(ascii + (mbyte / 2));
 }
 
 void
 uim_plugin_instance_init(void)
 {
-  uim_scm_init_subr_0("mana-lib-init", mana_init);
-  uim_scm_init_subr_1("mana-lib-eval", mana_eval);
-  uim_scm_init_subr_1("mana-lib-eucjp-string-length", eucjp_string_length);
+  uim_scm_init_proc0("mana-lib-init", mana_init);
+  uim_scm_init_proc1("mana-lib-eval", mana_eval);
+  uim_scm_init_proc1("mana-lib-eucjp-string-length", eucjp_string_length);
 }
 
 void
