@@ -3,7 +3,7 @@
 
   plugin.c: Plugin support for uim.
 
-  Copyright (c) 2004-2007 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2004-2008 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -47,16 +47,19 @@
 #include <errno.h>
 #include <pwd.h>
 
-#include "uim-stdint.h"
 #include "uim.h"
 #include "uim-scm.h"
-#include "uim-compat-scm.h"
-#include "uim-compat-scm.h"
+#include "uim-scm-abbrev.h"
 #include "plugin.h"
 #include "uim-internal.h"
+#if UIM_USE_NOTIFY
+#include "uim-notify.h"
+#endif
+#include "gettext.h"
 
 #ifndef HAVE_DLFUNC
-#define dlfunc dlsym
+#define dlfunc(handle, symbol) \
+  ((void (*)(void))(uintptr_t)dlsym((handle), (symbol)))
 #endif
 
 #define PLUGIN_PREFIX "libuim-"
@@ -69,17 +72,25 @@
 #define UIM_VLEVEL_PLUGIN 3
 
 #ifdef DEBUG
-#define DPRINTFN(n,x)  if ((n) <= uim_scm_get_verbose_level()) fprintf x;
+#define DPRINTFN(n,x)  if ((n) <= verbose_level()) fprintf x;
 #else
 #define DPRINTFN(n,x)
 #endif
 
-#if UIM_SCM_GCC4_READY_GC
+static long verbose_level(void);
 static void *plugin_unload_internal(void *uim_lisp_name);
 static void *uim_quit_plugin_internal(void *dummy);
-#endif
 
 
+static long 
+verbose_level(void)
+{
+  uim_lisp vlevel;
+
+  vlevel = uim_scm_callf("verbose", "");
+  return C_INT(vlevel);
+}
+
 static uim_lisp 
 plugin_load(uim_lisp _name)
 {
@@ -94,7 +105,7 @@ plugin_load(uim_lisp _name)
 
   size_t len;
 
-  plugin_name = uim_scm_refer_c_str(_name);
+  plugin_name = REFER_C_STR(_name);
   
   if (plugin_name == NULL) {
     return uim_scm_f();
@@ -103,15 +114,15 @@ plugin_load(uim_lisp _name)
   DPRINTFN(UIM_VLEVEL_PLUGIN, (stderr, "Searching libuim-%s.so.\n", plugin_name));
 
   for (path_cdr = lib_path;
-       !uim_scm_nullp(path_cdr);
-       path_cdr = uim_scm_cdr(path_cdr))
+       !NULLP(path_cdr);
+       path_cdr = CDR(path_cdr))
   {
     int fd;
     const char *path;
-    path_car = uim_scm_car(path_cdr);
-    path = uim_scm_refer_c_str(path_car);
+    path_car = CAR(path_cdr);
+    path = REFER_C_STR(path_car);
     len = strlen(path) + 1 + strlen(PLUGIN_PREFIX) + strlen(plugin_name)+ strlen(PLUGIN_SUFFIX) + 1;
-    plugin_lib_filename = malloc(sizeof(char) * len);
+    plugin_lib_filename = uim_malloc(sizeof(char) * len);
     snprintf(plugin_lib_filename, len, "%s/%s%s%s",
 	     path, PLUGIN_PREFIX, plugin_name, PLUGIN_SUFFIX);
     fd = open(plugin_lib_filename, O_RDONLY);
@@ -126,15 +137,15 @@ plugin_load(uim_lisp _name)
 
   DPRINTFN(UIM_VLEVEL_PLUGIN, (stderr, "Searching %s.scm.\n", plugin_name));
   for (path_cdr = scm_path;
-       !uim_scm_nullp(path_cdr);
-       path_cdr = uim_scm_cdr(path_cdr))
+       !NULLP(path_cdr);
+       path_cdr = CDR(path_cdr))
   {
     int fd;
     const char *path;
-    path_car = uim_scm_car(path_cdr);
-    path = uim_scm_refer_c_str(path_car);
+    path_car = CAR(path_cdr);
+    path = REFER_C_STR(path_car);
     len = strlen(path) + 1 + strlen(plugin_name)+ strlen(".scm") + 1;
-    plugin_scm_filename = malloc(sizeof(char) * len);
+    plugin_scm_filename = uim_malloc(sizeof(char) * len);
     snprintf(plugin_scm_filename, len, "%s/%s.scm", path, plugin_name);
     fd = open(plugin_scm_filename, O_RDONLY);
     if (fd >= 0) {
@@ -156,17 +167,17 @@ plugin_load(uim_lisp _name)
   free(plugin_lib_filename);
 
   if (library == NULL) {
-    fprintf(stderr, "load failed %s\n", dlerror());
+    uim_notify_fatal(N_("%s plugin: Load failed."), dlerror());
     free(plugin_scm_filename);
     return uim_scm_f();
   }
 
   plugin_instance_init
-    = (void (*)(void))(uintptr_t)dlfunc(library, "uim_plugin_instance_init");
+    = (void (*)(void))dlfunc(library, "uim_plugin_instance_init");
   plugin_instance_quit
-    = (void (*)(void))(uintptr_t)dlfunc(library, "uim_plugin_instance_quit");
+    = (void (*)(void))dlfunc(library, "uim_plugin_instance_quit");
   if (!plugin_instance_init) {
-    fprintf(stderr, "%s plugin init failed\n", plugin_name);
+    uim_notify_fatal(N_("%s plugin: Init failed."), plugin_name);
     free(plugin_scm_filename);
     return uim_scm_f();
   }
@@ -178,8 +189,11 @@ plugin_load(uim_lisp _name)
 
     succeeded = uim_scm_require_file(plugin_scm_filename);
     if (!succeeded) {
-      fprintf(stderr, "%s plugin subsequent %s loading failed\n",
-	      plugin_name, plugin_scm_filename);
+      /* FIXME: gettext here to expand %s in accordance with the
+       * locale for the selected notification agent. See also the TODO
+       * comment of uim-notify.h  -- YamaKen 2008-02-11 */
+      uim_notify_fatal(N_("%s plugin: Subsequent %s load failed."),
+		       plugin_name, plugin_scm_filename);
       free(plugin_scm_filename);
       return uim_scm_f();
     }
@@ -187,11 +201,11 @@ plugin_load(uim_lisp _name)
 
   {
     uim_lisp form;
-    form = uim_scm_list5(uim_scm_make_symbol("plugin-list-append"),
+    form = LIST5(MAKE_SYM("plugin-list-append"),
 		         _name,
-		         uim_scm_make_ptr(library),
-			 uim_scm_make_func_ptr(plugin_instance_init),
-			 uim_scm_make_func_ptr(plugin_instance_quit));
+		         MAKE_PTR(library),
+			 MAKE_FPTR(plugin_instance_init),
+			 MAKE_FPTR(plugin_instance_quit));
     uim_scm_eval(form);
   }
   free(plugin_scm_filename);
@@ -201,7 +215,6 @@ plugin_load(uim_lisp _name)
 
 static uim_lisp
 plugin_unload(uim_lisp _name)
-#if UIM_SCM_GCC4_READY_GC
 {
   return (uim_lisp)uim_scm_call_with_gc_ready_stack(plugin_unload_internal,
 						    (void *)_name);
@@ -209,91 +222,69 @@ plugin_unload(uim_lisp _name)
 
 static void *
 plugin_unload_internal(void *uim_lisp_name)
-#endif
 {
-#if UIM_SCM_GCC4_READY_GC
   uim_lisp _name;
-#else
-  uim_lisp stack_start;
-#endif
+  uim_lisp ret;
   void *library;
   void (*plugin_instance_quit)(void);
 
-#if UIM_SCM_GCC4_READY_GC
   _name = (uim_lisp)uim_lisp_name;
-#else
-  uim_scm_gc_protect_stack(&stack_start);
-#endif
 
-  UIM_EVAL_FSTRING1(NULL, "(plugin-list-query-library \"%s\")",
-		    uim_scm_refer_c_str(_name));
-  if (UIM_SCM_FALSEP(uim_scm_return_value()))
+  ret = uim_scm_callf("plugin-list-query-library", "o", _name);
+  if (FALSEP(ret))
     return uim_scm_f();
-  library = uim_scm_c_ptr(uim_scm_return_value());
+  library = C_PTR(ret);
 
-  UIM_EVAL_FSTRING1(NULL, "(plugin-list-query-instance-quit \"%s\")",
-		    uim_scm_refer_c_str(_name));
-  if (UIM_SCM_FALSEP(uim_scm_return_value()))
+  ret = uim_scm_callf("plugin-list-query-instance-quit", "o", _name);
+  if (FALSEP(ret))
     return uim_scm_f();
-  plugin_instance_quit = uim_scm_c_func_ptr(uim_scm_return_value());
+  plugin_instance_quit = C_FPTR(ret);
 
   (plugin_instance_quit)();
   dlclose(library);
 
-  UIM_EVAL_FSTRING1(NULL, "(plugin-list-delete \"%s\")",
-		    uim_scm_refer_c_str(_name));
+  uim_scm_callf("plugin-list-delete", "o", _name);
 
-#if UIM_SCM_GCC4_READY_GC
   return (void *)uim_scm_t();
-#else
-  uim_scm_gc_unprotect_stack(&stack_start);
-
-  return uim_scm_t();
-#endif
 }
 
 /* Called from uim_init */
 void
 uim_init_plugin(void)
 {
-  uim_scm_init_subr_1("load-plugin", plugin_load);
-  uim_scm_init_subr_1("unload-plugin", plugin_unload);
+  if (UIM_CATCH_ERROR_BEGIN())
+    return;
 
-  return;
+  uim_scm_init_proc1("load-plugin", plugin_load);
+  uim_scm_init_proc1("unload-plugin", plugin_unload);
+
+  UIM_CATCH_ERROR_END();
 }
 
 /* Called from uim_quit */
 void
 uim_quit_plugin(void)
-#if UIM_SCM_GCC4_READY_GC
 {
+  if (UIM_CATCH_ERROR_BEGIN())
+    return;
+
   uim_scm_call_with_gc_ready_stack(uim_quit_plugin_internal, NULL);
+
+  UIM_CATCH_ERROR_END();
 }
 
 static void *
 uim_quit_plugin_internal(void *dummy)
-#endif
 {
-#if !UIM_SCM_GCC4_READY_GC
-  uim_lisp stack_start;
-#endif
   uim_lisp alist, rest, entry, name;
 
-#if !UIM_SCM_GCC4_READY_GC
-  uim_scm_gc_protect_stack(&stack_start);
-#endif
-
   alist = uim_scm_eval_c_string("plugin-alist");
-  for (rest = alist; !uim_scm_nullp(rest); rest = uim_scm_cdr(rest)) {
-    entry = uim_scm_car(rest);
-    name = uim_scm_car(entry);
+  for (rest = alist; !NULLP(rest); rest = CDR(rest)) {
+    entry = CAR(rest);
+    name = CAR(entry);
 
     plugin_unload(name);
   }
 
-#if UIM_SCM_GCC4_READY_GC
   return NULL;
-#else
-  uim_scm_gc_unprotect_stack(&stack_start);
-#endif
 }
