@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -34,10 +34,12 @@
 
 #include <sys/types.h>
 #include <sys/socket.h>
+#include <sys/param.h>
 #include <sys/un.h>
 #include <netinet/in.h>
 #include <netdb.h>
 #include <signal.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
 #include <string.h>
@@ -48,22 +50,24 @@
 #include <strings.h>
 #endif
 
+#include "uim.h"
 #include "uim-scm.h"
-#include "uim-compat-scm.h"
+#include "uim-scm-abbrev.h"
 #include "plugin.h"
 #include "uim-helper.h"
+#include "uim-util.h"
+#include "uim-posix.h"
 
 #define BUFFER_SIZE (4 * 1024)
 
 static FILE *primer, *primew;
-static int prime_pid = 0;
+static pid_t prime_pid = 0;
 
 static char *prime_command = "prime";
 
-static char *prime_ud_path;
+static char prime_ud_path[MAXPATHLEN];
 static int prime_fd = -1;
 static uim_bool use_unix_domain_socket;
-#define PRIME_UNIX_SOCKET_PREFIX	"/tmp/uim-prime-"
 
 static int
 prime_init_ud(char *path)
@@ -71,10 +75,10 @@ prime_init_ud(char *path)
   int fd;
   struct sockaddr_un server;
     
-  if (!path)
+  if (path[0] == '\0')
     return -1;
 
-  bzero(&server, sizeof(server));
+  memset(&server, 0, sizeof(server));
   server.sun_family = PF_UNIX;
   strlcpy(server.sun_path, path, sizeof(server.sun_path));
 
@@ -106,28 +110,29 @@ prime_init_ud(char *path)
   return fd;
 }
 
-static char *
-prime_get_ud_path(void)
+static uim_bool
+prime_get_ud_path(char *prime_path, int len)
 {
-  char *path, *login;
-  struct passwd *pw = NULL;
-  int len;
- 
-  login = getenv("LOGNAME");
-  
-  if (!login) {
-    pw = getpwuid(getuid());
-    login = strdup(pw->pw_name);
-  }
+  if (len <= 0)
+    return UIM_FALSE;
 
-  len = strlen(login) + strlen(PRIME_UNIX_SOCKET_PREFIX) + 1;
-  path = (char *)malloc(len);
-  snprintf(path, len, PRIME_UNIX_SOCKET_PREFIX"%s", login);
+  if (!uim_get_config_path(prime_path, len, !uim_helper_is_setugid()))
+    goto path_error;
 
-  if (pw)
-    free(login);
+  if (strlcat(prime_path, "/socket", len) >= (size_t)len)
+    goto path_error;
 
-  return path;
+  if (!uim_check_dir(prime_path))
+    goto path_error;
+
+  if (strlcat(prime_path, "/uim-prime", len) >= (size_t)len)
+    goto path_error;
+
+  return UIM_TRUE;
+
+ path_error:
+  prime_path[0] = '\0';
+  return UIM_FALSE;
 }
 
 static void
@@ -135,8 +140,7 @@ clear_prime_fd()
 {
   close(prime_fd);
   prime_fd = -1;
-  free(prime_ud_path);
-  prime_ud_path = NULL;
+  prime_ud_path[0] = '\0';
 }
 
 static char *
@@ -149,15 +153,10 @@ prime_read_msg_from_ud(int fd)
   if (fd == -1)
     return NULL;
 
-  read_buf  = strdup("");
-  if (!read_buf)
-    return NULL;
+  read_buf  = uim_strdup("");
 
   for (;;) {
-    rc = read(fd, buf, sizeof(buf) - 1);
-    buf[rc] = '\0';
-    
-    if (rc <= 0) {
+    if ((rc = read(fd, buf, sizeof(buf) - 1)) == -1 || rc == 0) {
       perror("disconnected");
       if (errno == EAGAIN || errno == EINTR)
 	continue;
@@ -166,14 +165,16 @@ prime_read_msg_from_ud(int fd)
       clear_prime_fd();
       return NULL;
     }
-    read_buf = (char *)realloc(read_buf, strlen(read_buf) + strlen(buf) + 1);
-    if (!read_buf)
-      return NULL;
+    buf[rc] = '\0';
+    read_buf = uim_realloc(read_buf, strlen(read_buf) + strlen(buf) + 1);
     strcat(read_buf, buf);
     len += rc;
 
-    if (len >= 2 && read_buf[len - 1] == '\n' && read_buf[len - 2] == '\n')
+    if (len >= 2 && read_buf[len - 1] == '\n' && read_buf[len - 2] == '\n') {
+      /* drop last "\n" */
+      read_buf[len - 1] = '\0';
       break;
+    }
   }
 
   return read_buf;
@@ -194,27 +195,29 @@ prime_write_msg_to_ud(int fd, const char *message)
 static uim_lisp
 prime_send_command(uim_lisp str_)
 {
-  const char *str = uim_scm_refer_c_str(str_);
+  const char *str;
   char *result;
   uim_lisp ret;
 
+  str = REFER_C_STR(str_);
+
   if (use_unix_domain_socket) {
     prime_write_msg_to_ud(prime_fd, str);
     result = prime_read_msg_from_ud(prime_fd);
     if (!result)
-      return uim_scm_make_str("error\n\t\n\n");
+      return MAKE_STR("error\n\t\n");
   } else {
     int len = strlen(str);
-    char *buf = malloc(len + 2);
+    char *buf = uim_malloc(len + 2);
     snprintf(buf, len + 2,"%s\n", str);
     result = uim_ipc_send_command(&prime_pid, &primer, &primew, prime_command,
 				  buf);
     free(buf);
     if (!result)
-      return uim_scm_make_str("");
+      return MAKE_STR("");
   }
 
-  ret = uim_scm_make_str(result);
+  ret = MAKE_STR(result);
   free(result);
 
   return ret;
@@ -224,22 +227,23 @@ static uim_lisp
 prime_lib_init(uim_lisp use_udp_)
 {
   char *option;
+  int len, timeout_count = 0;
 
-  use_unix_domain_socket = uim_scm_c_bool(use_udp_);
+  use_unix_domain_socket = C_BOOL(use_udp_);
 
   if (use_unix_domain_socket) {
     if (prime_fd != -1)
       return uim_scm_t();
 
-    prime_ud_path = prime_get_ud_path();
-    if (!prime_ud_path)
+    if (!prime_get_ud_path(prime_ud_path, sizeof(prime_ud_path)))
       return uim_scm_f();
-      
+
     prime_fd = prime_init_ud(prime_ud_path);
     if (prime_fd == -1) {
       unlink(prime_ud_path);
-      option = malloc(strlen("-u ") + strlen(prime_ud_path) + 1);
-      sprintf(option, "-u %s", prime_ud_path);
+      len = strlen("-u ") + strlen(prime_ud_path) + 1;
+      option = uim_malloc(len);
+      snprintf(option, len, "-u %s", prime_ud_path);
       prime_pid = uim_ipc_open_command_with_option(prime_pid, &primer, &primew,
 						   prime_command, option);
       free(option);
@@ -247,8 +251,11 @@ prime_lib_init(uim_lisp use_udp_)
 	return uim_scm_f();
       else {
 	prime_fd = prime_init_ud(prime_ud_path);
-	while (prime_fd == -1)
+	while (prime_fd == -1 && timeout_count < 100) {
+	  usleep(100000);
 	  prime_fd = prime_init_ud(prime_ud_path);
+	  timeout_count++;
+	}
       }
     }
 
@@ -265,8 +272,8 @@ prime_lib_init(uim_lisp use_udp_)
 void
 uim_plugin_instance_init(void)
 {
-  uim_scm_init_subr_1("prime-lib-init", prime_lib_init);
-  uim_scm_init_subr_1("prime-lib-send-command", prime_send_command);
+  uim_scm_init_proc1("prime-lib-init", prime_lib_init);
+  uim_scm_init_proc1("prime-lib-send-command", prime_send_command);
 }
 
 void
