@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2004-2007 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2004-2008 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -42,8 +42,10 @@
 #include <scim.h>
 #include "uim.h"
 #include "uim-scm.h"
-#include "uim-compat-scm.h"
-#include "context.h"
+#include "uim-internal.h"
+#include "uim-notify.h"
+
+// FIXME: unconditional fprintf()s
 
 using namespace scim;
 
@@ -144,6 +146,16 @@ create_im_list()
     for ( ; it != factories.end(); ++it )
     {
         SCIMInputMethod *scim_im = new SCIMInputMethod();
+	// FIXME: IMEngineFactoryBase::get_name() returns the IM name
+	// for humans (and it is recommended to be 'localized'). So it
+	// does not fit the uim's symbolic IM name (idname), and
+	// causes parse errors on Scheme when being read as Scheme
+	// symbol. For example, an IM name "scim-Probhat(phonetic)"
+	// cannot be a safe Scheme symbol since it contains
+	// parenthesis. Some character replacements and
+	// anti-message-translation (localization) is required to
+	// provide uim IMs. uim rejects IMs that have invalid IM
+	// name.  -- YamaKen 2008-03-24
         scim_im->imname = (*it)->get_name();
         scim_im->lang = (*it)->get_language();
         scim_im->uuid = (*it)->get_uuid();
@@ -171,21 +183,27 @@ init_scim()
         config_module = new ConfigModule( "simple" );
         if ( !config_module )
         {
-            fprintf(stderr, "Could not create ConfigModule\n");
+            uim_notify_fatal("init_scim: Could not create ConfigModule");
             return uim_scm_f();
         }
 
+#if 0
+	/* for old versions (1.2 or older) of SCIM */
         config = config_module->create_config("scim");
+#else
+	/* for new versions (1.3, 1.4) of SCIM */
+        config = config_module->create_config();
+#endif
         if ( config.null() )
         {
-            fprintf(stderr, "create_config failed\n");
+            uim_notify_fatal("init_scim: create_config failed");
             return uim_scm_f();
         }
 
         be = new CommonBackEnd( config, engine_list );
         if ( be.null() )
         {
-            fprintf(stderr, "create CommonBackEnd failed\n");
+            uim_notify_fatal("init_scim: create CommonBackEnd failed");
             return uim_scm_f();
         }
 
@@ -197,7 +215,7 @@ init_scim()
              * If no SocketFrontend is runnig, launch a SCIM daemon.
              */
             fprintf(stderr, "launch SCIM daemon\n");
-            char *new_argv [] = { (char*)"--no-stay", 0 };
+            char *new_argv [] = { (char *)"--no-stay", 0 };
             scim_launch( true,
                          "simple", //FIX ME
                          (engine_list.size () ? scim_combine_string_list( engine_list, ',' ) : "all"),
@@ -226,7 +244,7 @@ get_input_method_lang(uim_lisp nth_)
 {
     // FIXME
     int nth = uim_scm_c_int( nth_ );
-    if ( nth < im_list.size() )
+    if ( nth < (int)im_list.size() )
     {
         return uim_scm_make_str( im_list.at( nth )->lang.c_str() );
     }
@@ -251,7 +269,7 @@ static uim_lisp
 get_input_method_name(uim_lisp nth_)
 {
     int nth = uim_scm_c_int( nth_ );
-    if ( nth < im_list.size() )
+    if ( nth < (int)im_list.size() )
     {
         // remove space
         String imname = WideStr_to_String( im_list.at( nth )->imname );
@@ -278,7 +296,7 @@ search_uuid_by_imname( String imname )
     fprintf(stderr, "search imname = %s\n", imname.c_str() );
 
     
-    std::vector<SCIMInputMethod*>::iterator it = im_list.begin();
+    std::vector<SCIMInputMethod *>::iterator it = im_list.begin();
     for ( ; it != im_list.end(); ++it )
     {
         fprintf(stderr, "im = %s\n", WideStr_to_CStr( (*it)->imname));
@@ -306,7 +324,7 @@ alloc_id( uim_lisp name_ )
     String uuid = search_uuid_by_imname( imname );
     if ( uuid.empty() )
     {
-        fprintf( stderr, "failed to search uuid\n" );
+        uim_notify_fatal( "uim-scim: failed to search uuid" );
         return uim_scm_f();
     }
     context->factory = be->get_factory( uuid );
@@ -318,7 +336,7 @@ alloc_id( uim_lisp name_ )
     context->instance = context->factory->create_instance( "UTF-8", instance_count );
     if ( context->instance.null() )
     {
-        fprintf(stderr, "failed to create IMEngineInstance\n");
+        uim_notify_fatal("uim-scim: failed to create IMEngineInstance");
         return uim_scm_f();
     }
 
@@ -337,7 +355,7 @@ alloc_id( uim_lisp name_ )
     context->instance->signal_connect_register_properties( slot( cb_prop_register) );
     context->instance->signal_connect_update_property( slot( cb_prop_update) );
     */
-    context->instance->set_frontend_data( static_cast <void*> (context) );
+    context->instance->set_frontend_data( static_cast <void *> (context) );
 
     // and store the context
     context_list.push_back( context );
@@ -357,7 +375,7 @@ free_id(uim_lisp id_)
 
 static SCIMContext *get_context_from_id(int id)
 {
-    std::vector<SCIMContext*>::iterator it = context_list.begin();
+    std::vector<SCIMContext *>::iterator it = context_list.begin();
     for ( ; it != context_list.end(); ++it )
     {
         if ( id == (*it)->id )
@@ -450,7 +468,7 @@ get_nth_candidate( uim_lisp id_, uim_lisp idx_ )
 static void cb_commit( IMEngineInstanceBase *instance, const WideString &wstr )
 {
     fprintf(stderr, "cb_commit\n");
-    SCIMContext *ic = static_cast<SCIMContext*>(instance->get_frontend_data());
+    SCIMContext *ic = static_cast<SCIMContext *>(instance->get_frontend_data());
     if ( !ic )
     {
         return;
@@ -469,7 +487,7 @@ static void cb_preedit_update( IMEngineInstanceBase *instance, const WideString
 {    
     fprintf(stderr, "cb_preedit_update : preedit_str = [%s]\n", WideStr_to_CStr(wstr));
 
-    SCIMContext *ic = static_cast<SCIMContext*>(instance->get_frontend_data());
+    SCIMContext *ic = static_cast<SCIMContext *>(instance->get_frontend_data());
     if ( !ic )
     {
         return;
@@ -486,7 +504,7 @@ static void cb_preedit_hide( IMEngineInstanceBase *instance )
 {
     fprintf(stderr, "cb_preedit_hide\n");
 
-    SCIMContext *ic = static_cast<SCIMContext*>(instance->get_frontend_data());
+    SCIMContext *ic = static_cast<SCIMContext *>(instance->get_frontend_data());
     if ( !ic )
     {
         return;
@@ -501,7 +519,7 @@ static void cb_preedit_caret( IMEngineInstanceBase *instance, int caret )
 {
     fprintf(stderr, "cb_preedit_caret\n");
 
-    SCIMContext *ic = static_cast<SCIMContext*>(instance->get_frontend_data());
+    SCIMContext *ic = static_cast<SCIMContext *>(instance->get_frontend_data());
     if ( !ic )
     {
         return;
@@ -517,7 +535,7 @@ static void cb_lookup_update( IMEngineInstanceBase *instance, const LookupTable
 {
     fprintf(stderr, "cb_lookup_update\n");
 
-    SCIMContext *ic = static_cast<SCIMContext*>(instance->get_frontend_data());
+    SCIMContext *ic = static_cast<SCIMContext *>(instance->get_frontend_data());
     if ( !ic )
     {
         return;
@@ -642,19 +660,19 @@ uim_keysymbol_to_scim_keysymbol( const char *sym, KeyEvent *key )
         char *symbol;
         unsigned int keycode;
     } keycode_map[] = {
-        {(char*)"Escape", SCIM_KEY_Escape},
-        {(char*)"Tab", SCIM_KEY_Tab},
-        {(char*)"BackSpace", SCIM_KEY_BackSpace},
-        {(char*)"Delete", SCIM_KEY_Delete},
-        {(char*)"Return", SCIM_KEY_Return},
-        {(char*)"Left", SCIM_KEY_Left},
-        {(char*)"Up", SCIM_KEY_Up},
-        {(char*)"Right", SCIM_KEY_Right},
-        {(char*)"Down", SCIM_KEY_Down},
-        {(char*)"Prior", SCIM_KEY_Prior},
-        {(char*)"Next", SCIM_KEY_Next},
-        {(char*)"Home", SCIM_KEY_Home},
-        {(char*)"End", SCIM_KEY_End},
+        {(char *)"Escape", SCIM_KEY_Escape},
+        {(char *)"Tab", SCIM_KEY_Tab},
+        {(char *)"BackSpace", SCIM_KEY_BackSpace},
+        {(char *)"Delete", SCIM_KEY_Delete},
+        {(char *)"Return", SCIM_KEY_Return},
+        {(char *)"Left", SCIM_KEY_Left},
+        {(char *)"Up", SCIM_KEY_Up},
+        {(char *)"Right", SCIM_KEY_Right},
+        {(char *)"Down", SCIM_KEY_Down},
+        {(char *)"Prior", SCIM_KEY_Prior},
+        {(char *)"Next", SCIM_KEY_Next},
+        {(char *)"Home", SCIM_KEY_Home},
+        {(char *)"End", SCIM_KEY_End},
         {NULL, 0}
     };
 
