@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003,2004,2005 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -41,7 +41,7 @@
   argument that causes no action.  -- YamaKen 2004-12-17
 */
 
-#include "config.h"
+#include <config.h>
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -49,8 +49,10 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <stdarg.h>
+#include "gettext.h"
+
 #include "uim-scm.h"
-#include "uim-compat-scm.h"
 #include "uim-custom.h"
 #include "uim-internal.h"
 #include "uim-helper.h"
@@ -75,24 +77,125 @@
 #define UIM_CUSTOM_EXPERIMENTAL_MTIME_SENSING
 #endif
 
+#define MAX_LENGTH_OF_INT_AS_STR (((sizeof(int) == 4) ? sizeof("-2147483648") : sizeof("-9223372036854775808")) - sizeof((char)'\0'))
+
+#define UGETTEXT(str) (dgettext(GETTEXT_PACKAGE, (str)))
+
+/* we cannot use the variadic macro (i.e. __VA_ARGS__) because we
+   should also support C89 compilers
+*/
+#define UIM_EVAL_STRING_INTERNAL(uc, sexp_str) \
+  (uim_scm_last_val = uim_scm_eval_c_string(sexp_str))
+
+#define UIM_EVAL_STRING(uc, sexp_str) \
+  { \
+    UIM_EVAL_STRING_INTERNAL(uc, sexp_str); \
+  }
+
+#define UIM_EVAL_FSTRING1(uc, sexp_tmpl, arg1) \
+  { \
+    int form_size; \
+    char *buf; \
+    form_size = uim_sizeof_sexp_str(sexp_tmpl, arg1); \
+    if (form_size != -1) { \
+      buf = malloc(form_size); \
+      snprintf(buf, form_size, sexp_tmpl, arg1); \
+      UIM_EVAL_STRING_INTERNAL(uc, buf); \
+      free(buf); \
+    } \
+  }
+
+#define UIM_EVAL_FSTRING2(uc, sexp_tmpl, arg1, arg2) \
+  { \
+    int form_size; \
+    char *buf; \
+    form_size = uim_sizeof_sexp_str(sexp_tmpl, arg1, arg2); \
+    if (form_size != -1) { \
+      buf = malloc(form_size); \
+      snprintf(buf, form_size, sexp_tmpl, arg1, arg2); \
+      UIM_EVAL_STRING_INTERNAL(uc, buf); \
+      free(buf); \
+    } \
+  }
+
+#define UIM_EVAL_FSTRING3(uc, sexp_tmpl, arg1, arg2, arg3) \
+  { \
+    int form_size; \
+    char *buf; \
+    form_size = uim_sizeof_sexp_str(sexp_tmpl, arg1, arg2, arg3); \
+    if (form_size != -1) { \
+      buf = malloc(form_size); \
+      snprintf(buf, form_size, sexp_tmpl, arg1, arg2, arg3); \
+      UIM_EVAL_STRING_INTERNAL(uc, buf); \
+      free(buf); \
+    } \
+  }
+
+#define UIM_EVAL_FSTRING4(uc, sexp_tmpl, arg1, arg2, arg3, arg4) \
+  { \
+    int form_size; \
+    char *buf; \
+    form_size = uim_sizeof_sexp_str(sexp_tmpl, arg1, arg2, arg3, arg4); \
+    if (form_size != -1) { \
+      buf = malloc(form_size); \
+      snprintf(buf, form_size, sexp_tmpl, arg1, arg2, arg3, arg4); \
+      UIM_EVAL_STRING_INTERNAL(uc, buf); \
+      free(buf); \
+    } \
+  }
+
+#define UIM_EVAL_FSTRING5(uc, sexp_tmpl, arg1, arg2, arg3, arg4, arg5) \
+  { \
+    int form_size; \
+    char *buf; \
+    form_size = uim_sizeof_sexp_str(sexp_tmpl, arg1, arg2, arg3, arg4, arg5); \
+    if (form_size != -1) { \
+      buf = malloc(form_size); \
+      snprintf(buf, form_size, sexp_tmpl, arg1, arg2, arg3, arg4, arg5); \
+      UIM_EVAL_STRING_INTERNAL(uc, buf); \
+      free(buf); \
+    } \
+  }
+
 typedef void (*uim_custom_cb_update_cb_t)(void *ptr, const char *custom_sym);
 typedef void (*uim_custom_global_cb_update_cb_t)(void *ptr);
 
+typedef void *(*uim_scm_c_list_conv_func)(uim_lisp elem);
+typedef void (*uim_scm_c_list_free_func)(void *elem);
+
 /* exported for internal use */
 uim_bool uim_custom_init(void);
 uim_bool uim_custom_quit(void);
 
+/* uim_scm_return_value() can only be used to retrieve result of
+ * UIM_EVAL_FSTRINGn() or UIM_EVAL_STRING(). */
+static uim_lisp uim_scm_return_value(void);
+static int uim_sizeof_sexp_str(const char *tmpl, ...);
+
+static void **uim_scm_c_list(const char *list_repl, const char *mapper_proc,
+                             uim_scm_c_list_conv_func conv_func);
+static char *uim_scm_c_str_failsafe(uim_lisp str);
+static char **uim_scm_c_str_list(const char *list_repl,
+                                 const char *mapper_proc);
+static void uim_scm_c_list_free(void **list,
+                                uim_scm_c_list_free_func free_func);
+
 static char *literalize_string(const char *str);
+static char *literalize_string_internal(const char *str);
 
 static char *c_list_to_str(const void *const *list, char *(*mapper)(const void *elem), const char *sep);
 
 static int uim_custom_type_eq(const char *custom_sym, const char *custom_type);
 static int uim_custom_type(const char *custom_sym);
 static int uim_custom_is_active(const char *custom_sym);
-static char *uim_custom_get_str(const char *custom_sym, const char *proc);
+static const char *uim_custom_get_str(const char *custom_sym,
+                                      const char *proc);
 static char *uim_custom_label(const char *custom_sym);
 static char *uim_custom_desc(const char *custom_sym);
 
+static struct uim_custom_pathname *uim_custom_pathname_get(const char *custom_sym, const char *getter_proc);
+static struct uim_custom_pathname *uim_custom_pathname_new(char *str, int type);
+static void uim_custom_pathname_free(struct uim_custom_pathname *custom_pathname);
 static struct uim_custom_choice *uim_custom_choice_get(const char *custom_sym, const char *choice_sym);
 static char *extract_choice_symbol(const struct uim_custom_choice *custom_choice);
 static char *choice_list_to_str(const struct uim_custom_choice *const *list, const char *sep);
@@ -107,7 +210,6 @@ static struct uim_custom_key **uim_custom_key_get(const char *custom_sym, const
 static void uim_custom_key_free(struct uim_custom_key *custom_key);
 static char *extract_key_literal(const struct uim_custom_key *custom_key);
 static char *key_list_to_str(const struct uim_custom_key *const *list, const char *sep);
-void uim_custom_key_list_free(struct uim_custom_key **list);
 
 static union uim_custom_value *uim_custom_value_internal(const char *custom_sym, const char *getter_proc);
 static union uim_custom_value *uim_custom_value(const char *custom_sym);
@@ -118,10 +220,19 @@ static union uim_custom_range *uim_custom_range_get(const char *custom_sym);
 static void uim_custom_range_free(int custom_type, union uim_custom_range *custom_range);
 static uim_lisp uim_custom_cb_update_cb_gate(uim_lisp cb, uim_lisp ptr, uim_lisp custom_sym);
 static uim_lisp uim_custom_global_cb_update_cb_gate(uim_lisp cb, uim_lisp ptr);
+static uim_bool custom_cb_remove(const char *key_sym, const char *hook);
 static uim_bool custom_cb_add(const char *hook, const char *validator,
 			      const char *custom_sym, void *ptr,
 			      const char *gate_func, void (*cb)(void));
-static uim_bool custom_cb_remove(const char *key_sym, const char *hook);
+struct custom_cb_add_args {
+  const char *hook;
+  const char *validator;
+  const char *custom_sym;
+  void *ptr;
+  const char *gate_func;
+  void (*cb)(void);
+};
+static void *custom_cb_add_internal(struct custom_cb_add_args *args);
 
 static void helper_disconnect_cb(void);
 static char *uim_conf_path(const char *subpath);
@@ -132,53 +243,145 @@ static uim_bool for_each_primary_groups(uim_bool (*func)(const char *));
 static uim_bool uim_custom_load_group(const char *group);
 static uim_bool uim_custom_save_group(const char *group);
 
-#if UIM_SCM_GCC4_READY_GC
-static char *literalize_string_internal(const char *str);
-static uim_bool custom_cb_add_internal(const char *hook, const char *validator,
-				       const char *custom_sym, void *ptr,
-				       const char *gate_func, void (*cb)(void));
-#endif
-
 static const char str_list_arg[] = "uim-custom-c-str-list-arg";
 static const char custom_subdir[] = "customs";
 static const char custom_msg_tmpl[] = "prop_update_custom\n%s\n%s\n";
 static int helper_fd = -1;
 static uim_lisp return_val;
+static uim_lisp uim_scm_last_val;
 
 
+static uim_lisp
+uim_scm_return_value(void)
+{
+  return uim_scm_last_val;
+}
+
+/** Calculate actual sexp string size from printf-style args.
+ * This function calculates actual sexp string size from printf-style
+ * args. Format string \a sexp_tmpl only accepts %d and %s.
+ */
+int
+uim_sizeof_sexp_str(const char *sexp_tmpl, ...)
+{
+  va_list ap;
+  int len, size;
+  int tmp;
+  const char *sexp_tmpl_end, *escp = sexp_tmpl, *strarg;
+  char fmtchr;
+
+  va_start(ap, sexp_tmpl);
+  len = strlen(sexp_tmpl);
+  sexp_tmpl_end = sexp_tmpl + len - 1;
+  while ((escp = strchr(escp, '%'))) {
+    if (escp < sexp_tmpl_end) {
+      escp += sizeof((char)'%');
+      fmtchr = *escp++;
+      switch (fmtchr) {
+      case 'd':
+	tmp = va_arg(ap, int);
+	len += MAX_LENGTH_OF_INT_AS_STR;
+	break;
+      case 's':
+	strarg = va_arg(ap, const char *);
+	len += strlen(strarg);
+	break;
+      default:
+	/* unexpected format string */
+	size = -1;
+	goto end;
+      }
+    } else {
+      /* invalid format string */
+      size = -1;
+      goto end;
+    }
+  }
+  size = len + sizeof((char)'\0');
+
+ end:
+  va_end(ap);
+
+  return size;
+}
+
+/*
+  - list_repl must always returns same list for each evaluation
+  - returns NULL terminated array. NULL will not appeared except terminator
+  - non-string element such as #f is converted to ""
+ */
+static void **
+uim_scm_c_list(const char *list_repl, const char *mapper_proc,
+	       uim_scm_c_list_conv_func conv_func)
+{
+  int list_len, i;
+  void **result;
+
+  UIM_EVAL_FSTRING1(NULL, "(length %s)", list_repl);
+  list_len = uim_scm_c_int(uim_scm_return_value());
+
+  result = (void **)malloc(sizeof(void *) * (list_len + 1));
+  if (!result)
+    return NULL;
+
+  result[list_len] = NULL;
+  for (i = 0; i < list_len; i++) {
+    UIM_EVAL_FSTRING3(NULL, "(%s (nth %d %s))", mapper_proc, i, list_repl);
+    result[i] = (*conv_func)(uim_scm_return_value());
+  }
+
+  return result;
+}
+
+static char *
+uim_scm_c_str_failsafe(uim_lisp str)
+{
+  return (uim_scm_truep(str)) ? uim_scm_c_str(str) : strdup("");
+}
+
+static char **
+uim_scm_c_str_list(const char *list_repl, const char *mapper_proc)
+{
+  void **list;
+  
+  list = uim_scm_c_list(list_repl, mapper_proc,
+			(uim_scm_c_list_conv_func)uim_scm_c_str_failsafe);
+
+  return (char **)list;
+}
+
+static void
+uim_scm_c_list_free(void **list, uim_scm_c_list_free_func free_func)
+{
+  void *elem;
+  void **p;
+
+  if (!list)
+    return;
+
+  for (p = list; *p; p++) {
+    elem = *p;
+    free_func(elem);
+  }
+  free(list);
+}
+
 static char *
 literalize_string(const char *str)
-#if UIM_SCM_GCC4_READY_GC
 {
-  char *ret;
-
-  UIM_SCM_GC_PROTECTED_CALL(ret, char *, literalize_string_internal, (str));
-
-  return ret;
+  return uim_scm_call_with_gc_ready_stack((uim_gc_gate_func_ptr)literalize_string_internal, (void *)str);
 }
 
 static char *
 literalize_string_internal(const char *str)
-#endif
 {
-#if !UIM_SCM_GCC4_READY_GC
-  uim_lisp stack_start;
-#endif
   uim_lisp form;
   char *escaped;
 
-#if !UIM_SCM_GCC4_READY_GC
-  uim_scm_gc_protect_stack(&stack_start);
-#endif
-
   form = uim_scm_list2(uim_scm_make_symbol("string-escape"),
 		       uim_scm_make_str(str));
   escaped = uim_scm_c_str(uim_scm_eval(form));
 
-#if !UIM_SCM_GCC4_READY_GC
-  uim_scm_gc_unprotect_stack(&stack_start);
-#endif
-
   return escaped;
 }
 
@@ -254,29 +457,83 @@ uim_custom_is_active(const char *custom_sym)
   return uim_scm_c_bool(return_val);
 }
 
-static char *
+static const char *
 uim_custom_get_str(const char *custom_sym, const char *proc)
 {
   UIM_EVAL_FSTRING2(NULL, "(%s '%s)", proc, custom_sym);
   return_val = uim_scm_return_value();
 
-  /*
-    The arg must be assigned to return_val to be proteced from GC
-    while evaluation
-  */
-  return uim_scm_c_str(return_val);
+  return uim_scm_refer_c_str(return_val);
 }
 
 static char *
 uim_custom_label(const char *custom_sym)
 {
-  return uim_custom_get_str(custom_sym, "custom-label");
+  const char *str;
+
+  str = uim_custom_get_str(custom_sym, "custom-label");
+  return strdup(UGETTEXT(str));
 }
 
 static char *
 uim_custom_desc(const char *custom_sym)
 {
-  return uim_custom_get_str(custom_sym, "custom-desc");
+  const char *str;
+
+  str = uim_custom_get_str(custom_sym, "custom-desc");
+  return strdup(UGETTEXT(str));
+}
+
+/* pathname */
+static struct uim_custom_pathname *
+uim_custom_pathname_get(const char *custom_sym, const char *getter_proc)
+{
+  struct uim_custom_pathname *custom_pathname;
+  char *str, *type_sym;
+  int type;
+
+  UIM_EVAL_FSTRING2(NULL, "(%s '%s)", getter_proc, custom_sym);
+  return_val = uim_scm_return_value();
+  str = uim_scm_c_str(return_val);
+
+  UIM_EVAL_FSTRING1(NULL, "(custom-pathname-type '%s)", custom_sym);
+  return_val = uim_scm_return_value();
+  type_sym = uim_scm_c_symbol(return_val);
+  if (strcmp(type_sym, "directory") == 0)
+    type = UCustomPathnameType_Directory;
+  else
+    type = UCustomPathnameType_RegularFile;
+  free(type_sym);
+
+  custom_pathname = uim_custom_pathname_new(str, type);
+  if (!custom_pathname)
+    return NULL;
+
+  return custom_pathname;
+}
+
+static struct uim_custom_pathname *
+uim_custom_pathname_new(char *str, int type)
+{
+  struct uim_custom_pathname *custom_pathname;
+
+  custom_pathname = malloc(sizeof(struct uim_custom_pathname));
+  if (!custom_pathname)
+    return NULL;
+
+  custom_pathname->str = str;
+  custom_pathname->type = type;
+
+  return custom_pathname;
+}
+
+static void
+uim_custom_pathname_free(struct uim_custom_pathname *custom_pathname)
+{
+  if (!custom_pathname)
+    return;
+
+  free(custom_pathname->str);
 }
 
 /* choice */
@@ -294,12 +551,12 @@ uim_custom_choice_get(const char *custom_sym, const char *choice_sym)
   UIM_EVAL_FSTRING2(NULL, "(custom-choice-label '%s '%s)",
 		    custom_sym, choice_sym);
   return_val = uim_scm_return_value();
-  c_choice->label = uim_scm_c_str(return_val);
+  c_choice->label = strdup(UGETTEXT(uim_scm_refer_c_str(return_val)));
 
   UIM_EVAL_FSTRING2(NULL, "(custom-choice-desc '%s '%s)",
 		    custom_sym, choice_sym);
   return_val = uim_scm_return_value();
-  c_choice->desc = uim_scm_c_str(return_val);
+  c_choice->desc = strdup(UGETTEXT(uim_scm_refer_c_str(return_val)));
 
   return c_choice;
 }
@@ -568,7 +825,7 @@ uim_custom_value_internal(const char *custom_sym, const char *getter_proc)
     value->as_str = uim_scm_c_str(return_val);
     break;
   case UCustom_Pathname:
-    value->as_pathname = uim_scm_c_str(return_val);
+    value->as_pathname = uim_custom_pathname_get(custom_sym, getter_proc);
     break;
   case UCustom_Choice:
     custom_value_symbol = uim_scm_c_symbol(return_val);
@@ -611,7 +868,7 @@ uim_custom_value_free(int custom_type, union uim_custom_value *custom_value)
     free(custom_value->as_str);
     break;
   case UCustom_Pathname:
-    free(custom_value->as_pathname);
+    uim_custom_pathname_free(custom_value->as_pathname);
     break;
   case UCustom_Choice:
     uim_custom_choice_free(custom_value->as_choice);
@@ -727,19 +984,27 @@ uim_custom_enable(void)
 uim_bool
 uim_custom_init(void)
 {
+  const char *client_codeset;
+
   return_val = uim_scm_f();
-
+  uim_scm_last_val = uim_scm_f();
   uim_scm_gc_protect(&return_val);
+  uim_scm_gc_protect(&uim_scm_last_val);
 
-  uim_scm_init_subr_3("custom-update-cb-gate", uim_custom_cb_update_cb_gate);
-  uim_scm_init_subr_2("custom-global-update-cb-gate",
-		      uim_custom_global_cb_update_cb_gate);
+  uim_scm_init_proc3("custom-update-cb-gate", uim_custom_cb_update_cb_gate);
+  uim_scm_init_proc2("custom-global-update-cb-gate",
+		     uim_custom_global_cb_update_cb_gate);
 
   uim_scm_require_file("custom.scm");
 
   /* temporary solution to control key definition expantion */
   UIM_EVAL_STRING(NULL, "(define uim-custom-expand-key? #t)");
 
+  /* Assumes that bind_textdomain_codeset() is already called in client
+   * program. */
+  client_codeset = bind_textdomain_codeset(textdomain(NULL), NULL);
+  bind_textdomain_codeset(GETTEXT_PACKAGE, client_codeset);
+
   return UIM_TRUE;
 }
 
@@ -767,7 +1032,7 @@ uim_conf_path(const char *subpath)
 {
   char *dir;
 
-  UIM_EVAL_STRING(NULL, "(string-append (getenv \"HOME\") \"/.uim.d\")");
+  UIM_EVAL_STRING(NULL, "(string-append (or (home-directory (user-name)) \"\") \"/.uim.d\")");
   dir = uim_scm_c_str(uim_scm_return_value());
   if (subpath) {
     UIM_EVAL_FSTRING2(NULL, "\"%s/%s\"", dir, subpath);
@@ -785,7 +1050,7 @@ custom_file_path(const char *group, pid_t pid)
 
   custom_dir = uim_conf_path(custom_subdir);
   if (pid) {
-    UIM_EVAL_FSTRING3(NULL, "\"%s/.custom-%s.scm.%d\"", custom_dir, group, pid);
+    UIM_EVAL_FSTRING3(NULL, "\"%s/.custom-%s.scm.%d\"", custom_dir, group, (int)pid);
   } else {
     UIM_EVAL_FSTRING2(NULL, "\"%s/custom-%s.scm\"", custom_dir, group);
   }
@@ -1043,7 +1308,7 @@ uim_custom_broadcast(void)
 	uim_custom_symbol_list_free(custom_syms);
 	return UIM_FALSE;
       }
-      sprintf(msg, custom_msg_tmpl, *sym, value);
+      snprintf(msg, msg_size, custom_msg_tmpl, *sym, value);
       uim_helper_send_message(helper_fd, msg);
       free(msg);
       free(value);
@@ -1155,7 +1420,7 @@ uim_custom_set(const struct uim_custom *custom)
     free(literal);
     break;
   case UCustom_Pathname:
-    literal = literalize_string(custom->value->as_pathname);
+    literal = literalize_string(custom->value->as_pathname->str);
     UIM_EVAL_FSTRING2(NULL, "(custom-set-value! '%s %s)",
 		      custom->symbol, literal);
     free(literal);
@@ -1219,7 +1484,7 @@ uim_custom_free(struct uim_custom *custom)
 char *
 uim_custom_value_as_literal(const char *custom_sym)
 {
-  return uim_custom_get_str(custom_sym, "custom-value-as-literal");
+  return strdup(uim_custom_get_str(custom_sym, "custom-value-as-literal"));
 }
 
 /**
@@ -1232,7 +1497,7 @@ uim_custom_value_as_literal(const char *custom_sym)
 char *
 uim_custom_definition_as_literal(const char *custom_sym)
 {
-  return uim_custom_get_str(custom_sym, "custom-definition-as-literal");
+  return strdup(uim_custom_get_str(custom_sym, "custom-definition-as-literal"));
 }
 
 /**
@@ -1247,14 +1512,18 @@ struct uim_custom_group *
 uim_custom_group_get(const char *group_sym)
 {
   struct uim_custom_group *custom_group;
+  const char *label, *desc;
 
   custom_group = (struct uim_custom_group *)malloc(sizeof(struct uim_custom_group));
   if (!custom_group)
     return NULL;
 
+  label = uim_custom_get_str(group_sym, "custom-group-label");
+  desc = uim_custom_get_str(group_sym, "custom-group-desc");
+
   custom_group->symbol = strdup(group_sym);
-  custom_group->label = uim_custom_get_str(group_sym, "custom-group-label");
-  custom_group->desc = uim_custom_get_str(group_sym, "custom-group-desc");
+  custom_group->label = strdup(UGETTEXT(label));
+  custom_group->desc = strdup(UGETTEXT(desc));
 
   return custom_group;
 }
@@ -1407,31 +1676,36 @@ static uim_bool
 custom_cb_add(const char *hook, const char *validator,
 	      const char *custom_sym, void *ptr,
 	      const char *gate_func, void (*cb)(void))
-#if UIM_SCM_GCC4_READY_GC
 {
-  uim_bool ret;
+  struct custom_cb_add_args args;
 
-  UIM_SCM_GC_PROTECTED_CALL(ret, uim_bool, custom_cb_add_internal,
-			    (hook, validator, custom_sym, ptr, gate_func, cb));
-
-  return ret;
+  args.hook = hook;
+  args.validator = validator;
+  args.custom_sym = custom_sym;
+  args.ptr = ptr;
+  args.gate_func = gate_func;
+  args.cb = cb;
+  return (uim_bool)(uintptr_t)uim_scm_call_with_gc_ready_stack((uim_gc_gate_func_ptr)custom_cb_add_internal, &args);
 }
 
-static uim_bool
-custom_cb_add_internal(const char *hook, const char *validator,
-		       const char *custom_sym, void *ptr,
-		       const char *gate_func, void (*cb)(void))
-#endif
+static void *
+custom_cb_add_internal(struct custom_cb_add_args *args)
 {
   uim_bool succeeded;
-#if !UIM_SCM_GCC4_READY_GC
-  uim_lisp stack_start;
-#endif
+  const char *hook;
+  const char *validator;
+  const char *custom_sym;
+  void *ptr;
+  const char *gate_func;
+  void (*cb)(void);
   uim_lisp form;
 
-#if !UIM_SCM_GCC4_READY_GC
-  uim_scm_gc_protect_stack(&stack_start);
-#endif
+  hook = args->hook;
+  validator = args->validator;
+  custom_sym = args->custom_sym;
+  ptr = args->ptr;
+  gate_func = args->gate_func;
+  cb = args->cb;
 
   form = uim_scm_list5(uim_scm_make_symbol(validator),
 		       uim_scm_quote(uim_scm_make_symbol(custom_sym)),
@@ -1442,11 +1716,7 @@ custom_cb_add_internal(const char *hook, const char *validator,
   form = uim_scm_cons(uim_scm_make_symbol("custom-register-cb"), form);
   succeeded = uim_scm_c_bool(uim_scm_eval(form));
 
-#if !UIM_SCM_GCC4_READY_GC
-  uim_scm_gc_unprotect_stack(&stack_start);
-#endif
-
-  return succeeded;
+  return (void *)(uintptr_t)succeeded;
 }
 
 static uim_bool
