@@ -1,7 +1,7 @@
 /*
   uim-helper-server.c: This file will be renamed uim-helper-messagebus.c later.
 
-  Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -37,6 +37,7 @@
 #include <pwd.h>
 #include <sys/socket.h>
 #include <sys/un.h>
+#include <sys/param.h>
 #include <netinet/in.h>
 #include <netdb.h>
 #include <stdlib.h>
@@ -52,6 +53,7 @@
 #endif
 
 #include "uim.h"
+#include "uim-internal.h"
 #include "uim-helper.h"
 
 
@@ -72,21 +74,14 @@ struct client {
 static fd_set s_fdset_read;
 static fd_set s_fdset_write;
 static int s_max_fd;
-
 static int nr_client_slots;
 static struct client *clients;
-
 static char read_buf[BUFFER_SIZE];
 
-/*
-  initialize server's file descriptor.
-*/
 static int
 init_server_fd(char *path)
 {
-  int foo;
-  int fd;
-  int flag;
+  int fd, flag;
   struct sockaddr_un myhost;
   struct passwd *pw;
   char *logname;
@@ -96,45 +91,43 @@ init_server_fd(char *path)
     perror("failed in socket()");
     return -1;
   }
+  fchmod(fd, S_IRUSR | S_IWUSR);
 
-  bzero(&myhost, sizeof(myhost));
+  memset(&myhost, 0, sizeof(myhost));
   myhost.sun_family = PF_UNIX;
   strlcpy(myhost.sun_path, path, sizeof(myhost.sun_path));
 
-  foo = bind(fd, (struct sockaddr *)&myhost, SUN_LEN(&myhost));
-  if (foo < -1) {
+  if (bind(fd, (struct sockaddr *)&myhost, SUN_LEN(&myhost)) < 0) {
     perror("failed in bind()");
     return -1;
   }
 
-  chmod(path, S_IRUSR|S_IWUSR);
-
   logname = getenv("LOGNAME");
   if (logname) {
     pw = getpwnam(logname);
     if (pw)
-      chown(path, pw->pw_uid, -1);
+      fchown(fd, pw->pw_uid, -1);
   }
 
-  if ((flag = fcntl(fd, F_GETFL)) == -1) {
+  if ((flag = fcntl(fd, F_GETFL)) < 0) {
     close(fd);
     return -1;
   }
 
   flag |= O_NONBLOCK;
-  if (fcntl(fd, F_SETFL, flag) == -1) {
+  if (fcntl(fd, F_SETFL, flag) < 0) {
     close(fd);
     return -1;
   }
 
-  foo = listen(fd, 5);
-  if (foo == -1) {
+  if (listen(fd, 5) < 0) {
     perror("failed in listen()");
     return -1;
   }
-  /*  fprintf(stderr,"listen at %s\n",path);*/
+
   FD_SET(fd, &s_fdset_read);
   s_max_fd = fd;
+
   return fd;
 }
 
@@ -142,15 +135,17 @@ static struct client *
 get_unused_client(void)
 {
   int i;
+
   for (i = 0; i < nr_client_slots; i++) {
-    if (clients[i].fd == -1) {
+    if (clients[i].fd == -1)
       return &clients[i];
-    }
   }
+
   nr_client_slots++;
-  clients = realloc(clients, sizeof(struct client) * nr_client_slots);
-  clients[nr_client_slots - 1].rbuf = strdup("");
-  clients[nr_client_slots - 1].wbuf = strdup("");
+  clients = uim_realloc(clients, sizeof(struct client) * nr_client_slots);
+  clients[nr_client_slots - 1].rbuf = uim_strdup("");
+  clients[nr_client_slots - 1].wbuf = uim_strdup("");
+
   return &clients[nr_client_slots - 1];
 }
 
@@ -160,11 +155,11 @@ close_client(struct client *cl)
   close(cl->fd);
   if (cl->rbuf) {
     free(cl->rbuf);
-    cl->rbuf = strdup("");
+    cl->rbuf = uim_strdup("");
   }
   if (cl->wbuf) {
     free(cl->wbuf);
-    cl->wbuf = strdup("");
+    cl->wbuf = uim_strdup("");
   }
   cl->fd = -1;
 }
@@ -188,16 +183,17 @@ distribute_message(char *msg, struct client *cl)
 static int
 reflect_message_fragment(struct client *cl)
 {
-  int rc;
+  ssize_t rc;
   char *msg;
 
   /* do read */
   rc = read(cl->fd, read_buf, sizeof(read_buf));
-  if (rc <= 0) {
-    if (rc < 0 && (errno == EAGAIN || errno == EINTR))
+  if (rc == -1) {
+    if (errno == EAGAIN || errno == EINTR)
       return 0;
     return -1;
-  }
+  } else if (rc == 0)
+    return -1;
 
   cl->rbuf = uim_helper_buffer_append(cl->rbuf, read_buf, rc);
 
@@ -205,6 +201,7 @@ reflect_message_fragment(struct client *cl)
     distribute_message(msg, cl);
     free(msg);
   }
+
   return 1;
 }
 
@@ -213,11 +210,12 @@ check_session_alive(void)
 {
   /* If there's no connection, we can assume user logged out. */
   int i;
+
   for (i = 0; i < nr_client_slots; i++) {
-    if (clients[i].fd != -1) {
+    if (clients[i].fd != -1)
       return UIM_TRUE;
-    }
   }
+
   return UIM_FALSE; /* User already logged out */
 }
 
@@ -226,10 +224,11 @@ static uim_bool
 accept_new_connection(int server_fd)
 {
   struct sockaddr_un clientsoc;
-  socklen_t len = sizeof(clientsoc);
-  int new_fd;
-  int flag;
+  socklen_t len;
+  int new_fd, flag;
   struct client *cl;
+
+  len = sizeof(clientsoc);
   new_fd = accept(server_fd, (struct sockaddr *)&clientsoc, &len);
   
   if (new_fd < 0) {
@@ -237,13 +236,13 @@ accept_new_connection(int server_fd)
     return UIM_FALSE;
   }
 
-  if ((flag = fcntl(new_fd, F_GETFL)) == -1) {
+  if ((flag = fcntl(new_fd, F_GETFL)) < 0) {
     close(new_fd);
     return UIM_FALSE;
   }
 
   flag |= O_NONBLOCK;
-  if (fcntl(new_fd, F_SETFL, flag) == -1) {
+  if (fcntl(new_fd, F_SETFL, flag) < 0) {
     close(new_fd);
     return UIM_FALSE;
   }
@@ -300,7 +299,7 @@ write_message(struct client *cl)
   }
   if (out_len == 0) {
     free(cl->wbuf);
-    cl->wbuf = strdup("");
+    cl->wbuf = uim_strdup("");
     FD_CLR(cl->fd, &s_fdset_write);    
   } else {
     uim_helper_buffer_shift(cl->wbuf, message_len - out_len);
@@ -312,6 +311,7 @@ static void
 read_message(struct client *cl)
 {
   int result;
+
   result = reflect_message_fragment(cl);
   
   if (result < 0) {
@@ -323,7 +323,6 @@ read_message(struct client *cl)
   }
 }
 
-
 static void
 uim_helper_server_process_connection(int server_fd)
 {
@@ -331,7 +330,6 @@ uim_helper_server_process_connection(int server_fd)
   fd_set readfds, writefds;
 
   while (1) {
-    /* Could we replace this memcpy with direct assignment? */
     /* Copy readfds from s_fdset_read/s_fdset_write because select removes
        readble/writable fd from readfds/writefds */
     memcpy(&readfds, &s_fdset_read, sizeof(fd_set));
@@ -348,24 +346,23 @@ uim_helper_server_process_connection(int server_fd)
     if (FD_ISSET(server_fd, &readfds)) {
       uim_bool accepted;
       accepted = accept_new_connection(server_fd);
-      if(accepted == UIM_FALSE) {
-	continue; /* acception failed, go next loop without message processing. */
+      if (accepted == UIM_FALSE) {
+	/* acception failed, go next loop without message processing. */
+	continue;
       }
     } else {
       /* check data to write and from clients reached */
       for (i = 0; i < nr_client_slots; i++) {
-	if (clients[i].fd != -1 && FD_ISSET(clients[i].fd, &writefds)) {
+	if (clients[i].fd != -1 && FD_ISSET(clients[i].fd, &writefds))
 	  write_message(&clients[i]);
-	}
-	if (clients[i].fd != -1 && FD_ISSET(clients[i].fd, &readfds)) {
+
+	if (clients[i].fd != -1 && FD_ISSET(clients[i].fd, &readfds))
 	  read_message(&clients[i]);
-	}
       }
     }
 
-    if(check_session_alive() == UIM_FALSE) {
+    if (!check_session_alive())
       return;
-    }
   }
 }
 
@@ -373,8 +370,14 @@ uim_helper_server_process_connection(int server_fd)
 int
 main(int argc, char **argv)
 {
-  char *path = uim_helper_get_pathname();
+  char path[MAXPATHLEN];
   int server_fd;
+
+  uim_init_error();
+
+  if (!uim_helper_get_pathname(path, sizeof(path)))
+    return 0;
+
   unlink(path);
 
   clients = NULL;
@@ -391,18 +394,13 @@ main(int argc, char **argv)
   fclose(stdin);
   fclose(stdout);
 
-  if (server_fd < 0) {
+  if (server_fd < 0)
     return 0;
-  }
+
   /*  fprintf(stderr,"Waiting for connection at %s\n", path);*/
 
-  free(path);
-
   signal(SIGPIPE, SIG_IGN);
-
   uim_helper_server_process_connection(server_fd);
 
   return 0;
 }
-
-
