@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2006 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -37,6 +37,7 @@
 #include <sys/socket.h>
 #include <sys/types.h>
 #include <sys/time.h>
+#include <sys/param.h>
 #include <sys/un.h>
 #include <netinet/in.h>
 #include <netdb.h>
@@ -46,9 +47,24 @@
 #include <pwd.h>
 #include <signal.h>
 #include <errno.h>
+#include <sys/stat.h>
+
 #include "uim-internal.h"
 #include "uim-helper.h"
-#include "uim-util.h"
+#include "uim-posix.h"
+
+/*
+ * uim-notify is disabled since I'm not confident about:
+ *   1. its stability when low-level error handling is being involved
+ *   2. whether these errors should be notified to endusers
+ * -- 2008-01-15 YamaKen
+ */
+#undef USE_UIM_NOTIFY
+#define USE_UIM_NOTIFY 0
+
+#if USE_UIM_NOTIFY && !UIM_NON_LIBUIM_PROG
+#include "uim-notify.h"
+#endif
 
 #ifndef HAVE_SIG_T
 typedef void (*sig_t)(int);
@@ -93,11 +109,22 @@ uim_helper_send_message(int fd, const char *message)
   sig_t old_sigpipe;
   char *buf, *bufp;
 
-  if (fd < 0 || !message)
+  if (UIM_CATCH_ERROR_BEGIN())
     return;
 
+#if 0
+  if (fd < 0)
+    uim_fatal_error("uim_helper_send_message(): invaid fd");
+  if (!message)
+    uim_fatal_error("uim_helper_send_message(): NULL message");
+#else
+  /* The condition fd < 0 ordinarily occurs. */
+  if (fd < 0 || !message)
+    return;
+#endif
+
   len = strlen(message) + 1;
-  buf = malloc(len + 1);
+  buf = uim_malloc(len + 1);
   snprintf(buf, len + 1, "%s\n", message);
 
   old_sigpipe = signal(SIGPIPE, SIG_IGN);
@@ -108,7 +135,7 @@ uim_helper_send_message(int fd, const char *message)
     if ((res = write(fd, bufp, out_len)) < 0) {
       if (errno == EAGAIN || errno == EINTR)
 	continue;
-      fprintf(stderr, "uim_helper_send_message(): unknown error\n");
+      perror("uim_helper_send_message(): unhandled error");
       break;
     }
 
@@ -117,31 +144,100 @@ uim_helper_send_message(int fd, const char *message)
   }
   free(buf);
   signal(SIGPIPE, old_sigpipe);
+
+  UIM_CATCH_ERROR_END();
+
   return;
 }
 
-char *
-uim_helper_get_pathname(void)
+static uim_bool
+check_dir(const char *dir)
 {
-  char *path;
-  char *login = NULL;
-  struct passwd *pw = NULL;
- 
-  if (!uim_issetugid()) {
-    login = getenv("LOGNAME");
+  struct stat st;
+
+  if (stat(dir, &st) < 0)
+    return (mkdir(dir, 0700) < 0) ? UIM_FALSE : UIM_TRUE;
+  else {
+    mode_t mode = S_IFDIR | S_IRUSR | S_IWUSR | S_IXUSR;
+    return ((st.st_mode & mode) == mode) ? UIM_TRUE : UIM_FALSE;
+  }
+}
+
+uim_bool
+uim_helper_get_pathname(char *helper_path, int len)
+{
+  struct passwd *pw;
+
+  if (len <= 0)
+    return UIM_FALSE;
+
+  if (UIM_CATCH_ERROR_BEGIN())
+    return UIM_FALSE;
+
+  pw = getpwuid(getuid());
+  if (!pw) {
+    endpwent();
+    goto path_error;
   }
 
-  if (!login) {
-    pw = getpwuid(getuid());
-    login = strdup(pw->pw_name);
+  if (strlcpy(helper_path, pw->pw_dir, len) >= (size_t)len) {
+    endpwent();
+    goto path_error;
   }
+  if (strlcat(helper_path, "/.uim.d", len) >= (size_t)len) {
+    endpwent();
+    goto path_error;
+  }
+  endpwent();
 
-  path = (char *)malloc(strlen(login)+ 20);
-  sprintf(path, "/tmp/uimhelper-%s",login);
-  if (pw) {
-    free(login);
+  /* check ~/.uim.d/ */
+  if (!check_dir(helper_path))
+    goto path_error;
+
+  /* check ~/.uim.d/socket/ */
+  if (strlcat(helper_path, "/socket", len) >= (size_t)len)
+    goto path_error;
+
+  if (!check_dir(helper_path))
+    goto path_error;
+
+  if (strlcat(helper_path, "/uim-helper", len) >= (size_t)len)
+    goto path_error;
+
+  UIM_CATCH_ERROR_END();
+
+  return UIM_TRUE;
+
+ path_error:
+  uim_fatal_error("uim_helper_get_pathname()");
+  helper_path[0] = '\0';
+
+  UIM_CATCH_ERROR_END();
+  return UIM_FALSE;
+}
+
+int
+uim_helper_check_connection_fd(int fd)
+{
+  uid_t euid;
+  gid_t egid;
+  if (getpeereid(fd, &euid, &egid) < 0) {
+#if USE_UIM_NOTIFY && !UIM_NON_LIBUIM_PROG
+    uim_notify_fatal("uim_helper: %s", strerror(errno));
+#else
+    perror("getpeereid failed");
+#endif
+    return -1;
   }
-  return path;
+  if ((euid != 0) && (euid != getuid())) {
+#if USE_UIM_NOTIFY && !UIM_NON_LIBUIM_PROG
+    uim_notify_fatal("uim_helper: uid mismatch");
+#else
+    fprintf(stderr, "uid mismatch\n");
+#endif
+    return -1;
+  }
+  return 0;
 }
 
 int uim_helper_fd_readable(int fd)
@@ -175,7 +271,7 @@ uim_helper_buffer_append(char *buf, const char *fragment, size_t fragment_size)
 
   buf_size = strlen(buf);
   extended_size = buf_size + fragment_size + 1;
-  buf = (char *)realloc(buf, extended_size);
+  buf = realloc(buf, extended_size);
   if (buf) {
     memcpy(&buf[buf_size], fragment, fragment_size);
     buf[extended_size - 1] = '\0';
@@ -198,16 +294,23 @@ uim_helper_buffer_get_message(char *buf)
   size_t msg_size;
   char *msg, *msg_term;
 
+  if (UIM_CATCH_ERROR_BEGIN())
+    return NULL;
+
   msg_term = strstr(buf, "\n\n");
   if (msg_term) {
     msg_size = msg_term + 2 - buf;
-    msg = (char *)malloc(msg_size + 1);
+    msg = uim_malloc(msg_size + 1);
     memcpy(msg, buf, msg_size);
     msg[msg_size] = '\0';
     uim_helper_buffer_shift(buf, msg_size);
-    return msg;
+  } else {
+    msg = NULL;
   }
-  return NULL;
+
+  UIM_CATCH_ERROR_END();
+
+  return msg;
 }
 
 /* Public API for uim_issetugid(). */
@@ -218,6 +321,7 @@ uim_helper_is_setugid(void)
   return (uim_issetugid()) ? UIM_TRUE : UIM_FALSE;
 }
 
+#if !HAVE_ISSETUGID
 /* For internal use only. libuim clients should use uim_helper_is_setugid()
  * since this is not a core uim function. */
 uim_bool
@@ -230,3 +334,4 @@ uim_issetugid(void)
 
   return (ruid != euid || rgid != egid);
 }
+#endif /* !HAVE_ISSETUGID */
