@@ -1,7 +1,7 @@
 /*
   uim-module-manager.c: source file for uim-module-manager.
 
-  Copyright (c) 2005-2007 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2005-2008 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -34,6 +34,7 @@
 
 #include <config.h>
 
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
@@ -42,6 +43,7 @@
 
 #include "uim.h"
 #include "uim-scm.h"
+#include "uim-scm-abbrev.h"
 #include "uim-internal.h"
 
 #define MODULE_LIST_FILENAME UIM_DATADIR"/modules"
@@ -64,6 +66,12 @@ char *action_command[] = {
   NULL
 };
 
+struct main_args {
+  int argc;
+  char **argv;
+};
+static void *main_internal(struct main_args *args);
+
 /* Utility function */
 static char *
 concat(const char *a, const char *b)
@@ -79,18 +87,10 @@ concat(const char *a, const char *b)
   return dest;
 }
 
-static char *
-append_module_names(char *modules, const char *new_module)
+static uim_lisp
+append_module_names(uim_lisp modules, const char *new_module)
 {
-  if (!modules)
-    return strdup(new_module);
-
-  modules = realloc(modules, strlen(modules) + strlen(new_module) + 2);
-  if (modules) {
-    strcat(modules, " ");
-    strcat(modules, new_module);
-  }
-  return modules;
+  return uim_scm_callf("append", "oo", modules, LIST1(MAKE_STR(new_module)));
 }
 
 static void
@@ -163,10 +163,9 @@ write_installed_modules_scm(uim_lisp str)
 }
 
 int
-main(int argc, char *argv[]) {
-  int i;
-  int action = None;
-  char *module_names = NULL;
+main(int argc, char *argv[])
+{
+  struct main_args args;
 
   if (argc <= 1) {
     print_usage();
@@ -179,8 +178,27 @@ main(int argc, char *argv[]) {
   setenv("LIBUIM_VANILLA", "1", 1);
   
   uim_init();
-  uim_scm_set_verbose_level(1);
 
+  args.argc = argc;
+  args.argv = argv;
+  uim_scm_call_with_gc_ready_stack((uim_gc_gate_func_ptr)main_internal, &args);
+
+  uim_quit();
+
+  return EXIT_SUCCESS;
+}
+
+static void *
+main_internal(struct main_args *args)
+{
+  int i, argc;
+  int action = None;
+  char **argv;
+  uim_lisp module_names;
+
+  argc = args->argc;
+  argv = args->argv;
+  module_names = uim_scm_null();
   for (i = 0; i < argc; i++) {
     if (strcmp(argv[i], "--register") == 0) {
       if (action != None) {
@@ -222,13 +240,11 @@ main(int argc, char *argv[]) {
     exit(EXIT_FAILURE);
   }
 
-  uim_scm_init_subr_1("write-loader.scm", write_loader_scm);
-  uim_scm_init_subr_1("write-installed-modules.scm", write_installed_modules_scm);
+  uim_scm_init_proc1("write-loader.scm", write_loader_scm);
+  uim_scm_init_proc1("write-installed-modules.scm", write_installed_modules_scm);
 
-  if (!uim_scm_require_file("uim-module-manager.scm")) {
-    perror("failed to require uim-module-manager.scm");
-    exit(EXIT_FAILURE);
-  }
+  if (!uim_scm_require_file("uim-module-manager.scm"))
+    uim_fatal_error("failed to require uim-module-manager.scm");
 
   if (path) {
     char *extra_file = concat(path, "/installed-modules.scm");
@@ -238,15 +254,5 @@ main(int argc, char *argv[]) {
     free(extra_file);
   }
 
-  /* for unregister-all */
-  if (!module_names)
-    module_names = "";
-
-  UIM_EVAL_FSTRING2(NULL, "(%s \"%s\")",
-		    action_command[action],
-		    module_names);
-
-  uim_quit();
-
-  return EXIT_SUCCESS;
+  return uim_scm_callf(action_command[action], "o", module_names);
 }
