@@ -2,7 +2,7 @@
   uim-sh.c: uim interactive shell for debugging, batch processing and
             serving as generic inferior process
 
-  Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -35,67 +35,30 @@
 
 #include "uim.h"
 #include "uim-scm.h"
-#include "uim-compat-scm.h"
 
-#ifdef LIBEDIT
-#include "editline.h"
-#endif
-
-#ifdef UIM_SH_USE_EXIT_HOOK
-extern int uim_siod_fatal;
-#endif
 
 int
-main(int argc, char *argv[]) {
-  long verbose;
-  uim_lisp args, form;
+main(int argc, char *argv[])
+{
+  uim_lisp args, exit_status_;
+  int exit_status;
 
-  uim_scm_set_output(stdout);
-  
   /* TODO: be able to suppress ordinary initialization process */
   uim_init();
 
-#ifdef LIBEDIT
-  editline_init();
-#endif
-
-  verbose = uim_scm_get_verbose_level();
-  uim_scm_set_verbose_level(1);
   uim_scm_require_file("uim-sh.scm");
-#ifdef UIM_SH_USE_EXIT_HOOK
-  /*
-     is not working even if uim_siod_fatal is accessible. outermost
-     *catch affects me?
-  */
-  if (uim_siod_fatal)
-    return 1;
-#endif
 
-  /*
-    verbose level must be greater than or equal to 1 to print anything
-  */
-  if (verbose < 1)
-    verbose = 1;
-  uim_scm_set_verbose_level(verbose);
+  args = uim_scm_null();
+  exit_status_ = uim_scm_f();
+  uim_scm_gc_protect(&args);
+  uim_scm_gc_protect(&exit_status_);
 
-  args = uim_scm_c_strs_into_list(argc, (const char *const *)argv);
-  form = uim_scm_list2(uim_scm_intern_c_str("uim-sh"),
-		       uim_scm_quote(args));
-  uim_scm_eval(form);
-
-#ifdef UIM_SH_USE_EXIT_HOOK
-  /* is not working even if uim_siod_fatal is accessible. outermost
-   * *catch affects me?
-   */
-  if (uim_siod_fatal)
-    return 1;
-#endif
-
-#ifdef LIBEDIT
-  editline_quit();
-#endif
+  args = uim_scm_array2list((void **)argv, argc,
+			    (uim_lisp (*)(void *))uim_scm_make_str);
+  exit_status_ = uim_scm_callf("uim-sh", "o", args);
+  exit_status  = uim_scm_c_int(exit_status_);
 
   uim_quit();
 
-  return 0;
+  return exit_status;
 }
