@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -31,8 +31,8 @@
 */
 
 // -*- C++ -*-
-#ifndef _canddisp_h_included_
-#define _canddisp_h_included_
+#ifndef UIM_XIM_CANDDISP_H
+#define UIM_XIM_CANDDISP_H
 
 #include <vector>
 
