@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -166,12 +166,13 @@ putbackch(int c, int *lastch)
 #endif
 
 static int
-nexttoken(FILE *fp, char *tokenbuf, int *lastch)
+nexttoken(FILE *fp, char **tokenbuf, int *lastch, size_t *buflen)
 {
     int c;
     int token;
     char *p;
     int i, j;
+    size_t len = 0;
 
     while ((c = nextch(fp, lastch)) == ' ' || c == '\t') {
     }
@@ -198,8 +199,13 @@ nexttoken(FILE *fp, char *tokenbuf, int *lastch)
 	token = TILDE;
 	break;
     case '"':
-	p = tokenbuf;
+	p = *tokenbuf;
 	while ((c = nextch(fp, lastch)) != '"') {
+	    if (len >= *buflen - 1) {
+		    *buflen += BUFSIZ;
+		    *tokenbuf = (char *)realloc(*tokenbuf, *buflen);
+		    p = *tokenbuf + len;
+	    }
 	    if (c == '\n' || c == EOF) {
 		putbackch(c, lastch);
 		token = ERROR;
@@ -209,16 +215,20 @@ nexttoken(FILE *fp, char *tokenbuf, int *lastch)
 		switch (c) {
 		case '\\':
 		case '"':
-		    *p++ = c;
+		    *p++ = (char)c;
+		    len++;
 		    break;
 		case 'n':
 		    *p++ = '\n';
+		    len++;
 		    break;
 		case 'r':
 		    *p++ = '\r';
+		    len++;
 		    break;
 		case 't':
 		    *p++ = '\t';
+		    len++;
 		    break;
 		case '0':
 		case '1':
@@ -237,6 +247,7 @@ nexttoken(FILE *fp, char *tokenbuf, int *lastch)
 		    }
 		    putbackch(c, lastch);
 		    *p++ = (char)i;
+		    len++;
 		    break;
 		case 'X':
 		case 'x':
@@ -261,17 +272,20 @@ nexttoken(FILE *fp, char *tokenbuf, int *lastch)
 			goto string_error;
 		    }
 		    *p++ = (char)i;
+		    len++;
 		    break;
 		case EOF:
 		    putbackch(c, lastch);
 		    token = ERROR;
 		    goto string_error;
 		default:
-		    *p++ = c;
+		    *p++ = (char)c;
+		    len++;
 		    break;
 		}
 	    } else {
-		*p++ = c;
+		*p++ = (char)c;
+		len++;
 	    }
 	}
 	*p = '\0';
@@ -288,11 +302,22 @@ nexttoken(FILE *fp, char *tokenbuf, int *lastch)
 	break;
     default:
 	if (isalnum(c) || c == '_' || c == '-') {
-	    p = tokenbuf;
-	    *p++ = c;
+	    if (len >= *buflen - 1) {
+		*buflen += BUFSIZ;
+		*tokenbuf = (char *)realloc(*tokenbuf,  *buflen);
+	    }
+	    p = *tokenbuf;
+	    *p++ = (char)c;
+	    len++;
 	    c = nextch(fp, lastch);
 	    while (isalnum(c) || c == '_' || c == '-') {
-		*p++ = c;
+	        if (len >= *buflen - 1) {
+			*buflen += BUFSIZ;
+			*tokenbuf = (char *)realloc(*tokenbuf,  *buflen);
+			p = *tokenbuf + len;
+		}
+		*p++ = (char)c;
+		len++;
 		c = nextch(fp, lastch);
 	    }
 	    *p = '\0';
@@ -433,7 +458,7 @@ XimIM::get_mb_string(char *buf, KeySym ks)
 #define SEQUENCE_MAX    10
 
 int
-XimIM::parse_compose_line(FILE *fp, char* tokenbuf)
+XimIM::parse_compose_line(FILE *fp, char **tokenbuf, size_t *buflen)
 {
     int token;
     unsigned modifier_mask;
@@ -461,7 +486,7 @@ XimIM::parse_compose_line(FILE *fp, char* tokenbuf)
     const char *encoding = get_encoding();;
 
     do {
-	token = nexttoken(fp, tokenbuf, &lastch);
+	token = nexttoken(fp, tokenbuf, &lastch, buflen);
     } while (token == ENDOFLINE);
 
     if (token == ENDOFFILE) {
@@ -470,41 +495,42 @@ XimIM::parse_compose_line(FILE *fp, char* tokenbuf)
 
     n = 0;
     do {
-	if ((token == KEY) && (strcmp("include", tokenbuf) == 0)) {
+	if ((token == KEY) && (strcmp("include", *tokenbuf) == 0)) {
 	    char *filename;
 	    FILE *infp;
-	    token = nexttoken(fp, tokenbuf, &lastch);
+	    token = nexttoken(fp, tokenbuf, &lastch, buflen);
 	    if (token != KEY && token != STRING)
 		goto error;
 
-	    if ((filename = TransFileName(tokenbuf)) == NULL)
+	    if ((filename = TransFileName(*tokenbuf)) == NULL)
 		goto error;
 	    infp = fopen(filename, "r");
 	    free(filename);
 	    if (infp == NULL)
 		goto error;
 	    ParseComposeStringFile(infp);
-	    return (0);
-	} else if ((token == KEY) && (strcmp("None", tokenbuf) == 0)) {
+	    fclose(infp);
+	    return 0;
+	} else if ((token == KEY) && (strcmp("None", *tokenbuf) == 0)) {
 	    modifier = 0;
 	    modifier_mask = AllMask;
-	    token = nexttoken(fp, tokenbuf, &lastch);
+	    token = nexttoken(fp, tokenbuf, &lastch, buflen);
 	} else {
 	    modifier_mask = modifier = 0;
 	    exclam = False;
 	    if (token == EXCLAM) {
 		exclam = True;
-		token = nexttoken(fp, tokenbuf, &lastch);
+		token = nexttoken(fp, tokenbuf, &lastch, buflen);
 	    }
 	    while (token == TILDE || token == KEY) {
 		tilde = False;
 		if (token == TILDE) {
 		    tilde = True;
-		    token = nexttoken(fp, tokenbuf, &lastch);
+		    token = nexttoken(fp, tokenbuf, &lastch, buflen);
 		    if (token != KEY)
 			goto error;
 		}
-		tmp = modmask(tokenbuf);
+		tmp = modmask(*tokenbuf);
 		if (!tmp) {
 		    goto error;
 		}
@@ -514,7 +540,7 @@ XimIM::parse_compose_line(FILE *fp, char* tokenbuf)
 		} else {
 		    modifier |= tmp;
 		}
-		token = nexttoken(fp, tokenbuf, &lastch);
+		token = nexttoken(fp, tokenbuf, &lastch, buflen);
 	    }
 	    if (exclam) {
 		modifier_mask = AllMask;
@@ -526,17 +552,17 @@ XimIM::parse_compose_line(FILE *fp, char* tokenbuf)
 	    goto error;
 	}
 
-	token = nexttoken(fp, tokenbuf, &lastch);
+	token = nexttoken(fp, tokenbuf, &lastch, buflen);
 	if (token != KEY) {
 	    goto error;
 	}
 
-	token = nexttoken(fp, tokenbuf, &lastch);
+	token = nexttoken(fp, tokenbuf, &lastch, buflen);
 	if (token != GREATER) {
 	    goto error;
 	}
 
-	keysym = XStringToKeysym(tokenbuf);
+	keysym = XStringToKeysym(*tokenbuf);
 	if (keysym == NoSymbol) {
 	    goto error;
 	}
@@ -547,33 +573,33 @@ XimIM::parse_compose_line(FILE *fp, char* tokenbuf)
 	n++;
 	if (n >= SEQUENCE_MAX)
 	    goto error;
-	token = nexttoken(fp, tokenbuf, &lastch);
+	token = nexttoken(fp, tokenbuf, &lastch, buflen);
     } while (token != COLON);
 
-    token = nexttoken(fp, tokenbuf, &lastch);
+    token = nexttoken(fp, tokenbuf, &lastch, buflen);
     if (token == STRING) {
-	if ((rhs_string_mb = (char *)malloc(strlen(tokenbuf) + 1)) == NULL)
+	if ((rhs_string_mb = (char *)malloc(strlen(*tokenbuf) + 1)) == NULL)
 	    goto error;
-	strcpy(rhs_string_mb, tokenbuf);
-	token = nexttoken(fp, tokenbuf, &lastch);
+	strcpy(rhs_string_mb, *tokenbuf);
+	token = nexttoken(fp, tokenbuf, &lastch, buflen);
 	if (token == KEY) {
-	    rhs_keysym = XStringToKeysym(tokenbuf);
+	    rhs_keysym = XStringToKeysym(*tokenbuf);
 	    if (rhs_keysym == NoSymbol) {
 		free(rhs_string_mb);
 		goto error;
 	    }
-	    token = nexttoken(fp, tokenbuf, &lastch);
+	    token = nexttoken(fp, tokenbuf, &lastch, buflen);
 	}
 	if (token != ENDOFLINE && token != ENDOFFILE) {
 	    free(rhs_string_mb);
 	    goto error;
 	}
     } else if (token == KEY) {
-	rhs_keysym = XStringToKeysym(tokenbuf);
+	rhs_keysym = XStringToKeysym(*tokenbuf);
 	if (rhs_keysym == NoSymbol) {
 	    goto error;
 	}
-	token = nexttoken(fp, tokenbuf, &lastch);
+	token = nexttoken(fp, tokenbuf, &lastch, buflen);
 	if (token != ENDOFLINE && token != ENDOFFILE) {
 	    goto error;
 	}
@@ -641,7 +667,7 @@ XimIM::parse_compose_line(FILE *fp, char* tokenbuf)
     return n;
 error:
     while (token != ENDOFLINE && token != ENDOFFILE) {
-	token = nexttoken(fp, tokenbuf, &lastch);
+	token = nexttoken(fp, tokenbuf, &lastch, buflen);
     }
     return 0;
 }
@@ -649,22 +675,19 @@ error:
 void
 XimIM::ParseComposeStringFile(FILE *fp)
 {
-    char tb[8192];
-    char* tbp;
+    char *tbp, *p[1];
     struct stat st;
+    size_t buflen = BUFSIZ;
 
-    if (fstat(fileno(fp), &st) != -1) {
-	unsigned long size = (unsigned long)st.st_size;
-	if (size <= sizeof tb)
-	    tbp = tb;
-	else
-	    tbp = (char *)malloc(size);
+    if (fstat(fileno(fp), &st) != -1 && S_ISREG(st.st_mode) &&
+	st.st_size > 0) {
 
+	tbp = (char *)malloc(buflen);
+	p[0] = tbp;
 	if (tbp != NULL) {
-	    while (parse_compose_line(fp, tbp) >= 0) {
+	    while (parse_compose_line(fp, p, &buflen) >= 0) {
 	    }
-	    if (tbp != tb)
-		free (tbp);
+	    free(p[0]);
 	}
     }
 }
@@ -785,7 +808,7 @@ char *XimIM::get_compose_filename()
 	char *p = buf;
 	int n;
 	char *args[2], *from, *to;
-	while (isspace(*p)) {
+	while (isspace((unsigned char)*p)) {
 	    ++p;
 	}
 	if (iscomment(*p)) {
@@ -828,7 +851,7 @@ parse_line(char *line, char **argv, int argsize)
     char *p = line;
 
     while (argc < argsize) {
-	while (isspace(*p)) {
+	while (isspace((unsigned char)*p)) {
 	    ++p;
 	}
 	if (*p == '\0') {
@@ -1174,8 +1197,7 @@ mb_string_to_utf8(char *utf8, const char *str, int len, const char *enc) {
     size_t ret_val;
     iconv_t cd;
 
-    cd = (iconv_t)uim_iconv_open("UTF-8", enc);
-    if (cd == (iconv_t)-1) {
+    if (!uim_iconv->is_convertible("UTF-8", enc)) {
 	perror("error in iconv_open");
 	utf8[0] = '\0';
 	return 0;
@@ -1183,35 +1205,51 @@ mb_string_to_utf8(char *utf8, const char *str, int len, const char *enc) {
 
     inbuf = str;
     if (!inbuf) {
-	iconv_close(cd);
 	utf8[0] = '\0';
 	return 0;
     }
+
+    cd = (iconv_t)uim_iconv->create("UTF-8", enc);
+
+    // no conversion needed
+    if (!cd)
+	return strlcpy(utf8, inbuf, len + 1);
+
     outbuf = (char *)malloc(outbufsize);
     if (!outbuf) {
-	iconv_close(cd);
+	uim_iconv->release(cd);
 	utf8[0] = '\0';
 	return 0;
     }
     inchar = (char *)inbuf;
     outchar = outbuf;
     inbytesleft = strlen(inbuf);
-    outbytesleft = outbufsize;
+    outbytesleft = outbufsize - 1;
     ret_val = iconv(cd, (ICONV_CONST char **)&inchar, &inbytesleft, &outchar, &outbytesleft);
 
     if (ret_val == (size_t)-1 && errno != E2BIG) {
 	//perror("error in iconv");
-	iconv_close(cd);
+	uim_iconv->release(cd);
 	free(outbuf);
 	utf8[0] = '\0';
 	return 0;
     }
-    iconv_close(cd);
+    ret_val = iconv(cd, NULL, NULL, &outchar, &outbytesleft);
+    if (ret_val == (size_t)-1 && errno != E2BIG) {
+	//perror("error in iconv");
+	uim_iconv->release(cd);
+	free(outbuf);
+	utf8[0] = '\0';
+	return 0;
+    }
+    uim_iconv->release(cd);
 
-    strlcpy(utf8, outbuf, outbufsize - outbytesleft + 1);
+    *outchar = '\0';
+
+    strlcpy(utf8, outbuf, len + 1);
     free(outbuf);
 
-    return outbufsize - outbytesleft;
+    return outbufsize - outbytesleft - 1;
 }
 
 /*
