@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2006 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -31,8 +31,8 @@
 */
 
 // -*- C++ -*-
-#ifndef _compose_h_included_
-#define _compose_h_included_
+#ifndef UIM_XIM_COMPOSE_H
+#define UIM_XIM_COMPOSE_H
 
 #include <X11/X.h>
 
