@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -54,7 +54,7 @@
 
 #include "uim/uim-util.h"
 #include "uim/uim-im-switcher.h"
-#include "uim/uim-compat-scm.h"
+#include "uim/uim-scm.h"
 
 Display *XimServer::gDpy;
 std::map<Window, XimServer *> XimServer::gServerMap;
@@ -523,42 +523,49 @@ terminate_x_connection()
 }
 
 void
-reload_uim(int /* x */)
+reload_uim(int reload_libuim)
 {
-    fprintf(stderr, "\nReloading uim...\n\n");
+    if (reload_libuim) {
+	fprintf(stderr, "\nReloading uim...\n\n");
 
-    terminate_canddisp_connection();
-    helper_disconnect_cb();
-    terminate_x_connection();
+	terminate_canddisp_connection();
+	helper_disconnect_cb();
+	terminate_x_connection();
 
-    std::map<Window, XimServer *>::iterator it;
-    std::list<InputContext *>::iterator it_c;
+	std::map<Window, XimServer *>::iterator it;
+	std::list<InputContext *>::iterator it_c;
 
-    for (it = XimServer::gServerMap.begin(); it != XimServer::gServerMap.end(); ++it) {
-	XimServer *xs = it->second;
-	for (it_c = xs->ic_list.begin(); it_c != xs->ic_list.end(); ++it_c)
-	    (*it_c)->clear();
+	for (it = XimServer::gServerMap.begin(); it != XimServer::gServerMap.end(); ++it) {
+	    XimServer *xs = it->second;
+	    for (it_c = xs->ic_list.begin(); it_c != xs->ic_list.end(); ++it_c)
+		(*it_c)->clear();
+	}
+	uim_quit();
     }
 
-    uim_quit();
     clear_uim_info();
     get_uim_info();
     //print_uim_info();
 
-    for (it = XimServer::gServerMap.begin(); it != XimServer::gServerMap.end(); ++it) {
-	XimServer *xs = it->second;
-	for (it_c = xs->ic_list.begin(); it_c != xs->ic_list.end(); ++it_c) {
-	    const char *engine = (*it_c)->get_engine_name();
-	    (*it_c)->createUimContext(engine);
+    if (reload_libuim) {
+	std::map<Window, XimServer *>::iterator it;
+	std::list<InputContext *>::iterator it_c;
+
+	for (it = XimServer::gServerMap.begin(); it != XimServer::gServerMap.end(); ++it) {
+	    XimServer *xs = it->second;
+	    for (it_c = xs->ic_list.begin(); it_c != xs->ic_list.end(); ++it_c) {
+		const char *engine = (*it_c)->get_engine_name();
+		(*it_c)->createUimContext(engine);
+	    }
 	}
+
+	// make sure to use appropriate locale for the focused context
+	InputContext *focusedContext = InputContext::focusedContext();
+	if (focusedContext)
+	    focusedContext->focusIn();
+
+	pretrans_setup();
     }
-
-    // make sure to use appropriate locale for the focused context
-    InputContext *focusedContext = InputContext::focusedContext();
-    if (focusedContext)
-	focusedContext->focusIn();
-
-    pretrans_setup();
 }
 
 int
